@@ -27,7 +27,6 @@
 #include <pk-backend.h>
 #include <pk-backend-internal.h>
 #include <egg-debug.h>
-#include <pk-enum.h>
 
 #include <libopkg/opkg.h>
 
@@ -48,7 +47,7 @@ typedef struct {
 } SearchParams;
 
 static void
-opkg_unknown_error (PkBackend *backend, gint error_code, gchar *failed_cmd)
+opkg_unknown_error (PkBackend *backend, gint error_code, const gchar *failed_cmd)
 {
 	gchar *msg;
 
@@ -106,8 +105,8 @@ opkg_is_devel_pkg (opkg_package_t *pkg)
  *
  * returns true if the tag is present
  */
-gboolean
-opkg_check_tag (opkg_package_t *pkg, gchar *tag)
+static gboolean 
+opkg_check_tag (opkg_package_t *pkg, const gchar *tag)
 {
 	if (pkg->tags && tag)
 		return (g_strrstr (pkg->tags, tag) != NULL);
@@ -164,7 +163,7 @@ backend_initialize (PkBackend *backend)
 	}
 
 #ifdef OPKG_OFFLINE_ROOT
-	opkg_set_option (opkg, "offline_root", OPKG_OFFLINE_ROOT);
+	opkg_set_option (opkg, (char *) "offline_root", OPKG_OFFLINE_ROOT);
 	opkg_re_read_config_files (opkg);
 #endif
 
@@ -181,7 +180,7 @@ backend_destroy (PkBackend *backend)
 
 
 static void
-pk_opkg_progress_cb (opkg_t *opkg, const opkg_progress_data_t *pdata, void *data)
+pk_opkg_progress_cb (opkg_t *_opkg, const opkg_progress_data_t *pdata, void *data)
 {
 	PkBackend *backend = PK_BACKEND (data);
 	if (!backend)
@@ -258,7 +257,7 @@ backend_refresh_cache (PkBackend *backend, gboolean force)
  */
 
 static void
-pk_opkg_package_list_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
+pk_opkg_package_list_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
 {
 	SearchParams *params = (SearchParams*) data;
 	gchar *uid;
@@ -307,17 +306,23 @@ pk_opkg_package_list_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
 
 	/* check filters */
 
-	if ((filters & PK_FILTER_ENUM_DEVELOPMENT) && !opkg_is_devel_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_DEVELOPMENT) && 
+                !opkg_is_devel_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_DEVELOPMENT) && opkg_is_devel_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_DEVELOPMENT) && 
+                opkg_is_devel_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_GUI) && !opkg_is_gui_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_GUI) && 
+                !opkg_is_gui_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_GUI) && opkg_is_gui_pkg (pkg))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_GUI) && 
+                opkg_is_gui_pkg (pkg))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_INSTALLED) && (!pkg->installed))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_INSTALLED) && 
+                (!pkg->installed))
 		goto end_handle;
-	if ((filters & PK_FILTER_ENUM_NOT_INSTALLED) && (pkg->installed))
+	if (pk_bitfield_contain(filters, PK_FILTER_ENUM_NOT_INSTALLED) && 
+                (pkg->installed))
 		goto end_handle;
 
 	pk_backend_package (params->backend, status, uid, pkg->description);
@@ -341,7 +346,7 @@ backend_search_thread (PkBackend *backend)
 	g_free (params->needle);
 	g_free (params);
 
-	return FALSE;
+	return TRUE;
 }
 
 static void
@@ -463,8 +468,8 @@ backend_remove_packages_thread (PkBackend *backend)
 	autoremove = GPOINTER_TO_INT (data[2]);
 	g_free (data);
 
-	opkg_set_option (opkg, "autoremove", &autoremove);
-	opkg_set_option (opkg, "force_removal_of_dependent_packages", &allow_deps);
+	opkg_set_option (opkg, (char *)"autoremove", &autoremove);
+	opkg_set_option (opkg, (char *)"force_removal_of_dependent_packages", &allow_deps);
 
 	err = 0;
 
@@ -604,7 +609,7 @@ backend_update_packages (PkBackend *backend, gchar **package_ids)
  */
 
 static void
-pk_opkg_list_upgradable_cb (opkg_t *opkg, opkg_package_t *pkg, void *data)
+pk_opkg_list_upgradable_cb (opkg_t *_opkg, opkg_package_t *pkg, void *data)
 {
 	PkBackend *backend = PK_BACKEND (data);
 	gchar *uid;
@@ -669,7 +674,7 @@ backend_get_details_thread (PkBackend *backend)
 	opkg_package_t *pkg;
 	gchar *newid;
 
-	package_ids = pk_backend_get_string (backend, "package_ids");
+        package_ids = pk_backend_get_strv(backend, "package_ids");
 	pi = pk_package_id_new_from_string (package_ids[0]);
 	if (pi == NULL)
 	{
@@ -695,7 +700,7 @@ backend_get_details_thread (PkBackend *backend)
 		for (group_index = 0; group < PK_GROUP_ENUM_UNKNOWN; group_index++) {
 			group = 1 << group_index;
 			if (!(group & backend_get_groups(backend))) continue;
-			if (opkg_check_tag(pkg, (gchar *)pk_group_enum_to_text(group))) 
+			if (opkg_check_tag(pkg, (const gchar *)pk_group_enum_to_text(group))) 
 				break;
 		}
 	}
