@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2008 Marcin Banasiak <megabajt@pld-linux.org>
+ * Copyright (C) 2008-2009 Marcin Banasiak <megabajt@pld-linux.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -81,7 +81,7 @@ static PLDGroupRegex group_perlre[] = {
 	{PK_GROUP_ENUM_MULTIMEDIA, "/.*Multimedia\\|.*Sound/"},
 	{PK_GROUP_ENUM_NETWORK, "/.*Networking.*\\|/.*Mail\\|.*News\\|.*WWW/"},
 	{PK_GROUP_ENUM_OFFICE, "/.*Editors.*\\|.*Spreadsheets/"},
-	{PK_GROUP_ENUM_OTHER, "/^Applications$\\|.*Console\\|.*Emulators\\|.*File\\|.*Printing\\|.*Terminal\\|.*Text\\|Documentation\\|^Libraries.*\\|^Themes.*\\|^X11$\\|.*Amusements\\|^X11\\/Applications$\\|^X11\\/Libraries$\\|.*Window\\ Managers.*/"},
+	{PK_GROUP_ENUM_OTHER, "/^Applications$\\|.*Console\\|.*Emulators\\|.*File\\|.*Printing\\|.*Terminal\\|.*Text\\|^Libraries.*\\|^Themes.*\\|^X11$\\|.*Amusements\\|^X11\\/Applications$\\|^X11\\/Libraries$\\|.*Window\\ Managers.*/"},
 	{PK_GROUP_ENUM_PROGRAMMING, "/.*Development.*/"},
 	{PK_GROUP_ENUM_PUBLISHING, "/.*Publishing.*/"},
 	{PK_GROUP_ENUM_SERVERS, "/Daemons\\|.*Servers/"},
@@ -260,7 +260,7 @@ get_locale_variants (PkBackend *backend, const gchar *lang)
 	tn_array *langs;
 	gchar *copy;
 	gchar *wocc = NULL;
-	gchar *sep = "@.";
+	const gchar *sep = "@.";
 	gint len;
 
 	/* first check cached_locale_variants */
@@ -393,7 +393,7 @@ poldek_get_bytes_to_download_from_ts (struct poldek_ts *ts)
 static long
 do_get_bytes_to_download (struct poldek_ts *ts, tn_array *pkgs)
 {
-	gint i;
+	size_t i;
 	long bytes = 0;
 
 	for (i = 0; i < n_array_size (pkgs); i++) {
@@ -498,7 +498,7 @@ poldek_vf_progress_reset (void *bar)
 static gint
 poldek_pkg_in_array_idx (const struct pkg *pkg, const tn_array *array, tn_fn_cmp cmp_fn)
 {
-	gint	i;
+	size_t	i;
 
 	if (array) {
 		for (i = 0; i < n_array_size (array); i++) {
@@ -534,7 +534,8 @@ ts_confirm (void *data, struct poldek_ts *ts)
 {
 	tn_array	*ipkgs, *dpkgs, *rpkgs;
 	PkBackend	*backend = (PkBackend *)data;
-	gint		i = 0, result = 1;
+	size_t		i = 0;
+	gint		result = 1;
 
 	egg_debug ("START\n");
 
@@ -681,6 +682,41 @@ pkg_cmp_name_evr_rev_recno (const struct pkg *p1, const struct pkg *p2) {
 	return rc;
 }
 
+/**
+ * do_post_search_process:
+ *
+ * Merges installed, available and removes duplicates.
+ *
+ **/
+static tn_array *
+do_post_search_process (tn_array *installed, tn_array *available)
+{
+	tn_array       *packages = NULL;
+	guint		i;
+
+	if (available != NULL) {
+		packages = n_ref (available);
+
+		if (installed != NULL) {
+			for (i = 0; i < n_array_size (installed); i++) {
+				struct pkg *pkg = n_array_nth (installed, i);
+
+				/* check for duplicates */
+				if (poldek_pkg_in_array (pkg, packages, (tn_fn_cmp)pkg_cmp_name_evr) == FALSE)
+					n_array_push (packages, pkg_link (pkg));
+
+			}
+
+			n_array_sort_ex (packages, (tn_fn_cmp)pkg_cmp_name_evr_rev_recno);
+		}
+
+	} else if (installed != NULL) {
+		packages = n_ref (installed);
+	}
+
+	return packages;
+}
+
 static gboolean
 pkg_is_installed (struct pkg *pkg)
 {
@@ -706,6 +742,53 @@ pkg_is_installed (struct pkg *pkg)
 	return is_installed ? TRUE : FALSE;
 }
 
+/**
+ * get_pkgid_from_localpath:
+ *
+ * Query rpmdb by localpath.
+ *
+ * localpath: full path to the file on local filesystem
+ *
+ * Returns: pkgid (foo-bar-0.1.2-3.i686) that owns file at localpath or NULL.
+ **/
+static gchar *
+get_pkgid_from_localpath (const gchar *localpath)
+{
+	struct pkgdb *db = NULL;
+	struct poldek_ts *ts = NULL;
+	gchar *pkgid = NULL;
+
+	g_return_val_if_fail (localpath != NULL, NULL);
+
+	ts = poldek_ts_new (ctx, 0);
+	db = pkgdb_open (ts->pmctx, ts->rootdir, NULL, O_RDONLY, NULL);
+
+	if (db) {
+		const struct pm_dbrec *ldbrec;
+		struct pkgdb_it it;
+
+		pkgdb_it_init (db, &it, PMTAG_FILE, localpath);
+
+		/* get only one package */
+		if ((ldbrec = pkgdb_it_get (&it)) != NULL) {
+			gchar *name = NULL, *version = NULL, *release = NULL, *arch = NULL;
+			gint epoch;
+
+			pm_dbrec_nevr (ldbrec, &name, &epoch, &version, &release, &arch, NULL);
+
+			pkgid = g_strdup_printf ("%s-%s-%s.%s", name, version, release, arch);
+		}
+
+		pkgdb_it_destroy (&it);
+		/* it calls pkgdb_close (db) */
+		pkgdb_free (db);
+	}
+
+	poldek_ts_free (ts);
+
+	return pkgid;
+}
+
 /**
  * poldek_get_security_updates:
  **/
@@ -942,6 +1025,34 @@ poldek_pkg_get_cves_from_pld_changelog (struct pkg *pkg, time_t since)
 	return cves;
 }
 
+/**
+ * poldek_pkg_is_devel:
+ */
+static gboolean
+poldek_pkg_is_devel (struct pkg *pkg)
+{
+	if (g_str_has_suffix (pkg->name, "-devel"))
+		return TRUE;
+	if (g_str_has_suffix (pkg->name, "-debuginfo"))
+		return TRUE;
+	if (g_str_has_suffix (pkg->name, "-static"))
+		return TRUE;
+
+	return FALSE;
+}
+
+/**
+ * poldek_pkg_is_gui:
+ */
+static gboolean
+poldek_pkg_is_gui (struct pkg *pkg)
+{
+	if (g_str_has_prefix (pkg_group (pkg), "X11"))
+		return TRUE;
+
+	return FALSE;
+}
+
 static void
 do_newest (tn_array *pkgs)
 {
@@ -964,6 +1075,53 @@ do_newest (tn_array *pkgs)
 	}
 }
 
+/**
+ * do_filtering:
+ *
+ * Apply newest, devel and gui filters (if requested).
+ *
+ **/
+static void
+do_filtering (tn_array *packages, PkBitfield filters)
+{
+	guint	i = 0;
+
+	g_return_if_fail (packages != NULL);
+
+	if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NEWEST))
+		do_newest (packages);
+
+	while (i < n_array_size (packages)) {
+		struct pkg     *pkg = n_array_nth (packages, i);
+
+		if (pk_bitfield_contain (filters, PK_FILTER_ENUM_DEVELOPMENT))
+			if (!poldek_pkg_is_devel (pkg)) {
+				n_array_remove_nth (packages, i);
+				continue;
+			}
+
+		if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_DEVELOPMENT))
+			if (poldek_pkg_is_devel (pkg)) {
+				n_array_remove_nth (packages, i);
+				continue;
+			}
+
+		if (pk_bitfield_contain (filters, PK_FILTER_ENUM_GUI))
+			if (!poldek_pkg_is_gui (pkg)) {
+				n_array_remove_nth (packages, i);
+				continue;
+			}
+
+		if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_GUI))
+			if (poldek_pkg_is_gui (pkg)) {
+				n_array_remove_nth (packages, i);
+				continue;
+			}
+
+		i++;
+	}
+}
+
 /**
  * do_requires:
  */
@@ -972,7 +1130,7 @@ do_requires (tn_array *installed, tn_array *available, tn_array *requires,
 	     struct pkg *pkg, PkBackend *backend)
 {
 	tn_array	*tmp = NULL;
-	gint		i;
+	size_t		i;
 	PkBitfield filters;
 	gboolean recursive;
 
@@ -983,7 +1141,7 @@ do_requires (tn_array *installed, tn_array *available, tn_array *requires,
 	if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) {
 		for (i = 0; i < n_array_size (installed); i++) {
 			struct pkg      *ipkg = n_array_nth (installed, i);
-			int j;
+			size_t j;
 
 			/* self match */
 			if (pkg_cmp_name_evr (pkg, ipkg) == 0)
@@ -1016,7 +1174,7 @@ do_requires (tn_array *installed, tn_array *available, tn_array *requires,
 	if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED)) {
 		for (i = 0; i < n_array_size (available); i++) {
 			struct pkg      *apkg = n_array_nth (available, i);
-			int j;
+			size_t j;
 
 			/* self match */
 			if (pkg_cmp_name_evr (pkg, apkg) == 0)
@@ -1068,7 +1226,7 @@ do_depends (tn_array *installed, tn_array *available, tn_array *depends, struct
 {
 	tn_array	*reqs = pkg->reqs;
 	tn_array	*tmp = NULL;
-	gint		i;
+	size_t		i;
 	PkBitfield filters;
 	gboolean recursive;
 
@@ -1083,7 +1241,7 @@ do_depends (tn_array *installed, tn_array *available, tn_array *depends, struct
 	for (i = 0; i < n_array_size (reqs); i++) {
 		struct capreq	*req = n_array_nth (reqs, i);
 		gboolean	found = FALSE;
-		gint		j;
+		size_t		j;
 
 		/* skip it */
 		if (capreq_is_rpmlib (req))
@@ -1277,32 +1435,6 @@ poldek_get_pkg_from_package_id (const gchar *package_id)
 	return result;
 }
 
-/**
- * poldek_pkg_is_devel:
- */
-static gboolean
-poldek_pkg_is_devel (struct pkg *pkg)
-{
-	if (g_str_has_suffix (pkg->name, "-devel"))
-		return TRUE;
-	if (g_str_has_suffix (pkg->name, "-debuginfo"))
-		return TRUE;
-
-	return FALSE;
-}
-
-/**
- * poldek_pkg_is_gui:
- */
-static gboolean
-poldek_pkg_is_gui (struct pkg *pkg)
-{
-	if (g_str_has_prefix (pkg_group (pkg), "X11"))
-		return TRUE;
-
-	return FALSE;
-}
-
 /**
  * search_package_thread:
  */
@@ -1311,7 +1443,9 @@ search_package_thread (PkBackend *backend)
 {
 	PkBitfield		filters;
 	PkProvidesEnum		provides;
-	gchar			*search_cmd = NULL;
+	gchar		       *search_cmd_available = NULL;
+	gchar		       *search_cmd_installed = NULL;
+	tn_array	       *pkgs = NULL;
 	const gchar *search;
 	guint mode;
 
@@ -1322,12 +1456,14 @@ search_package_thread (PkBackend *backend)
 
 	/* GetPackages */
 	if (mode == SEARCH_ENUM_NONE) {
-		search_cmd = g_strdup ("ls -q");
+		search_cmd_installed = g_strdup ("ls -q");
+		search_cmd_available = g_strdup (search_cmd_installed);
 	/* SearchName */
 	} else if (mode == SEARCH_ENUM_NAME) {
 		search = pk_backend_get_string (backend, "search");
 
-		search_cmd = g_strdup_printf ("ls -q *%s*", search);
+		search_cmd_installed = g_strdup_printf ("ls -q *%s*", search);
+		search_cmd_available = g_strdup (search_cmd_installed);
 	/* SearchGroup */
 	} else if (mode == SEARCH_ENUM_GROUP) {
 		PkGroupEnum	group;
@@ -1338,30 +1474,46 @@ search_package_thread (PkBackend *backend)
 		group = pk_group_enum_from_text (search);
 		regex = pld_group_get_regex_from_enum (group);
 
-		search_cmd = g_strdup_printf ("search -qg --perlre %s", regex);
+		search_cmd_installed = g_strdup_printf ("search -qg --perlre %s", regex);
+		search_cmd_available = g_strdup (search_cmd_installed);
 	/* SearchDetails */
 	} else if (mode == SEARCH_ENUM_DETAILS) {
 		search = pk_backend_get_string (backend, "search");
 
-		search_cmd = g_strdup_printf ("search -dsq *%s*", search);
+		search_cmd_installed = g_strdup_printf ("search -dsq *%s*", search);
+		search_cmd_available = g_strdup (search_cmd_installed);
 	/* SearchFile */
 	} else if (mode == SEARCH_ENUM_FILE) {
 		search = pk_backend_get_string (backend, "search");
 
-		search_cmd = g_strdup_printf ("search -qlf *%s*", search);
+		if (*search == '/') {
+			gchar *pkgid = NULL;
+
+			/* use rpmdb to get local packages (equivalent to: rpm -qf /foo/bar) */
+			if ((pkgid = get_pkgid_from_localpath (search))) {
+				search_cmd_installed = g_strdup_printf ("ls -q %s", pkgid);
+				g_free (pkgid);
+			}
+
+			search_cmd_available = g_strdup_printf ("search -ql --perlre /^%s$/", search);
+		} else {
+			search_cmd_installed = g_strdup_printf ("search -ql --perlre /.*%s.*/", search);
+			search_cmd_available = g_strdup (search_cmd_installed);
+		}
 	/* WhatProvides */
 	} else if (mode == SEARCH_ENUM_PROVIDES) {
 		provides = pk_backend_get_uint (backend, "provides");
 
 		search = pk_backend_get_string (backend, "search");
 
-		if (provides == PK_PROVIDES_ENUM_ANY) {
-			search_cmd = g_strdup_printf ("search -qp %s", search);
+		if (provides == PK_PROVIDES_ENUM_ANY || provides == PK_PROVIDES_ENUM_CODEC) {
+			search_cmd_installed = g_strdup_printf ("search -qp %s", search);
 		} else if (provides == PK_PROVIDES_ENUM_MODALIAS) {
-		} else if (provides == PK_PROVIDES_ENUM_CODEC) {
 		} else if (provides == PK_PROVIDES_ENUM_MIMETYPE) {
-			search_cmd = g_strdup_printf ("search -qp mimetype(%s)", search);
+			search_cmd_installed = g_strdup_printf ("search -qp mimetype(%s)", search);
 		}
+
+		search_cmd_available = g_strdup (search_cmd_installed);
 	/* Resolve */
 	} else if (mode == SEARCH_ENUM_RESOLVE) {
 		gchar **package_ids;
@@ -1371,20 +1523,21 @@ search_package_thread (PkBackend *backend)
 
 		packages_str = g_strjoinv(" ", package_ids);
 
-		search_cmd = g_strdup_printf ("ls -q %s", packages_str);
+		search_cmd_installed = g_strdup_printf ("ls -q %s", packages_str);
+		search_cmd_available = g_strdup (search_cmd_installed);
 
 		g_free (packages_str);
 	}
 
-	if (search_cmd != NULL) {
-		tn_array	*pkgs = NULL, *installed = NULL, *available = NULL;
+	if (search_cmd_installed != NULL || search_cmd_available != NULL) {
+		tn_array	*installed = NULL, *available = NULL;
 
-		if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED)) {
-			installed = execute_packages_command ("cd /installed; %s", search_cmd);
+		if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED) && search_cmd_installed) {
+			installed = execute_packages_command ("cd /installed; %s", search_cmd_installed);
 		}
 
-		if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED)) {
-			available = execute_packages_command ("cd /all-avail; %s", search_cmd);
+		if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) && search_cmd_available) {
+			available = execute_packages_command ("cd /all-avail; %s", search_cmd_available);
 		}
 
 		if (!pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) &&
@@ -1406,59 +1559,49 @@ search_package_thread (PkBackend *backend)
 			n_array_sort_ex(pkgs, (tn_fn_cmp)pkg_cmp_name_evr_rev_recno);
 
 			n_array_free (installed);
-		} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED) || available) {
+
+		} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED) && available) {
+			tn_array *dbpkgs = NULL;
+			guint i;
+
+			dbpkgs = poldek_get_installed_packages ();
+			pkgs = n_array_new (4, (tn_fn_free)pkg_free, NULL);
+
+			for (i = 0; i < n_array_size (available); i++) {
+				struct pkg *pkg = n_array_nth (available, i);
+
+				/* drop installed packages */
+				if (!poldek_pkg_in_array (pkg, dbpkgs, (tn_fn_cmp)pkg_cmp_name_evr)) {
+					n_array_push (pkgs, pkg_link (pkg));
+				}
+			}
+
+			n_array_free (available);
+			n_array_free (dbpkgs);
+
+		} else if (available) {
 			pkgs = available;
+
 		} else if (pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) || installed)
 			pkgs = installed;
+	}
 
-		if (pkgs) {
-			gint	i;
+	do_filtering (pkgs, filters);
 
-			if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NEWEST))
-				do_newest (pkgs);
+	if (pkgs) {
+		guint	i;
 
-			for (i = 0; i < n_array_size (pkgs); i++) {
-				struct pkg *pkg = n_array_nth (pkgs, i);
+		for (i = 0; i < n_array_size (pkgs); i++) {
+			struct pkg *pkg = n_array_nth (pkgs, i);
 
-				if (sigint_reached ())
-					break;
+			if (sigint_reached ())
+				break;
 
-				/* check if we have to do development filtering
-				 * (devel or ~devel in filters) */
-				if (pk_bitfield_contain (filters, PK_FILTER_ENUM_DEVELOPMENT) ||
-				    pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_DEVELOPMENT)) {
-					if (pk_bitfield_contain (filters, PK_FILTER_ENUM_DEVELOPMENT)) {
-						/* devel in filters */
-						if (!poldek_pkg_is_devel (pkg))
-							continue;
-					} else {
-						/* ~devel in filters */
-						if (poldek_pkg_is_devel (pkg))
-							continue;
-					}
-				}
-
-				/* check if we have to do gui filtering
-				 * (gui or ~gui in filters) */
-				if (pk_bitfield_contain (filters, PK_FILTER_ENUM_GUI) ||
-				    pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_GUI)) {
-					if (pk_bitfield_contain (filters, PK_FILTER_ENUM_GUI)) {
-						/* gui in filters */
-						if (!poldek_pkg_is_gui (pkg))
-							continue;
-					} else {
-						/* ~gui in filters */
-						if (poldek_pkg_is_gui (pkg))
-							continue;
-					}
-				}
-
-				poldek_backend_package (backend, pkg, PK_INFO_ENUM_UNKNOWN, filters);
-			}
-			n_array_free (pkgs);
-		} else {
-			pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND, "Package not found");
+			poldek_backend_package (backend, pkg, PK_INFO_ENUM_UNKNOWN, filters);
 		}
+		n_array_free (pkgs);
+	} else {
+		pk_backend_error_code (backend, PK_ERROR_ENUM_PACKAGE_NOT_FOUND, "Package not found");
 	}
 
 	if (sigint_reached ()) {
@@ -1475,7 +1618,8 @@ search_package_thread (PkBackend *backend)
 		}
 	}
 
-	g_free (search_cmd);
+	g_free (search_cmd_installed);
+	g_free (search_cmd_available);
 
 	pk_backend_finished (backend);
 	return TRUE;
@@ -1660,7 +1804,7 @@ show_rpm_progress (PkBackend *backend, gchar *message)
 
 		/* emit remove for packages marked for removal */
 		if (rpkgs) {
-			gint i;
+			size_t i;
 
 			/* XXX: don't release rpkgs array here! */
 			for (i = 0; i < n_array_size (rpkgs); i++) {
@@ -1948,7 +2092,7 @@ backend_download_packages_thread (PkBackend *backend)
 	tn_array *pkgs;
 	gchar **package_ids;
 	const gchar *destdir;
-	gint i;
+	size_t i;
 
 	package_ids = pk_backend_get_strv (backend, "package_ids");
 	destdir = pk_backend_get_string (backend, "directory");
@@ -1972,8 +2116,29 @@ backend_download_packages_thread (PkBackend *backend)
 	pd->bytesget = 0;
 	pd->bytesdownload = poldek_get_bytes_to_download (ts, pkgs);
 
-	if (!packages_fetch (poldek_get_pmctx (ts->ctx), pkgs, destdir, 1)) {
-		/* something goes wrong */
+	if (packages_fetch (poldek_get_pmctx (ts->ctx), pkgs, destdir, 1)) {
+		GString *filelist = NULL;
+		gchar *result = NULL;
+
+		filelist = g_string_new ("");
+
+		/* emit the file list we downloaded */
+		for (i = 0; i < n_array_size (pkgs); i++) {
+			struct pkg *pkg = n_array_nth (pkgs, i);
+			gchar buf[256];
+
+			if (i > 0)
+			    g_string_append_c (filelist, ';');
+
+			g_string_append_printf (filelist, "%s/%s", destdir,
+						pkg_filename (pkg, buf, sizeof(buf)));
+		}
+
+		result = g_string_free (filelist, FALSE);
+
+		pk_backend_files (backend, NULL, result);
+
+		g_free (result);
 	}
 
 	poldek_ts_free (ts);
@@ -2038,6 +2203,15 @@ backend_get_filters (PkBackend *backend)
 		-1);
 }
 
+/**
+ * backend_get_mime_types:
+ */
+static gchar *
+backend_get_mime_types (PkBackend *backend)
+{
+	return g_strdup ("application/x-rpm");
+}
+
 /**
  * backend_get_cancel:
  **/
@@ -2057,7 +2231,7 @@ backend_get_depends_thread (PkBackend *backend)
 {
 	struct pkg	*pkg;
 	tn_array	*deppkgs, *available, *installed;
-	gint		i;
+	size_t		i;
 	gchar **package_ids;
 
 	pb_load_packages (backend);
@@ -2166,7 +2340,7 @@ static gboolean
 backend_get_files_thread (PkBackend *backend)
 {
 	gchar **package_ids;
-	gint n;
+	size_t n;
 
 	package_ids = pk_backend_get_strv (backend, "package_ids");
 
@@ -2180,7 +2354,8 @@ backend_get_files_thread (PkBackend *backend)
 		if (pkg != NULL) {
 			struct pkgflist *flist = pkg_get_flist (pkg);
 			GString *filelist;
-			gchar *result, *sep;
+			gchar *result;
+			const gchar *sep;
 			gint i, j;
 
 			sep = "";
@@ -2238,6 +2413,61 @@ backend_get_files (PkBackend *backend, gchar **package_ids)
 /**
  * backend_get_packages:
  **/
+static gboolean
+backend_get_packages_thread (PkBackend *backend)
+{
+	PkBitfield	filters;
+	tn_array       *installed = NULL;
+	tn_array       *available = NULL;
+	tn_array       *packages = NULL;
+	guint		i;
+
+	filters = pk_backend_get_uint (backend, "filters");
+
+	pk_backend_set_percentage (backend, 0);
+
+	pb_load_packages (backend);
+
+	if (pk_bitfield_contain (filters, PK_FILTER_ENUM_NOT_INSTALLED) == FALSE)
+		installed = poldek_get_installed_packages ();
+
+	if (pk_bitfield_contain (filters, PK_FILTER_ENUM_INSTALLED) == FALSE)
+		available = poldek_get_avail_packages (ctx);
+
+	pk_backend_set_percentage (backend, 4);
+
+	packages = do_post_search_process (installed, available);
+
+	do_filtering (packages, filters);
+
+	pk_backend_set_percentage (backend, 10);
+
+	if (packages != NULL) {
+		for (i = 0; i < n_array_size (packages); i++) {
+			struct pkg     *pkg = n_array_nth (packages, i);
+
+			if (sigint_reached ())
+				break;
+
+			pk_backend_set_percentage (backend, (guint)(10 + (90 * (float)(i + 1) / n_array_size (packages))));
+
+			poldek_backend_package (backend, pkg, PK_INFO_ENUM_UNKNOWN, filters);
+		}
+	}
+
+	if (sigint_reached ())
+		pk_backend_error_code (backend, PK_ERROR_ENUM_TRANSACTION_CANCELLED, "");
+	else
+		pk_backend_set_percentage (backend, 100);
+
+	n_array_cfree (&installed);
+	n_array_cfree (&available);
+	n_array_cfree (&packages);
+
+	pk_backend_finished (backend);
+	return TRUE;
+}
+
 static void
 backend_get_packages (PkBackend *backend, PkBitfield filters)
 {
@@ -2245,8 +2475,7 @@ backend_get_packages (PkBackend *backend, PkBitfield filters)
 	poldek_backend_set_allow_cancel (backend, TRUE, TRUE);
 	pb_error_clean ();
 
-	pk_backend_set_uint (backend, "mode", SEARCH_ENUM_NONE);
-	pk_backend_thread_create (backend, search_package_thread);
+	pk_backend_thread_create (backend, backend_get_packages_thread);
 }
 
 /**
@@ -2257,7 +2486,7 @@ backend_get_requires_thread (PkBackend *backend)
 {
 	struct pkg	*pkg;
 	tn_array	*reqpkgs, *available, *installed;
-	gint		i;
+	size_t		i;
 	gchar **package_ids;
 
 	pb_load_packages (backend);
@@ -2306,7 +2535,7 @@ get_obsoletedby_pkg (struct pkg *pkg)
 {
 	tn_array *dbpkgs;
 	GString *obsoletes = NULL;
-	gint i;
+	size_t i;
 
 	g_return_val_if_fail (pkg != NULL, NULL);
 
@@ -2511,7 +2740,7 @@ backend_install_packages_thread (PkBackend *backend)
 	struct vf_progress	vf_progress;
 	gchar **package_ids;
 	GString *cmd;
-	gint i;
+	size_t i;
 
 	pk_backend_set_uint (backend, "ts_type", TS_TYPE_ENUM_INSTALL);
 	package_ids = pk_backend_get_strv (backend, "package_ids");
@@ -2583,7 +2812,7 @@ backend_refresh_cache_thread (PkBackend *backend)
 	sources = poldek_get_sources (ctx);
 
 	if (sources) {
-		gint	i;
+		size_t	i;
 
 		pk_backend_set_uint (backend, "ts_type", TS_TYPE_ENUM_REFRESH_CACHE);
 		pd->step = 0;
@@ -2649,7 +2878,7 @@ backend_remove_packages_thread (PkBackend *backend)
 	GString *cmd;
 	gchar *command;
 	gchar **package_ids;
-	gint i;
+	size_t i;
 
 	package_ids = pk_backend_get_strv (backend, "package_ids");
 	pb_load_packages (backend);
@@ -2808,7 +3037,7 @@ backend_get_repo_list (PkBackend *backend, PkBitfield filters)
 	sources = poldek_get_sources (ctx);
 
 	if (sources) {
-		gint i;
+		size_t i;
 
 		for (i = 0; i < n_array_size (sources); i++) {
 			struct source *src = n_array_nth (sources, i);
@@ -2847,7 +3076,7 @@ PK_BACKEND_OPTIONS (
 	backend_destroy,				/* destroy */
 	backend_get_groups,				/* get_groups */
 	backend_get_filters,				/* get_filters */
-	NULL,						/* get_mime_types */
+	backend_get_mime_types,				/* get_mime_types */
 	backend_get_cancel,				/* cancel */
 	backend_download_packages,			/* download_packages */
 	NULL,						/* get_categories */
