@@ -107,9 +107,12 @@ class PackageKitSmartProgress(Progress):
                     or filename.find('Release') != -1:
                         self._backend.status(STATUS_DOWNLOAD_REPOSITORY)
                     elif filename.find('primary') != -1 \
-                    or filename.find('Packages') != -1:
+                    or filename.find('Packages') != -1 \
+                    or filename.find('PACKAGES.TXT') != -1 \
+                    or filename.find('.db.tar.gz') != -1:
                         self._backend.status(STATUS_DOWNLOAD_PACKAGELIST)
-                    elif filename.find('filelists') != -1:
+                    elif filename.find('filelists') != -1 \
+                    or filename.find('.files.tar.gz') != -1:
                         self._backend.status(STATUS_DOWNLOAD_FILELIST)
                     elif filename.find('other') != -1:
                         self._backend.status(STATUS_DOWNLOAD_CHANGELOG)
@@ -943,10 +946,12 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
         return False
 
     def _splitpackage(self, package):
-        from smart.backends.rpm.base import RPMPackage
+        #from smart.backends.rpm.base import RPMPackage
         from smart.backends.deb.base import DebPackage
         from smart.backends.slack.base import SlackPackage
-        if isinstance(package, RPMPackage):
+        #from smart.backends.arch.base import ArchPackage
+        #if isinstance(package, RPMPackage):
+        if package.__class__.__name__ == 'RPMPackage':
             name = package.name
             version, arch = package.version.split('@')
         elif isinstance(package, DebPackage):
@@ -956,7 +961,13 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
             name = package.name
             ver, arch, rel = package.version.rsplit('-')
             version = "%s-%s" % (ver, rel)
+        #elif isinstance(package, ArchPackage):
+        elif package.__class__.__name__ == 'ArchPackage':
+            name = package.name
+            ver, rel, arch = package.version.rsplit('-')
+            version = "%s-%s" % (ver, rel)
         else:
+            name = package.name
             version, arch = package.version, self._machine()
         return name, version, arch
 
@@ -970,6 +981,8 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
                 self.systemchannel = "deb-sys"
             elif "slack-sys" in channels:
                 self.systemchannel = "slack-sys"
+            elif "arch-sys" in channels:
+                self.systemchannel = "arch-sys"
         if self.systemchannel == "rpm-sys":
             pkg = "%s-%s@%s" % (name, version, arch)
         elif self.systemchannel == "deb-sys":
@@ -977,6 +990,9 @@ class PackageKitSmartBackend(PackageKitBaseBackend):
         elif self.systemchannel == "slack-sys":
             ver, rel = version.rsplit("-")
             pkg = "%s-%s-%s-%s" % (name, ver, arch, rel)
+        elif self.systemchannel == "arch-sys":
+            ver, rel = version.rsplit("-")
+            pkg = "%s-%s-%s-%s" % (name, ver, rel, arch)
         else:
             pkg = "%s-%s" % (name, version)
         return pkg
