@@ -33,6 +33,7 @@ PK_BACKEND_OPTIONS (
 	NULL,					/* get_mime_types */
 	NULL,					/* cancel */
 	NULL,					/* download_packages */
+	NULL,					/* get_categories */
 	NULL,					/* get_depends */
 	NULL,					/* get_details */
 	NULL,					/* get_distro_upgrades */
@@ -55,7 +56,6 @@ PK_BACKEND_OPTIONS (
 	NULL,					/* search_file */
 	NULL,					/* search_group */
 	NULL,					/* search_name */
-	NULL,					/* service_pack */
 	NULL,					/* update_package */
 	NULL,					/* update_system */
 	NULL					/* what_provides */
