@@ -162,6 +162,10 @@ pk_console_package_cb (PkClient *client, const PkPackageObj *obj, gpointer data)
 	gchar *info_pad = NULL;
 	gchar *text = NULL;
 
+	/* ignore finished */
+	if (obj->info == PK_INFO_ENUM_FINISHED)
+		goto out;
+
 	/* make these all the same length */
 	info_pad = pk_strpad (pk_info_enum_to_text (obj->info), 12);
 
@@ -218,27 +222,67 @@ pk_console_transaction_cb (PkClient *client, const PkTransactionObj *obj, gpoint
 {
 	struct passwd *pw;
 	const gchar *role_text;
+	gchar **lines;
+	gchar **parts;
+	guint i, lines_len;
+	PkPackageId *id;
 
 	role_text = pk_role_enum_to_text (obj->role);
 	if (awaiting_space)
 		g_print ("\n");
-	g_print ("Transaction  : %s\n", obj->tid);
-	g_print (" timespec    : %s\n", obj->timespec);
-	g_print (" succeeded   : %i\n", obj->succeeded);
-	g_print (" role        : %s\n", role_text);
-	g_print (" duration    : %i (seconds)\n", obj->duration);
-	g_print (" data        : %s\n", obj->data);
-	g_print (" cmdline     : %s\n", obj->cmdline);
-	g_print (" uid         : %i\n", obj->uid);
+	/* TRANSLATORS: this is an atomic transaction */
+	g_print ("%s: %s\n", _("Transaction"), obj->tid);
+	/* TRANSLATORS: this is the time the transaction was started in system timezone */
+	g_print (" %s: %s\n", _("System time"), obj->timespec);
+	/* TRANSLATORS: this is if the transaction succeeded or not */
+	g_print (" %s: %s\n", _("Succeeded"), obj->timespec ? _("True") : _("False"));
+	/* TRANSLATORS: this is the transactions role, e.g. "update-system" */
+	g_print (" %s: %s\n", _("Role"), role_text);
+
+	/* only print if not null */
+	if (obj->duration > 0) {
+		/* TRANSLATORS: this is The duration of the transaction */
+		g_print (" %s: %i %s\n", _("Duration"), obj->duration, _("(seconds)"));
+	}
+
+	/* TRANSLATORS: this is The command line used to do the action */
+	g_print (" %s: %s\n", _("Command line"), obj->cmdline);
+	/* TRANSLATORS: this is the user ID of the user that started the action */
+	g_print (" %s: %i\n", _("User ID"), obj->uid);
 
 	/* query real name */
 	pw = getpwuid(obj->uid);
 	if (pw != NULL) {
-		if (pw->pw_name != NULL)
-			g_print (" user name   : %s\n", pw->pw_name);
-		if (pw->pw_gecos != NULL)
-			g_print (" real name   : %s\n", pw->pw_gecos);
+		if (pw->pw_name != NULL) {
+			/* TRANSLATORS: this is the username, e.g. hughsie */
+			g_print (" %s: %s\n", _("Username"), pw->pw_name);
+		}
+		if (pw->pw_gecos != NULL) {
+			/* TRANSLATORS: this is the users real name, e.g. "Richard Hughes" */
+			g_print (" %s: %s\n", _("Real name"), pw->pw_gecos);
+		}
 	}
+
+	/* TRANSLATORS: these are packages touched by the transaction */
+	lines = g_strsplit (obj->data, "\n", -1);
+	lines_len = g_strv_length (lines);
+	if (lines_len > 0)
+		g_print (" %s\n", _("Affected packages:"));
+	else
+		g_print (" %s\n", _("Affected packages: None"));
+	for (i=0; i<lines_len; i++) {
+		parts = g_strsplit (lines[i], "\t", 3);
+		id = pk_package_id_new_from_string (parts[1]);
+		g_print (" - %s %s", parts[0], id->name);
+		if (!egg_strzero (id->version))
+			g_print ("-%s", id->version);
+		if (!egg_strzero (id->arch))
+			g_print (".%s", id->arch);
+		g_print ("\n");
+		pk_package_id_free (id);
+		g_strfreev (parts);
+	}
+	g_strfreev (lines);
 }
 
 /**
@@ -249,9 +293,12 @@ pk_console_distro_upgrade_cb (PkClient *client, const PkDistroUpgradeObj *obj, g
 {
 	if (awaiting_space)
 		g_print ("\n");
-	g_print ("Distro       : %s\n", obj->name);
-	g_print (" type        : %s\n", pk_update_state_enum_to_text (obj->state));
-	g_print (" summary     : %s\n", obj->summary);
+	/* TRANSLATORS: this is the distro, e.g. Fedora 10 */
+	g_print ("%s: %s\n", _("Distribution"), obj->name);
+	/* TRANSLATORS: this is type of update, stable or testing */
+	g_print (" %s: %s\n", _("Type"), pk_update_state_enum_to_text (obj->state));
+	/* TRANSLATORS: this is any summary text describing the upgrade */
+	g_print (" %s: %s\n", _("Summary"), obj->summary);
 }
 
 /**
@@ -262,14 +309,22 @@ pk_console_category_cb (PkClient *client, const PkCategoryObj *obj, gpointer use
 {
 	if (awaiting_space)
 		g_print ("\n");
-	g_print ("Category  : %s\n", obj->name);
-	g_print (" cat_id   : %s\n", obj->cat_id);
-	if (!egg_strzero (obj->parent_id))
-		g_print (" parent   : %s\n", obj->parent_id);
-	g_print (" name     : %s\n", obj->name);
-	if (!egg_strzero (obj->summary))
-		g_print (" summary  : %s\n", obj->summary);
-	g_print (" icon     : %s\n", obj->icon);
+	/* TRANSLATORS: this is the group category name */
+	g_print ("%s: %s\n", _("Category"), obj->name);
+	/* TRANSLATORS: this is group identifier */
+	g_print (" %s: %s\n", _("ID"), obj->cat_id);
+	if (!egg_strzero (obj->parent_id)) {
+		/* TRANSLATORS: this is the parent group */
+		g_print (" %s: %s\n", _("Parent"), obj->parent_id);
+	}
+	/* TRANSLATORS: this is the name of the parent group */
+	g_print (" %s: %s\n", _("Name"), obj->name);
+	if (!egg_strzero (obj->summary)) {
+		/* TRANSLATORS: this is the summary of the group */
+		g_print (" %s: %s\n", _("Summary"), obj->summary);
+	}
+	/* TRANSLATORS: this is preferred icon for the group */
+	g_print (" %s: %s\n", _("Icon"), obj->icon);
 }
 
 /**
@@ -285,31 +340,54 @@ pk_console_update_detail_cb (PkClient *client, const PkUpdateDetailObj *detail,
 		g_print ("\n");
 	/* TRANSLATORS: this is a header for the package that can be updated */
 	g_print ("%s\n", _("Details about the update:"));
-	g_print ("  package:    '%s-%s.%s'\n", detail->id->name, detail->id->version, detail->id->arch);
-	if (!egg_strzero (detail->updates))
-		g_print ("  updates:    %s\n", detail->updates);
-	if (!egg_strzero (detail->obsoletes))
-		g_print ("  obsoletes:  %s\n", detail->obsoletes);
-	if (!egg_strzero (detail->vendor_url))
-		g_print ("  vendor URL: %s\n", detail->vendor_url);
-	if (!egg_strzero (detail->bugzilla_url))
-		g_print ("  bug URL:    %s\n", detail->bugzilla_url);
-	if (!egg_strzero (detail->cve_url))
-		g_print ("  cve URL:    %s\n", detail->cve_url);
-	if (detail->restart != PK_RESTART_ENUM_NONE)
-		g_print ("  restart:    %s\n", pk_restart_enum_to_text (detail->restart));
-	if (!egg_strzero (detail->update_text))
-		g_print ("  update_text:%s\n", detail->update_text);
-	if (!egg_strzero (detail->changelog))
-		g_print ("  changelog:  %s\n", detail->changelog);
-	if (detail->state != PK_UPDATE_STATE_ENUM_UNKNOWN)
-		g_print ("  state:      %s\n", pk_update_state_enum_to_text (detail->state));
+	/* TRANSLATORS: details about the update, package name and version */
+	g_print (" %s: '%s-%s.%s'\n", _("Package"), detail->id->name, detail->id->version, detail->id->arch);
+	if (!egg_strzero (detail->updates)) {
+		/* TRANSLATORS: details about the update, any packages that this update updates */
+		g_print (" %s: %s\n", _("Updates"), detail->updates);
+	}
+	if (!egg_strzero (detail->obsoletes)) {
+		/* TRANSLATORS: details about the update, any packages that this update obsoletes */
+		g_print (" %s: %s\n", _("Obsoletes"), detail->obsoletes);
+	}
+	if (!egg_strzero (detail->vendor_url)) {
+		/* TRANSLATORS: details about the update, the vendor URLs */
+		g_print (" %s: %s\n", _("Vendor"), detail->vendor_url);
+	}
+	if (!egg_strzero (detail->bugzilla_url)) {
+		/* TRANSLATORS: details about the update, the bugzilla URLs */
+		g_print (" %s: %s\n", _("Bugzilla"), detail->bugzilla_url);
+	}
+	if (!egg_strzero (detail->cve_url)) {
+		/* TRANSLATORS: details about the update, the CVE URLs */
+		g_print (" %s: %s\n", _("CVE"), detail->cve_url);
+	}
+	if (detail->restart != PK_RESTART_ENUM_NONE) {
+		/* TRANSLATORS: details about the update, if the package requires a restart */
+		g_print (" %s: %s\n", _("Restart"), pk_restart_enum_to_text (detail->restart));
+	}
+	if (!egg_strzero (detail->update_text)) {
+		/* TRANSLATORS: details about the update, any description of the update */
+		g_print (" %s: %s\n", _("Update text"), detail->update_text);
+	}
+	if (!egg_strzero (detail->changelog)) {
+		/* TRANSLATORS: details about the update, the changelog for the package */
+		g_print (" %s: %s\n", _("Changes"), detail->changelog);
+	}
+	if (detail->state != PK_UPDATE_STATE_ENUM_UNKNOWN) {
+		/* TRANSLATORS: details about the update, the ongoing state of the update */
+		g_print (" %s: %s\n", _("State"), pk_update_state_enum_to_text (detail->state));
+	}
 	issued = pk_iso8601_from_date (detail->issued);
-	if (!egg_strzero (issued))
-		g_print ("  issued:     %s\n", issued);
+	if (!egg_strzero (issued)) {
+		/* TRANSLATORS: details about the update, date the update was issued */
+		g_print (" %s: %s\n", _("Issued"), issued);
+	}
 	updated = pk_iso8601_from_date (detail->updated);
-	if (!egg_strzero (updated))
-		g_print ("  updated:    %s\n", updated);
+	if (!egg_strzero (updated)) {
+		/* TRANSLATORS: details about the update, date the update was updated */
+		g_print (" %s: %s\n", _("Updated"), updated);
+	}
 	g_free (issued);
 	g_free (updated);
 }
@@ -321,15 +399,10 @@ static void
 pk_console_repo_detail_cb (PkClient *client, const gchar *repo_id,
 			   const gchar *description, gboolean enabled, gpointer data)
 {
-	gchar *repo;
-	repo = pk_strpad (repo_id, 28);
 	if (awaiting_space)
 		g_print ("\n");
-	if (enabled)
-		g_print ("  enabled   %s %s\n", repo, description);
-	else
-		g_print ("  disabled  %s %s\n", repo, description);
-	g_free (repo);
+	/* TRANSLATORS: if the repo is enabled */
+	g_print (" %s\t%s\t%s\n", enabled ? _("True") : _("False"), repo_id, description);
 }
 
 /**
@@ -338,7 +411,7 @@ pk_console_repo_detail_cb (PkClient *client, const gchar *repo_id,
 static gboolean
 pk_console_pulse_bar (PulseState *pulse_state)
 {
-	guint i;
+	gint i;
 
 	if (!has_output_bar)
 		return TRUE;
@@ -400,9 +473,9 @@ pk_console_progress_changed_cb (PkClient *client, guint percentage, guint subper
 {
 	if (!is_console) {
 		if (percentage != PK_CLIENT_PERCENTAGE_INVALID)
-			g_print ("percentage: %i%%\n", percentage);
+			g_print ("%s: %i%%\n", _("Percentage"), percentage);
 		else
-			g_print ("percentage: unknown\n");
+			g_print ("%s: %s\n", _("Percentage"), _("Unknown"));
 		return;
 	}
 	percentage_last = percentage;
@@ -422,7 +495,7 @@ pk_console_progress_changed_cb (PkClient *client, guint percentage, guint subper
  * pk_console_signature_finished_cb:
  **/
 static void
-pk_console_signature_finished_cb (PkClient *client_signature, PkExitEnum exit, guint runtime, gpointer data)
+pk_console_signature_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runtime, gpointer data)
 {
 	gboolean ret;
 	GError *error = NULL;
@@ -440,20 +513,54 @@ pk_console_signature_finished_cb (PkClient *client_signature, PkExitEnum exit, g
  * pk_console_install_files_finished_cb:
  **/
 static void
-pk_console_install_files_finished_cb (PkClient *client_signature, PkExitEnum exit, guint runtime, gpointer data)
+pk_console_install_files_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runtime, gpointer data)
 {
 	g_main_loop_quit (loop);
 }
 
+/**
+ * pk_console_require_restart_cb:
+ **/
+static void
+pk_console_require_restart_cb (PkClient *client, PkRestartEnum restart, const PkPackageId *id, gpointer data)
+{
+	if (restart == PK_RESTART_ENUM_SYSTEM) {
+		/* TRANSLATORS: a package requires the system to be restarted */
+		g_print ("%s %s-%s.%s\n", _("System restart required by:"), id->name, id->version, id->arch);
+	} else if (restart == PK_RESTART_ENUM_SESSION) {
+		/* TRANSLATORS: a package requires the session to be restarted */
+		g_print ("%s %s-%s.%s\n", _("Session restart required:"), id->name, id->version, id->arch);
+	} else if (restart == PK_RESTART_ENUM_APPLICATION) {
+		/* TRANSLATORS: a package requires the application to be restarted */
+		g_print ("%s %s-%s.%s\n", _("Application restart required by:"), id->name, id->version, id->arch);
+	}
+}
+
+/**
+ * pk_console_destroy_cb:
+ **/
+static void
+pk_console_destroy_cb (PkClient *client, gpointer data)
+{
+	gboolean ret;
+
+	/* exit our private loop if it's running */
+	ret = g_main_loop_is_running (loop);
+	if (ret) {
+		egg_warning ("quitting loop due to transaction being destroyed");
+		g_main_loop_quit (loop);
+	}
+}
+
 /**
  * pk_console_finished_cb:
  **/
 static void
-pk_console_finished_cb (PkClient *client, PkExitEnum exit, guint runtime, gpointer data)
+pk_console_finished_cb (PkClient *client, PkExitEnum exit_enum, guint runtime, gpointer data)
 {
 	PkRoleEnum role;
 	const gchar *role_text;
-	gfloat time;
+	gfloat time_s;
 	PkRestartEnum restart;
 
 	pk_client_get_role (client, &role, NULL, NULL);
@@ -467,36 +574,40 @@ pk_console_finished_cb (PkClient *client, PkExitEnum exit, guint runtime, gpoint
 		g_source_remove (timer_id);
 
 	role_text = pk_role_enum_to_text (role);
-	time = (gfloat) runtime / 1000.0;
+	time_s = (gfloat) runtime / 1000.0;
 
 	/* do we need to new line? */
 	if (awaiting_space)
 		g_print ("\n");
-	egg_debug ("%s runtime was %.1f seconds", role_text, time);
+	egg_debug ("%s runtime was %.1f seconds", role_text, time_s);
 
 	/* is there any restart to notify the user? */
 	restart = pk_client_get_require_restart (client);
-	if (restart == PK_RESTART_ENUM_SYSTEM)
+	if (restart == PK_RESTART_ENUM_SYSTEM) {
+		/* TRANSLATORS: a package needs to restart they system */
 		g_print ("%s\n", _("Please restart the computer to complete the update."));
-	else if (restart == PK_RESTART_ENUM_SESSION)
+	} else if (restart == PK_RESTART_ENUM_SESSION) {
+		/* TRANSLATORS: a package needs to restart the session */
 		g_print ("%s\n", _("Please logout and login to complete the update."));
-	else if (restart == PK_RESTART_ENUM_APPLICATION)
+	} else if (restart == PK_RESTART_ENUM_APPLICATION) {
+		/* TRANSLATORS: a package needs to restart the application */
 		g_print ("%s\n", _("Please restart the application as it is being used."));
+	}
 
 	if (role == PK_ROLE_ENUM_INSTALL_FILES &&
-	    exit == PK_EXIT_ENUM_FAILED && need_requeue) {
+	    exit_enum == PK_EXIT_ENUM_FAILED && need_requeue) {
 		egg_warning ("waiting for second install file to finish");
 		return;
 	}
 
 	/* have we failed to install, and the gpg key is now installed */
-	if (exit == PK_EXIT_ENUM_KEY_REQUIRED && need_requeue) {
+	if (exit_enum == PK_EXIT_ENUM_KEY_REQUIRED && need_requeue) {
 		egg_debug ("key now installed");
 		return;
 	}
 
 	/* have we failed to install, and the eula key is now installed */
-	if (exit == PK_EXIT_ENUM_EULA_REQUIRED && need_requeue) {
+	if (exit_enum == PK_EXIT_ENUM_EULA_REQUIRED && need_requeue) {
 		egg_debug ("eula now agreed");
 		return;
 	}
@@ -703,7 +814,7 @@ pk_console_remove_packages (PkClient *client, gchar **packages, GError **error)
 	const PkPackageObj *obj;
 	guint i;
 	guint length;
-	gboolean remove;
+	gboolean remove_deps;
 	GPtrArray *array;
 	gchar **package_ids = NULL;
 	PkPackageList *list;
@@ -797,10 +908,10 @@ pk_console_remove_packages (PkClient *client, gchar **packages, GError **error)
 	}
 
 	/* TRANSLATORS: We are checking if it's okay to remove a list of packages */
-	remove = pk_console_get_prompt (_("Proceed removing additional packages?"), FALSE);
+	remove_deps = pk_console_get_prompt (_("Proceed removing additional packages?"), FALSE);
 
 	/* we chickened out */
-	if (!remove) {
+	if (!remove_deps) {
 		/* TRANSLATORS: We did not remove any packages */
 		g_print ("%s\n", _("The package removal was canceled!"));
 		ret = FALSE;
@@ -1225,8 +1336,9 @@ pk_console_list_install (PkClient *client, const gchar *file, GError **error)
 	g_print ("%s:\n", _("To install"));
 	for (i=0; i<length; i++) {
 		obj = pk_package_list_get_obj (new, i);
-		g_print ("%i\t%s\n", i+1, obj->id->name);
+		g_print ("%s ", obj->id->name);
 	}
+	g_print ("\n");
 
 	/* resolve */
 	filters = pk_bitfield_from_enums (PK_FILTER_ENUM_NOT_INSTALLED, PK_FILTER_ENUM_NEWEST, -1);
@@ -1234,12 +1346,12 @@ pk_console_list_install (PkClient *client, const gchar *file, GError **error)
 		obj = pk_package_list_get_obj (new, i);
 		/* TRANSLATORS: searching takes some time.... */
 		g_print ("%.0f%%\t%s %s...", (100.0f/length)*i, _("Searching for package: "), obj->id->name);
-		package_id = pk_console_perhaps_resolve (client, filters, obj->id->name, &error_local);
+		package_id = pk_console_perhaps_resolve (client, filters, obj->id->name, NULL);
 		if (package_id == NULL) {
 			/* TRANSLATORS: package was not found -- this is the end of a string ended in ... */
 			g_print (" %s\n", _("not found."));
 		} else {
-			g_print (" %s\n", package_id);
+			g_print (" %s\n", obj->id->version);
 			g_ptr_array_add (array, package_id);
 			/* no need to free */
 		}
@@ -1287,7 +1399,7 @@ pk_console_get_update_detail (PkClient *client, const gchar *package, GError **e
 	gchar **package_ids;
 	GError *error_local = NULL;
 
-	package_id = pk_console_perhaps_resolve (client, pk_bitfield_value (PK_FILTER_ENUM_INSTALLED), package, &error_local);
+	package_id = pk_console_perhaps_resolve (client, pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), package, &error_local);
 	if (package_id == NULL) {
 		/* TRANSLATORS: The package name was not found in any software sources. The detailed error follows */
 		*error = g_error_new (1, 0, _("This tool could not find the update details for %s: %s"), package, error_local->message);
@@ -1344,7 +1456,7 @@ pk_console_error_code_cb (PkClient *client, PkErrorCodeEnum error_code, const gc
 	if (awaiting_space)
 		g_print ("\n");
 	/* TRANSLATORS: This was an unhandled error, and we don't have _any_ context */
-	g_print ("%s %s : %s\n", _("Error:"), pk_error_enum_to_text (error_code), details);
+	g_print ("%s %s: %s\n", _("Error:"), pk_error_enum_to_text (error_code), details);
 }
 
 /**
@@ -1367,6 +1479,16 @@ pk_console_details_cb (PkClient *client, const PkDetailsObj *details, gpointer d
 	g_print ("  url:         %s\n", details->url);
 }
 
+/**
+ * pk_watch_message_cb:
+ **/
+static void
+pk_watch_message_cb (PkClient *client, PkMessageEnum message, const gchar *details, gpointer data)
+{
+	/* TRANSLATORS: This a message (like a little note that may be of interest) from the transaction */
+	g_print ("%s %s: %s\n", _("Message:"), pk_message_enum_to_text (message), details);
+}
+
 /**
  * pk_console_files_cb:
  **/
@@ -1376,6 +1498,7 @@ pk_console_files_cb (PkClient *client, const gchar *package_id,
 {
 	PkRoleEnum role;
 	gchar **filevector;
+	gchar **current_file;
 
 	/* don't print if we are DownloadPackages */
 	pk_client_get_role (client, &role, NULL, NULL);
@@ -1392,8 +1515,7 @@ pk_console_files_cb (PkClient *client, const gchar *package_id,
 	if (*filevector != NULL) {
 		/* TRANSLATORS: This a list files contained in the package */
 		g_print ("%s\n", _("Package files"));
-		gchar **current_file = filevector;
-
+		current_file = filevector;
 		while (*current_file != NULL) {
 			g_print ("  %s\n", *current_file);
 			current_file++;
@@ -1558,13 +1680,15 @@ pk_console_sigint_handler (int sig)
  * pk_console_get_summary:
  **/
 static gchar *
-pk_console_get_summary (PkBitfield roles)
+pk_console_get_summary (void)
 {
 	GString *string;
 	string = g_string_new ("");
 
 	/* TRANSLATORS: This is the header to the --help menu */
-	g_string_append_printf (string, "%s\n\n%s\n", _("PackageKit Console Interface"), _("Subcommands:"));
+	g_string_append_printf (string, "%s\n\n%s\n", _("PackageKit Console Interface"),
+				/* these are commands we can use with pkcon */
+				_("Subcommands:"));
 
 	/* always */
 	g_string_append_printf (string, "  %s\n", "get-actions");
@@ -1610,6 +1734,8 @@ pk_console_get_summary (PkBitfield roles)
 		g_string_append_printf (string, "  %s\n", "get-update-detail [package]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_PACKAGES))
 		g_string_append_printf (string, "  %s\n", "get-packages");
+	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_ROLLBACK))
+		g_string_append_printf (string, "  %s\n", "rollback");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_REPO_LIST))
 		g_string_append_printf (string, "  %s\n", "repo-list");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REPO_ENABLE))
@@ -1641,25 +1767,28 @@ main (int argc, char *argv[])
 	gchar *options_help;
 	gchar *filter = NULL;
 	gchar *summary;
-	gboolean ret;
+	gboolean ret = FALSE;
 	const gchar *mode;
 	const gchar *value = NULL;
 	const gchar *details = NULL;
 	const gchar *parameter = NULL;
 	PkBitfield groups;
 	gchar *text;
-	ret = FALSE;
 	gboolean maybe_sync = TRUE;
 	PkBitfield filters = 0;
 
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
+			/* TRANSLATORS: command line argument, if we should show debugging information */
 			_("Show extra debugging information"), NULL },
 		{ "version", '\0', 0, G_OPTION_ARG_NONE, &program_version,
+			/* TRANSLATORS: command line argument, just show the version string */
 			_("Show the program version and exit"), NULL},
 		{ "filter", '\0', 0, G_OPTION_ARG_STRING, &filter,
+			/* TRANSLATORS: command line argument, use a filter to narrow down results */
 			_("Set the filter, e.g. installed"), NULL},
 		{ "nowait", 'n', 0, G_OPTION_ARG_NONE, &nowait,
+			/* TRANSLATORS: command line argument, work asynchronously */
 			_("Exit without waiting for actions to complete"), NULL},
 		{ NULL}
 	};
@@ -1693,7 +1822,7 @@ main (int argc, char *argv[])
 	/* we need the roles early, as we only show the user only what they can do */
 	control = pk_control_new ();
 	roles = pk_control_get_actions (control, NULL);
-	summary = pk_console_get_summary (roles);
+	summary = pk_console_get_summary ();
 
 	context = g_option_context_new ("PackageKit Console Program");
 	g_option_context_set_summary (context, summary) ;
@@ -1748,14 +1877,24 @@ main (int argc, char *argv[])
 			  G_CALLBACK (pk_console_progress_changed_cb), NULL);
 	g_signal_connect (client_async, "finished",
 			  G_CALLBACK (pk_console_finished_cb), NULL);
+	g_signal_connect (client_async, "destroy",
+			  G_CALLBACK (pk_console_destroy_cb), NULL);
+	g_signal_connect (client_async, "require-restart",
+			  G_CALLBACK (pk_console_require_restart_cb), NULL);
 	g_signal_connect (client_async, "error-code",
 			  G_CALLBACK (pk_console_error_code_cb), NULL);
+	g_signal_connect (client_async, "message",
+			  G_CALLBACK (pk_watch_message_cb), NULL);
 
 	client_task = pk_client_new ();
 	pk_client_set_use_buffer (client_task, TRUE, NULL);
 	pk_client_set_synchronous (client_task, TRUE, NULL);
 	g_signal_connect (client_task, "finished",
 			  G_CALLBACK (pk_console_finished_cb), NULL);
+	g_signal_connect (client_task, "message",
+			  G_CALLBACK (pk_watch_message_cb), NULL);
+	g_signal_connect (client_task, "destroy",
+			  G_CALLBACK (pk_console_destroy_cb), NULL);
 
 	client_install_files = pk_client_new ();
 	g_signal_connect (client_install_files, "finished",
@@ -1789,79 +1928,99 @@ main (int argc, char *argv[])
 	/* parse the big list */
 	if (strcmp (mode, "search") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a search type, e.g. name"));
+			/* TRANSLATORS: a search type can be name, details, file, etc */
+			error = g_error_new (1, 0, "%s", _("A search type is required, e.g. name"));
 			goto out;
 
 		} else if (strcmp (value, "name") == 0) {
 			if (details == NULL) {
-				error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+				/* TRANSLATORS: the user needs to provide a search term */
+				error = g_error_new (1, 0, "%s", _("A search term is required"));
 				goto out;
 			}
 			ret = pk_client_search_name (client_async, filters, details, &error);
 
 		} else if (strcmp (value, "details") == 0) {
 			if (details == NULL) {
-				error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+				/* TRANSLATORS: the user needs to provide a search term */
+				error = g_error_new (1, 0, "%s", _("A search term is required"));
 				goto out;
 			}
 			ret = pk_client_search_details (client_async, filters, details, &error);
 
 		} else if (strcmp (value, "group") == 0) {
 			if (details == NULL) {
-				error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+				/* TRANSLATORS: the user needs to provide a search term */
+				error = g_error_new (1, 0, "%s", _("A search term is required"));
 				goto out;
 			}
 			ret = pk_client_search_group (client_async, filters, details, &error);
 
 		} else if (strcmp (value, "file") == 0) {
 			if (details == NULL) {
-				error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+				/* TRANSLATORS: the user needs to provide a search term */
+				error = g_error_new (1, 0, "%s", _("A search term is required"));
 				goto out;
 			}
 			ret = pk_client_search_file (client_async, filters, details, &error);
 		} else {
+			/* TRANSLATORS: the search type was provided, but invalid */
 			error = g_error_new (1, 0, "%s", _("Invalid search type"));
 		}
 
 	} else if (strcmp (mode, "install") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a package or file to install"));
+			/* TRANSLATORS: the user did not specify what they wanted to install */
+			error = g_error_new (1, 0, "%s", _("A package name or filename to install is required"));
 			goto out;
 		}
 		ret = pk_console_install_stuff (client_async, argv, &error);
 
 	} else if (strcmp (mode, "install-sig") == 0) {
 		if (value == NULL || details == NULL || parameter == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a type, key_id and package_id"));
+			/* TRANSLATORS: geeky error, 99.9999% of users won't see this */
+			error = g_error_new (1, 0, "%s", _("A type, key_id and package_id are required"));
 			goto out;
 		}
 		ret = pk_client_install_signature (client_async, PK_SIGTYPE_ENUM_GPG, details, parameter, &error);
 
 	} else if (strcmp (mode, "remove") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a package to remove"));
+			/* TRANSLATORS: the user did not specify what they wanted to remove */
+			error = g_error_new (1, 0, "%s", _("A package name to remove is required"));
 			goto out;
 		}
 		ret = pk_console_remove_packages (client_async, argv, &error);
 	} else if (strcmp (mode, "download") == 0) {
 		if (value == NULL || details == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify the destination directory and then the packages to download"));
+			/* TRANSLATORS: the user did not specify anything about what to download or where */
+			error = g_error_new (1, 0, "%s", _("A destination directory and then the package names to download are required"));
 			goto out;
 		}
 		ret = g_file_test (value, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
 		if (!ret) {
+			/* TRANSLATORS: the directory does not exist, so we can't continue */
 			error = g_error_new (1, 0, "%s: %s", _("Directory not found"), value);
 			goto out;
 		}
 		ret = pk_console_download_packages (client_async, argv, value, &error);
 	} else if (strcmp (mode, "accept-eula") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a licence identifier (eula-id)"));
+			/* TRANSLATORS: geeky error, 99.9999% of users won't see this */
+			error = g_error_new (1, 0, "%s", _("A licence identifier (eula-id) is required"));
 			goto out;
 		}
 		ret = pk_client_accept_eula (client_async, value, &error);
 		maybe_sync = FALSE;
 
+	} else if (strcmp (mode, "rollback") == 0) {
+		if (value == NULL) {
+			/* TRANSLATORS: geeky error, 99.9999% of users won't see this */
+			error = g_error_new (1, 0, "%s", _("A transaction identifier (tid) is required"));
+			goto out;
+		}
+		ret = pk_client_rollback (client_async, value, &error);
+
 	} else if (strcmp (mode, "update") == 0) {
 		if (value == NULL) {
 			/* do the system update */
@@ -1872,28 +2031,32 @@ main (int argc, char *argv[])
 
 	} else if (strcmp (mode, "resolve") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a package name to resolve"));
+			/* TRANSLATORS: The user did not specify a package name */
+			error = g_error_new (1, 0, "%s", _("A package name to resolve is required"));
 			goto out;
 		}
 		ret = pk_client_resolve (client_async, filters, argv+2, &error);
 
 	} else if (strcmp (mode, "repo-enable") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a repository name"));
+			/* TRANSLATORS: The user did not specify a repository (software source) name */
+			error = g_error_new (1, 0, "%s", _("A repository name is required"));
 			goto out;
 		}
 		ret = pk_client_repo_enable (client_async, value, TRUE, &error);
 
 	} else if (strcmp (mode, "repo-disable") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a repository name"));
+			/* TRANSLATORS: The user did not specify a repository (software source) name */
+			error = g_error_new (1, 0, "%s", _("A repository name is required"));
 			goto out;
 		}
 		ret = pk_client_repo_enable (client_async, value, FALSE, &error);
 
 	} else if (strcmp (mode, "repo-set-data") == 0) {
 		if (value == NULL || details == NULL || parameter == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a repo name/parameter and value"));
+			/* TRANSLATORS: The user didn't provide any data */
+			error = g_error_new (1, 0, "%s", _("A repo name, parameter and value are required"));
 			goto out;
 		}
 		ret = pk_client_repo_set_data (client_async, value, details, parameter, &error);
@@ -1903,28 +2066,31 @@ main (int argc, char *argv[])
 
 	} else if (strcmp (mode, "get-time") == 0) {
 		PkRoleEnum role;
-		guint time;
-		gboolean ret;
+		guint time_ms;
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify an action, e.g. 'update-system'"));
+			/* TRANSLATORS: The user didn't specify what action to use */
+			error = g_error_new (1, 0, "%s", _("An action, e.g. 'update-system' is required"));
 			goto out;
 		}
 		role = pk_role_enum_from_text (value);
 		if (role == PK_ROLE_ENUM_UNKNOWN) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a correct role"));
+			/* TRANSLATORS: The user specified an invalid action */
+			error = g_error_new (1, 0, "%s", _("A correct role is required"));
 			goto out;
 		}
-		ret = pk_control_get_time_since_action (control, role, &time, &error);
+		ret = pk_control_get_time_since_action (control, role, &time_ms, &error);
 		if (!ret) {
-			error = g_error_new (1, 0, "%s", _("Failed to get last time"));
+			/* TRANSLATORS: we keep a database updated with the time that an action was last executed */
+			error = g_error_new (1, 0, "%s", _("Failed to get the time since this action was last completed"));
 			goto out;
 		}
-		g_print ("time since %s is %is\n", value, time);
+		g_print ("time since %s is %is\n", value, time_ms);
 		maybe_sync = FALSE;
 
 	} else if (strcmp (mode, "get-depends") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+			/* TRANSLATORS: The user did not provide a package name */
+			error = g_error_new (1, 0, "%s", _("A package name is required"));
 			goto out;
 		}
 		ret = pk_console_get_depends (client_async, filters, value, &error);
@@ -1934,42 +2100,48 @@ main (int argc, char *argv[])
 
 	} else if (strcmp (mode, "get-update-detail") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+			/* TRANSLATORS: The user did not provide a package name */
+			error = g_error_new (1, 0, "%s", _("A package name is required"));
 			goto out;
 		}
 		ret = pk_console_get_update_detail (client_async, value, &error);
 
 	} else if (strcmp (mode, "get-requires") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+			/* TRANSLATORS: The user did not provide a package name */
+			error = g_error_new (1, 0, "%s", _("A package name is required"));
 			goto out;
 		}
 		ret = pk_console_get_requires (client_async, filters, value, &error);
 
 	} else if (strcmp (mode, "what-provides") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a search term"));
+			/* TRANSLATORS: each package "provides" certain things, e.g. mime(gstreamer-decoder-mp3), the user didn't specify it */
+			error = g_error_new (1, 0, "%s", _("A package provide string is required"));
 			goto out;
 		}
 		ret = pk_client_what_provides (client_async, filters, PK_PROVIDES_ENUM_CODEC, value, &error);
 
 	} else if (strcmp (mode, "get-details") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a package to find the details for"));
+			/* TRANSLATORS: The user did not provide a package name */
+			error = g_error_new (1, 0, "%s", _("A package name is required"));
 			goto out;
 		}
 		ret = pk_console_get_details (client_async, value, &error);
 
 	} else if (strcmp (mode, "get-files") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a package to find the files for"));
+			/* TRANSLATORS: The user did not provide a package name */
+			error = g_error_new (1, 0, "%s", _("A package name is required"));
 			goto out;
 		}
 		ret = pk_console_get_files (client_async, value, &error);
 
 	} else if (strcmp (mode, "list-create") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a list file to create"));
+			/* TRANSLATORS: The user didn't specify a filename to create as a list */
+			error = g_error_new (1, 0, "%s", _("A list file name to create is required"));
 			goto out;
 		}
 		ret = pk_console_list_create (client_async, value, &error);
@@ -1977,7 +2149,8 @@ main (int argc, char *argv[])
 
 	} else if (strcmp (mode, "list-diff") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a list file to open"));
+			/* TRANSLATORS: The user didn't specify a filename to open as a list */
+			error = g_error_new (1, 0, "%s", _("A list file to open is required"));
 			goto out;
 		}
 		ret = pk_console_list_diff (client_async, value, &error);
@@ -1985,7 +2158,8 @@ main (int argc, char *argv[])
 
 	} else if (strcmp (mode, "list-install") == 0) {
 		if (value == NULL) {
-			error = g_error_new (1, 0, "%s", _("You need to specify a list file to open"));
+			/* TRANSLATORS: The user didn't specify a filename to open as a list */
+			error = g_error_new (1, 0, "%s", _("A list file to open is required"));
 			goto out;
 		}
 		ret = pk_console_list_install (client_async, value, &error);
@@ -2051,7 +2225,7 @@ out:
 			g_print ("Command failed in an unknown way. Please report!\n");
 		} else if (g_str_has_prefix (error->message, "org.freedesktop.packagekit."))  {
 			/* TRANSLATORS: User does not have permission to do this */
-			g_print ("%s\n", _("You don't have the necessary privileges for this operation"));
+			g_print ("%s\n", _("Incorrect privileges for this operation"));
 		} else {
 			/* TRANSLATORS: Generic failure of what they asked to do */
 			g_print ("%s:  %s\n", _("Command failed"), error->message);
