@@ -74,10 +74,12 @@ pk_get_machine_type (void)
 gchar *
 pk_get_distro_id (void)
 {
+	guint i;
 	gboolean ret;
 	gchar *contents = NULL;
 	gchar *distro = NULL;
 	gchar *arch = NULL;
+	gchar *version = NULL;
 	gchar **split = NULL;
 
 	/* check for fedora */
@@ -145,8 +147,57 @@ pk_get_distro_id (void)
 		goto out;
 	}
 
+	/* check for Arch */
+	ret = g_file_test ("/etc/arch-release", G_FILE_TEST_EXISTS);
+	if (ret) {
+		/* we can't get arch from /etc */
+		arch = pk_get_machine_type ();
+		if (arch == NULL)
+			goto out;
+
+		/* complete! */
+		distro = g_strdup_printf ("arch-current-%s", arch);
+		goto out;
+	}
+
+	/* check for LSB */
+	ret = g_file_get_contents ("/etc/lsb-release", &contents, NULL, NULL);
+	if (ret) {
+		/* we can't get arch from /etc */
+		arch = pk_get_machine_type ();
+		if (arch == NULL)
+			goto out;
+
+		/* split by lines */
+		split = g_strsplit (contents, "\n", -1);
+		for (i=0; split[i] != NULL; i++) {
+			if (g_str_has_prefix (split[i], "DISTRIB_ID="))
+				distro = g_ascii_strdown (&split[i][11], -1);
+			if (g_str_has_prefix (split[i], "DISTRIB_RELEASE="))
+				version = g_ascii_strdown (&split[i][16], -1);
+		}
+
+		/* complete! */
+		distro = g_strdup_printf ("%s-%s-%s", distro, version, arch);
+		goto out;
+	}
+
+	/* check for Debian or Debian derivatives */
+	ret = g_file_get_contents ("/etc/debian_version", &contents, NULL, NULL);
+	if (ret) {
+		/* remove "\n": "squeeze/sid\n" */
+		g_strdelimit (contents, "\n", '\0');
+		/* removes leading and trailing whitespace */
+		g_strstrip (contents);
+
+		/* complete! */
+		distro = g_strdup_printf ("debian-(%s)", contents);
+		goto out;
+	}
+
 out:
 	g_strfreev (split);
+	g_free (version);
 	g_free (arch);
 	g_free (contents);
 	return distro;
@@ -182,7 +233,7 @@ pk_iso8601_difference (const gchar *isodate)
 	GTimeVal timeval_then;
 	GTimeVal timeval_now;
 	gboolean ret;
-	guint time;
+	guint time_s;
 
 	g_return_val_if_fail (isodate != NULL, 0);
 
@@ -195,9 +246,9 @@ pk_iso8601_difference (const gchar *isodate)
 	g_get_current_time (&timeval_now);
 
 	/* work out difference */
-	time = timeval_now.tv_sec - timeval_then.tv_sec;
+	time_s = timeval_now.tv_sec - timeval_then.tv_sec;
 
-	return time;
+	return time_s;
 }
 
 /**
@@ -231,30 +282,41 @@ pk_iso8601_to_date (const gchar *iso_date)
 {
 	gboolean ret;
 	guint retval;
-	guint d, m, y;
-	GTimeVal time;
+	guint d = 0;
+	guint m = 0;
+	guint y = 0;
+	GTimeVal time_val;
 	GDate *date = NULL;
 
 	if (egg_strzero (iso_date))
 		goto out;
 
 	/* try to parse complete ISO8601 date */
-	ret = g_time_val_from_iso8601 (iso_date, &time);
+	ret = g_time_val_from_iso8601 (iso_date, &time_val);
 	if (ret) {
 		date = g_date_new ();
-		g_date_set_time_val (date, &time);
+		g_date_set_time_val (date, &time_val);
 		goto out;
 	}
 
 	/* g_time_val_from_iso8601() blows goats and won't
 	 * accept a valid ISO8601 formatted date without a
 	 * time value - try and parse this case */
-	retval = sscanf (iso_date, "%i-%i-%i", &y, &m, &d);
-	if (retval == 3) {
-		date = g_date_new_dmy (d, m, y);
+	retval = sscanf (iso_date, "%u-%u-%u", &y, &m, &d);
+	if (retval != 3) {
+		egg_warning ("could not parse '%s'", iso_date);
 		goto out;
 	}
-	egg_warning ("could not parse '%s'", iso_date);
+
+	/* check it's valid */
+	ret = g_date_valid_dmy (d, m, y);
+	if (!ret) {
+		egg_warning ("invalid date %i/%i/%i from '%s'", y, m, d, iso_date);
+		goto out;
+	}
+
+	/* create valid object */
+	date = g_date_new_dmy (d, m, y);
 out:
 	return date;
 }
@@ -293,7 +355,7 @@ pk_strvalidate_char (gchar item)
  * @text: The input text to make safe
  *
  * Replaces chars in the text that may be dangerous, or that may print
- * incorrectly. These chars include new lines, tabs and quotes, and are
+ * incorrectly. These chars include new lines, tabs and line feed, and are
  * replaced by spaces.
  *
  * Return value: the new string with no insane chars
@@ -448,52 +510,6 @@ pk_va_list_to_argv (const gchar *string_first, va_list *args)
 	return array;
 }
 
-/**
- * pk_strv_to_text:
- * @array: a string array of package_id's
- *
- * Cats the string array of package_id's into one tab delimited string
- *
- * Return value: a string representation of all the package_id's.
- **/
-gchar *
-pk_strv_to_text (gchar **array, const gchar *delimiter)
-{
-	guint i;
-	guint size;
-	GString *string;
-	gchar *string_ret;
-
-	g_return_val_if_fail (array != NULL, NULL);
-	g_return_val_if_fail (delimiter != NULL, NULL);
-
-	string = g_string_new ("");
-
-	/* print all */
-	size = g_strv_length (array);
-
-	/* shortcut */
-	if (size == 1)
-		return g_strdup (array[0]);
-
-	/* append with delimiter */
-	for (i=0; i<size; i++) {
-		g_string_append (string, array[i]);
-		g_string_append (string, delimiter);
-	}
-
-	/* ITS4: ignore, we check this for validity */
-	size = strlen (delimiter);
-
-	/* remove trailing delimiter */
-	if (string->len > size)
-		g_string_set_size (string, string->len-size);
-
-	string_ret = g_string_free (string, FALSE);
-
-	return string_ret;
-}
-
 /***************************************************************************
  ***                          MAKE CHECK TESTS                           ***
  ***************************************************************************/
@@ -519,10 +535,10 @@ pk_common_test (EggTest *test)
 {
 	gboolean ret;
 	gchar **array;
-	gchar *text;
 	gchar *text_safe;
 	gchar *present;
 	guint seconds;
+	GDate *date;
 
 	if (!egg_test_start (test, "PkCommon"))
 		return;
@@ -562,17 +578,6 @@ pk_common_test (EggTest *test)
 		egg_test_failed (test, "incorrect array '%s','%s','%s'", array[0], array[1], array[2]);
 	g_strfreev (array);
 
-	/************************************************************/
-	egg_test_title (test, "to text");
-	array = pk_va_list_to_argv_test ("richard", "phillip", "hughes", NULL);
-	text = pk_strv_to_text (array, "\t");
-	if (egg_strequal (text, "richard\tphillip\thughes"))
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, NULL);
-	g_free (text);
-	g_strfreev (array);
-
 	/************************************************************
 	 ****************        validate text         **************
 	 ************************************************************/
@@ -661,6 +666,31 @@ pk_common_test (EggTest *test)
 	/************************************************************/
 	g_free (present);
 
+
+	/************************************************************
+	 **************        Date handling         ****************
+	 ************************************************************/
+	egg_test_title (test, "zero length date");
+	date = pk_iso8601_to_date ("");
+	egg_test_assert (test, (date == NULL));
+
+	/************************************************************/
+	egg_test_title (test, "no day specified");
+	date = pk_iso8601_to_date ("2004-01");
+	egg_test_assert (test, (date == NULL));
+
+	/************************************************************/
+	egg_test_title (test, "date _and_ time specified");
+	date = pk_iso8601_to_date ("2009-05-08 13:11:12");
+	egg_test_assert (test, (date->day == 8 && date->month == 5 && date->year == 2009));
+	g_date_free (date);
+
+	/************************************************************/
+	egg_test_title (test, "correct date format");
+	date = pk_iso8601_to_date ("2004-02-01");
+	egg_test_assert (test, (date->day == 1 && date->month == 2 && date->year == 2004));
+	g_date_free (date);
+
 	egg_test_end (test);
 }
 #endif
