@@ -47,11 +47,10 @@
 #include <packagekit-glib/pk-connection.h>
 #include <packagekit-glib/pk-common.h>
 #include <packagekit-glib/pk-enum.h>
+#include <packagekit-glib/pk-version.h>
 
 #include "egg-debug.h"
 
-static void     pk_control_class_init	(PkControlClass *klass);
-static void     pk_control_init		(PkControl      *control);
 static void     pk_control_finalize	(GObject     *object);
 
 #define PK_CONTROL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_CONTROL, PkControlPrivate))
@@ -68,6 +67,9 @@ struct _PkControlPrivate
 	PkConnection		*pconnection;
 	gchar			**array;
 	guint			 idle_id;
+	gboolean		 version_major;
+	gboolean		 version_minor;
+	gboolean		 version_micro;
 };
 
 enum {
@@ -80,6 +82,14 @@ enum {
 	PK_CONTROL_LAST_SIGNAL
 };
 
+enum {
+	PROP_0,
+	PROP_VERSION_MAJOR,
+	PROP_VERSION_MINOR,
+	PROP_VERSION_MICRO,
+	PROP_LAST
+};
+
 static guint signals [PK_CONTROL_LAST_SIGNAL] = { 0 };
 static gpointer pk_control_object = NULL;
 
@@ -316,6 +326,45 @@ out:
 	return types;
 }
 
+/**
+ * pk_control_get_daemon_state:
+ * @control: a valid #PkControl instance
+ * @error: a %GError to put the error code and message in, or %NULL
+ *
+ * The engine state debugging output
+ *
+ * Return value: a string of debugging data of unspecified format
+ **/
+gchar *
+pk_control_get_daemon_state (PkControl *control, GError **error)
+{
+	gboolean ret;
+	GError *error_local = NULL;
+	gchar *state = NULL;
+
+	g_return_val_if_fail (PK_IS_CONTROL (control), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* check to see if we have a valid proxy */
+	if (control->priv->proxy == NULL) {
+		egg_warning ("No proxy for manager");
+		goto out;
+	}
+	ret = dbus_g_proxy_call (control->priv->proxy, "GetDaemonState", &error_local,
+				 G_TYPE_INVALID,
+				 G_TYPE_STRING, &state,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		/* abort as the DBUS method failed */
+		egg_warning ("GetDaemonState failed :%s", error_local->message);
+		pk_control_error_set (error, PK_CONTROL_ERROR_FAILED, error_local->message);
+		g_error_free (error_local);
+		goto out;
+	}
+out:
+	return state;
+}
+
 /**
  * pk_control_get_network_state:
  * @control: a valid #PkControl instance
@@ -446,7 +495,7 @@ pk_control_get_backend_detail (PkControl *control, gchar **name, gchar **author,
 	if (name != NULL)
 		*name = tname;
 	else
-		g_free (tauthor);
+		g_free (tname);
 	/* copy needed bits */
 	if (author != NULL)
 		*author = tauthor;
@@ -716,7 +765,7 @@ pk_control_transaction_list_changed_cb (DBusGProxy *proxy, gchar **array, PkCont
 		g_strfreev (control->priv->array);
 	control->priv->array = g_strdupv (array);
 	egg_debug ("emit transaction-list-changed");
-	g_signal_emit (control , signals [PK_CONTROL_LIST_CHANGED], 0);
+	g_signal_emit (control, signals [PK_CONTROL_LIST_CHANGED], 0);
 }
 
 /**
@@ -792,6 +841,42 @@ pk_control_locked_cb (DBusGProxy *proxy, gboolean is_locked, PkControl *control)
 	g_signal_emit (control , signals [PK_CONTROL_LOCKED], 0, is_locked);
 }
 
+/**
+ * pk_control_get_property:
+ **/
+static void
+pk_control_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	PkControl *control = PK_CONTROL (object);
+	switch (prop_id) {
+	case PROP_VERSION_MAJOR:
+		g_value_set_uint (value, control->priv->version_major);
+		break;
+	case PROP_VERSION_MINOR:
+		g_value_set_uint (value, control->priv->version_minor);
+		break;
+	case PROP_VERSION_MICRO:
+		g_value_set_uint (value, control->priv->version_micro);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+/**
+ * pk_control_set_property:
+ **/
+static void
+pk_control_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 /**
  * pk_control_class_init:
  * @klass: The PkControlClass
@@ -799,9 +884,36 @@ pk_control_locked_cb (DBusGProxy *proxy, gboolean is_locked, PkControl *control)
 static void
 pk_control_class_init (PkControlClass *klass)
 {
+	GParamSpec *pspec;
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	object_class->get_property = pk_control_get_property;
+	object_class->set_property = pk_control_set_property;
 	object_class->finalize = pk_control_finalize;
 
+	/**
+	 * PkControl:version-major:
+	 */
+	pspec = g_param_spec_uint ("version-major", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MAJOR, pspec);
+
+	/**
+	 * PkControl:version-minor:
+	 */
+	pspec = g_param_spec_uint ("version-minor", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MINOR, pspec);
+
+	/**
+	 * PkControl:version-micro:
+	 */
+	pspec = g_param_spec_uint ("version-micro", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MICRO, pspec);
+
 	/**
 	 * PkControl::updates-changed:
 	 * @control: the #PkControl instance that emitted the signal
@@ -842,11 +954,13 @@ pk_control_class_init (PkControlClass *klass)
 			      NULL, NULL, g_cclosure_marshal_VOID__UINT,
 			      G_TYPE_NONE, 1, G_TYPE_UINT);
 	/**
-	 * PkControl::restart_schedule:
+	 * PkControl::restart-schedule:
 	 * @control: the #PkControl instance that emitted the signal
 	 *
-	 * The ::restart_schedule signal is emitted when the service has been
-	 * restarted. Client programs should reload themselves.
+	 * The ::restart_schedule signal is emitted when the packagekitd service
+	 * has been restarted because it has been upgraded.
+	 * Client programs should reload themselves when it is convenient to
+	 * do so, as old client tools may not be compatable with the new daemon.
 	 **/
 	signals [PK_CONTROL_RESTART_SCHEDULE] =
 		g_signal_new ("restart-schedule",
@@ -885,6 +999,63 @@ pk_control_class_init (PkControlClass *klass)
 	g_type_class_add_private (klass, sizeof (PkControlPrivate));
 }
 
+/**
+ * pk_control_get_properties:
+ **/
+static GHashTable *
+pk_control_get_properties (PkControl *control)
+{
+	gboolean ret;
+	GError *error = NULL;
+	GHashTable *hash_table = NULL;
+	DBusGProxy *proxy;
+
+	/* connect to the correct path for properties */
+	proxy = dbus_g_proxy_new_for_name (control->priv->connection,
+					   "org.freedesktop.PackageKit",
+					   "/org/freedesktop/PackageKit",
+					   "org.freedesktop.DBus.Properties");
+	if (proxy == NULL) {
+		egg_warning ("Couldn't connect to proxy");
+		goto out;
+	}
+
+	/* get all properties */
+	ret = dbus_g_proxy_call (proxy, "GetAll", &error,
+				 G_TYPE_STRING, "org.freedesktop.PackageKit",
+				 G_TYPE_INVALID,
+				 dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE),
+				 &hash_table,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		egg_warning ("Couldn't call GetAll() to get properties for %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	if (proxy != NULL)
+		g_object_unref (proxy);
+	return hash_table;
+}
+
+/**
+ * pk_control_collect_props_cb:
+ **/
+static void
+pk_control_collect_props_cb (const char *key, const GValue *value, PkControl *control)
+{
+	if (g_strcmp0 (key, "version-major") == 0)
+		control->priv->version_major = g_value_get_uint (value);
+	else if (g_strcmp0 (key, "version-minor") == 0)
+		control->priv->version_minor = g_value_get_uint (value);
+	else if (g_strcmp0 (key, "version-micro") == 0)
+		control->priv->version_micro = g_value_get_uint (value);
+	else {
+		egg_warning ("unhandled property '%s'", key);
+		g_assert_not_reached ();
+	}
+}
+
 /**
  * pk_control_init:
  * @control: This class instance
@@ -893,6 +1064,7 @@ static void
 pk_control_init (PkControl *control)
 {
 	GError *error = NULL;
+	GHashTable *hash;
 
 	control->priv = PK_CONTROL_GET_PRIVATE (control);
 	/* check dbus connections, exit if not valid */
@@ -906,6 +1078,9 @@ pk_control_init (PkControl *control)
 	/* we maintain a local copy */
 	control->priv->array = NULL;
 	control->priv->idle_id = 0;
+	control->priv->version_major = 0;
+	control->priv->version_minor = 0;
+	control->priv->version_micro = 0;
 
 	/* watch for PackageKit on the bus, and try to connect up at start */
 	control->priv->pconnection = pk_connection_new ();
@@ -949,6 +1124,13 @@ pk_control_init (PkControl *control)
 	dbus_g_proxy_connect_signal (control->priv->proxy, "Locked",
 				     G_CALLBACK (pk_control_locked_cb), control, NULL);
 
+	/* get properties if they exist */
+	hash = pk_control_get_properties (control);
+	if (hash != NULL) {
+		g_hash_table_foreach (hash, (GHFunc) pk_control_collect_props_cb, control);
+		g_hash_table_unref (hash);
+	}
+
 	/* idle add a refresh so we have valid data */
 	control->priv->idle_id = g_idle_add ((GSourceFunc) pk_control_transaction_list_refresh_idle_cb, control);
 }
@@ -1019,6 +1201,7 @@ pk_control_test (EggTest *test)
 	gboolean ret;
 	PkControl *control;
 	PkConnection *connection;
+	guint version;
 
 	if (!egg_test_start (test, "PkControl"))
 		return;
@@ -1036,6 +1219,22 @@ pk_control_test (EggTest *test)
 	egg_test_title (test, "get control");
 	control = pk_control_new ();
 	egg_test_assert (test, control != NULL);
+
+	/************************************************************/
+	egg_test_title (test, "version major");
+	g_object_get (control, "version-major", &version, NULL);
+	egg_test_assert (test, (version == PK_MAJOR_VERSION));
+
+	/************************************************************/
+	egg_test_title (test, "version minor");
+	g_object_get (control, "version-minor", &version, NULL);
+	egg_test_assert (test, (version == PK_MINOR_VERSION));
+
+	/************************************************************/
+	egg_test_title (test, "version micro");
+	g_object_get (control, "version-micro", &version, NULL);
+	egg_test_assert (test, (version == PK_MICRO_VERSION));
+
 	g_object_unref (control);
 out:
 	egg_test_end (test);
