@@ -47,6 +47,7 @@ static const PkEnumMatch enum_exit[] = {
 	{PK_EXIT_ENUM_CANCELLED,		"cancelled"},
 	{PK_EXIT_ENUM_KEY_REQUIRED,		"key-required"},
 	{PK_EXIT_ENUM_EULA_REQUIRED,		"eula-required"},
+	{PK_EXIT_ENUM_MEDIA_CHANGE_REQUIRED,	"media-change-required"},
 	{PK_EXIT_ENUM_KILLED,			"killed"},
 	{0, NULL}
 };
@@ -74,7 +75,7 @@ static const PkEnumMatch enum_status[] = {
 	{PK_STATUS_ENUM_FINISHED,		"finished"},
 	{PK_STATUS_ENUM_CANCEL,			"cancel"},
 	{PK_STATUS_ENUM_DOWNLOAD_REPOSITORY,	"download-repository"},
-	{PK_STATUS_ENUM_DOWNLOAD_PACKAGELIST,	"download-package"},
+	{PK_STATUS_ENUM_DOWNLOAD_PACKAGELIST,	"download-packagelist"},
 	{PK_STATUS_ENUM_DOWNLOAD_FILELIST,	"download-filelist"},
 	{PK_STATUS_ENUM_DOWNLOAD_CHANGELOG,	"download-changelog"},
 	{PK_STATUS_ENUM_DOWNLOAD_GROUP,		"download-group"},
@@ -83,6 +84,8 @@ static const PkEnumMatch enum_status[] = {
 	{PK_STATUS_ENUM_LOADING_CACHE,		"loading-cache"},
 	{PK_STATUS_ENUM_SCAN_APPLICATIONS,	"scan-applications"},
 	{PK_STATUS_ENUM_GENERATE_PACKAGE_LIST,	"generate-package-list"},
+	{PK_STATUS_ENUM_WAITING_FOR_LOCK,	"waiting-for-lock"},
+	{PK_STATUS_ENUM_WAITING_FOR_AUTH,	"waiting-for-auth"},
 	{0, NULL}
 };
 
@@ -160,13 +163,20 @@ static const PkEnumMatch enum_error[] = {
 	{PK_ERROR_ENUM_FILE_CONFLICTS,		"file-conflicts"},
 	{PK_ERROR_ENUM_PACKAGE_CONFLICTS,	"package-conflicts"},
 	{PK_ERROR_ENUM_REPO_NOT_AVAILABLE,	"repo-not-available"},
-	{PK_ERROR_ENUM_INVALID_PACKAGE_FILE,    "invalid-package-file"},
+	{PK_ERROR_ENUM_INVALID_PACKAGE_FILE,	"invalid-package-file"},
 	{PK_ERROR_ENUM_PACKAGE_INSTALL_BLOCKED, "package-install-blocked"},
-	{PK_ERROR_ENUM_PACKAGE_CORRUPT,         "package-corrupt"},
+	{PK_ERROR_ENUM_PACKAGE_CORRUPT,		"package-corrupt"},
 	{PK_ERROR_ENUM_ALL_PACKAGES_ALREADY_INSTALLED, "all-packages-already-installed"},
 	{PK_ERROR_ENUM_FILE_NOT_FOUND,		"file-not-found"},
 	{PK_ERROR_ENUM_NO_MORE_MIRRORS_TO_TRY,	"no-more-mirrors-to-try"},
 	{PK_ERROR_ENUM_NO_DISTRO_UPGRADE_DATA,	"no-distro-upgrade-data"},
+	{PK_ERROR_ENUM_INCOMPATIBLE_ARCHITECTURE,	"incompatible-architecture"},
+	{PK_ERROR_ENUM_NO_SPACE_ON_DEVICE,	"no-space-on-device"},
+	{PK_ERROR_ENUM_MEDIA_CHANGE_REQUIRED,	"media-change-required"},
+	{PK_ERROR_ENUM_NOT_AUTHORIZED,		"not-authorized"},
+	{PK_ERROR_ENUM_UPDATE_NOT_FOUND,	"update-not-found"},
+	{PK_ERROR_ENUM_CANNOT_INSTALL_REPO_UNSIGNED,	"cannot-install-repo-unsigned"},
+	{PK_ERROR_ENUM_CANNOT_UPDATE_REPO_UNSIGNED,	"cannot-update-repo-unsigned"},
 	{0, NULL}
 };
 
@@ -221,6 +231,8 @@ static const PkEnumMatch enum_filter[] = {
 	{PK_FILTER_ENUM_NOT_SOURCE,		"~source"},
 	{PK_FILTER_ENUM_COLLECTIONS,		"collections"},
 	{PK_FILTER_ENUM_NOT_COLLECTIONS,	"~collections"},
+	{PK_FILTER_ENUM_APPLICATION,		"application"},
+	{PK_FILTER_ENUM_NOT_APPLICATION,	"~application"},
 	{0, NULL}
 };
 
@@ -290,6 +302,7 @@ static const PkEnumMatch enum_info[] = {
 	{PK_INFO_ENUM_OBSOLETING,		"obsoleting"},
 	{PK_INFO_ENUM_COLLECTION_INSTALLED,	"collection-installed"},
 	{PK_INFO_ENUM_COLLECTION_AVAILABLE,	"collection-available"},
+	{PK_INFO_ENUM_FINISHED,			"finished"},
 	{0, NULL}
 };
 
@@ -321,8 +334,9 @@ static const PkEnumMatch enum_network[] = {
 	{PK_NETWORK_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
 	{PK_NETWORK_ENUM_OFFLINE,		"offline"},
 	{PK_NETWORK_ENUM_ONLINE,		"online"},
-	{PK_NETWORK_ENUM_SLOW,			"slow"},
-	{PK_NETWORK_ENUM_FAST,			"fast"},
+	{PK_NETWORK_ENUM_WIRED,			"wired"},
+	{PK_NETWORK_ENUM_WIFI,			"wifi"},
+	{PK_NETWORK_ENUM_MOBILE,		"mobile"},
 	{0, NULL}
 };
 
@@ -450,9 +464,17 @@ static const PkEnumMatch enum_free_licenses[] = {
 	{PK_LICENSE_ENUM_STIX,			"STIX"},
 	{PK_LICENSE_ENUM_XANO,			"XANO"},
 	{PK_LICENSE_ENUM_VOSTROM,		"VOSTROM"},
-	{PK_LICENSE_ENUM_XEROX,                 "Xerox License"},
-	{PK_LICENSE_ENUM_RICEBSD,               "RiceBSD"},
-	{PK_LICENSE_ENUM_QHULL,                 "Qhull"},
+	{PK_LICENSE_ENUM_XEROX,			"Xerox License"},
+	{PK_LICENSE_ENUM_RICEBSD,		"RiceBSD"},
+	{PK_LICENSE_ENUM_QHULL,			"Qhull"},
+	{0, NULL}
+};
+
+static const PkEnumMatch enum_media_type[] = {
+	{PK_MEDIA_TYPE_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
+	{PK_MEDIA_TYPE_ENUM_CD,			"cd"},
+	{PK_MEDIA_TYPE_ENUM_DVD,		"dvd"},
+	{PK_MEDIA_TYPE_ENUM_DISC,		"disc"},
 	{0, NULL}
 };
 
@@ -634,9 +656,9 @@ pk_info_enum_to_text (PkInfoEnum info)
  * Return value: the enumerated constant value, e.g. PK_SIGTYPE_ENUM_GPG
  */
 PkExitEnum
-pk_exit_enum_from_text (const gchar *exit)
+pk_exit_enum_from_text (const gchar *exit_text)
 {
-	return pk_enum_find_value (enum_exit, exit);
+	return pk_enum_find_value (enum_exit, exit_text);
 }
 
 /**
@@ -648,9 +670,9 @@ pk_exit_enum_from_text (const gchar *exit)
  * Return value: the enumerated constant value, e.g. "available"
  **/
 const gchar *
-pk_exit_enum_to_text (PkExitEnum exit)
+pk_exit_enum_to_text (PkExitEnum exit_enum)
 {
-	return pk_enum_find_string (enum_exit, exit);
+	return pk_enum_find_string (enum_exit, exit_enum);
 }
 
 /**
@@ -933,6 +955,32 @@ pk_license_enum_to_text (PkLicenseEnum license)
 	return pk_enum_find_string (enum_free_licenses, license);
 }
 
+/**
+ * pk_media_type_enum_from_text:
+ * @code: Text describing the enumerated type
+ *
+ * Converts a text enumerated type to its unsigned integer representation
+ *
+ * Return value: the enumerated constant value, e.g. PK_MEDIA_TYPE_ENUM_CD
+ **/
+PkMediaTypeEnum pk_media_type_enum_from_text (const gchar *media_type)
+{
+	return pk_enum_find_value (enum_media_type, media_type);
+}
+
+/**
+ * pk_media_type_enum_to_text:
+ * @code: The enumerated type value
+ *
+ * Converts a enumerated type to its text representation
+ *
+ * Return value: the enumerated constant value, e.g. "dvd"
+ **/
+const gchar* pk_media_type_enum_to_text (PkMediaTypeEnum media_type)
+{
+	return pk_enum_find_string (enum_media_type, media_type);
+}
+
 /***************************************************************************
  ***                          MAKE CHECK TESTS                           ***
  ***************************************************************************/
@@ -1102,6 +1150,17 @@ pk_enum_test (EggTest *test)
 	}
 	egg_test_success (test, NULL);
 
+	/************************************************************/
+	egg_test_title (test, "check we convert all the media type bitfield");
+	for (i=0; i<=PK_MEDIA_TYPE_ENUM_UNKNOWN; i++) {
+		string = pk_media_type_enum_to_text (i);
+		if (string == NULL) {
+			egg_test_failed (test, "failed to get %i", i);
+			break;
+		}
+	}
+	egg_test_success (test, NULL);
+
 	egg_test_end (test);
 }
 #endif
