@@ -260,7 +260,7 @@ static gboolean
 pk_service_pack_extract (const gchar *filename, const gchar *directory, GError **error)
 {
 	*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_EXTRACTION,
-			      "Cannot check PackageKit as not built with libarchive support");
+			      "The service pack %s cannot be extracted as PackageKit was not built with libarchive support", filename);
 	return FALSE;
 }
 #endif /* HAVE_ARCHIVE_H */
@@ -599,7 +599,7 @@ pk_service_pack_archive_add_file (struct archive *arch, const gchar *filename, G
 				      "file not found %s", filename);
 		goto out;
 	}
-	egg_debug ("stat(%s), size=%lu bytes\n", filename, st.st_size);
+	egg_debug ("stat(%s), size=%lu bytes\n", filename, (glong) st.st_size);
 
 	/* create new entry */
 	entry = archive_entry_new ();
@@ -667,9 +667,9 @@ pk_service_pack_create_from_files (PkServicePack *pack, GPtrArray *file_array, G
 	filename = g_build_filename (g_get_tmp_dir (), "metadata.conf", NULL);
 	ret = pk_service_pack_create_metadata_file (pack, filename);
 	if (!ret) {
-	        *error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
+		*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
 				      "failed to generate metadata file %s", filename);
-	        goto out;
+		goto out;
 	}
 	g_ptr_array_add (file_array, g_strdup (filename));
 
@@ -714,7 +714,7 @@ pk_service_pack_create_from_files (PkServicePack *pack, GPtrArray *file_array, G
 {
 	g_return_val_if_fail (PK_IS_SERVICE_PACK (pack), FALSE);
 	*error = g_error_new (PK_SERVICE_PACK_ERROR, PK_SERVICE_PACK_ERROR_FAILED_CREATE,
-			      "Cannot create pack as PackageKit as not built with libarchive support");
+			      "The service pack %s cannot be created as PackageKit was not built with libarchive support", pack->priv->filename);
 	return FALSE;
 }
 #endif
@@ -730,8 +730,8 @@ pk_service_pack_scan_files_in_directory (PkServicePack *pack)
 	GDir *dir;
 	const gchar *filename;
 
-	g_return_val_if_fail (PK_IS_SERVICE_PACK (pack), FALSE);
-	g_return_val_if_fail (pack->priv->directory != NULL, FALSE);
+	g_return_val_if_fail (PK_IS_SERVICE_PACK (pack), NULL);
+	g_return_val_if_fail (pack->priv->directory != NULL, NULL);
 
 	/* try and open the directory */
 	dir = g_dir_open (pack->priv->directory, 0, NULL);
