@@ -54,9 +54,12 @@ class Client : public QObject
 	Q_ENUMS(Group)
 	Q_ENUMS(NetworkState)
 	Q_ENUMS(SignatureType)
+	Q_ENUMS(ProvidesType)
 	Q_ENUMS(ErrorType)
+	Q_ENUMS(MessageType)
 	Q_ENUMS(RestartType)
-	Q_ENUMS(UpgradeType)
+	Q_ENUMS(UpdateState)
+	Q_ENUMS(DistroUpgradeType)
 
 public:
 	/**
@@ -79,39 +82,39 @@ public:
 	 * \sa getActions
 	 */
 	typedef enum {
-		ActionCancel,
-		ActionGetDepends,
-		ActionGetDetails,
-		ActionGetFiles,
-		ActionGetPackages,
-		ActionGetRepoList,
-		ActionGetRequires,
-		ActionGetUpdateDetail,
-		ActionGetUpdates,
-		ActionInstallFiles,
-		ActionInstallPackages,
-		ActionInstallSignature,
-		ActionRefreshCache,
-		ActionRemovePackages,
-		ActionRepoEnable,
-		ActionRepoSetData,
-		ActionResolve,
-		ActionRollback,
-		ActionSearchDetails,
-		ActionSearchFile,
-		ActionSearchGroup,
-		ActionSearchName,
-		ActionServicePack,
-		ActionUpdatePackages,
-		ActionUpdateSystem,
-		ActionWhatProvides,
-		ActionAcceptEula,
-		ActionDownloadPackages,
-		ActionGetDistroUpgrades,
-		ActionGetCategories,
-		UnkownAction = -1
+		ActionCancel		 = 0x00000001,
+		ActionGetDepends	 = 0x00000002,
+		ActionGetDetails	 = 0x00000004,
+		ActionGetFiles		 = 0x00000008,
+		ActionGetPackages	 = 0x00000010,
+		ActionGetRepoList	 = 0x00000020,
+		ActionGetRequires	 = 0x00000040,
+		ActionGetUpdateDetail	 = 0x00000080,
+		ActionGetUpdates	 = 0x00000100,
+		ActionInstallFiles	 = 0x00000200,
+		ActionInstallPackages	 = 0x00000400,
+		ActionInstallSignature	 = 0x00000800,
+		ActionRefreshCache	 = 0x00001000,
+		ActionRemovePackages	 = 0x00002000,
+		ActionRepoEnable	 = 0x00004000,
+		ActionRepoSetData	 = 0x00008000,
+		ActionResolve		 = 0x00010000,
+		ActionRollback		 = 0x00020000,
+		ActionSearchDetails	 = 0x00040000,
+		ActionSearchFile	 = 0x00080000,
+		ActionSearchGroup	 = 0x00100000,
+		ActionSearchName	 = 0x00200000,
+		ActionUpdatePackages	 = 0x00400000,
+		ActionUpdateSystem	 = 0x00800000,
+		ActionWhatProvides	 = 0x01000000,
+		ActionAcceptEula	 = 0x02000000,
+		ActionDownloadPackages	 = 0x04000000,
+		ActionGetDistroUpgrades	 = 0x08000000,
+		ActionGetCategories	 = 0x10000000,
+		ActionGetOldTransactions = 0x20000000,
+		UnknownAction		 = 0x40000000
 	} Action;
-	typedef QSet<Action> Actions;
+	Q_DECLARE_FLAGS(Actions, Action);
 
 	/**
 	 * Returns all the actions supported by the current backend
@@ -138,32 +141,34 @@ public:
 	 * Describes the different filters
 	 */
 	typedef enum {
-		NoFilter,
-		FilterInstalled,
-		FilterNotInstalled,
-		FilterDevelopment,
-		FilterNotDevelopment,
-		FilterGui,
-		FilterNotGui,
-		FilterFree,
-		FilterNotFree,
-		FilterVisible,
-		FilterNotVisible,
-		FilterSupported,
-		FilterNotSupported,
-		FilterBasename,
-		FilterNotBasename,
-		FilterNewest,
-		FilterNotNewest,
-		FilterArch,
-		FilterNotArch,
-		FilterSource,
-		FilterNotSource,
-		FilterCollections,
-		FilterNotCollections,
-		UnknownFilter = -1
+		NoFilter		 = 0x0000001,
+		FilterInstalled		 = 0x0000002,
+		FilterNotInstalled	 = 0x0000004,
+		FilterDevelopment	 = 0x0000008,
+		FilterNotDevelopment	 = 0x0000010,
+		FilterGui		 = 0x0000020,
+		FilterNotGui		 = 0x0000040,
+		FilterFree		 = 0x0000080,
+		FilterNotFree		 = 0x0000100,
+		FilterVisible		 = 0x0000200,
+		FilterNotVisible	 = 0x0000400,
+		FilterSupported		 = 0x0000800,
+		FilterNotSupported	 = 0x0001000,
+		FilterBasename		 = 0x0002000,
+		FilterNotBasename	 = 0x0004000,
+		FilterNewest		 = 0x0008000,
+		FilterNotNewest		 = 0x0010000,
+		FilterArch		 = 0x0020000,
+		FilterNotArch		 = 0x0040000,
+		FilterSource		 = 0x0080000,
+		FilterNotSource		 = 0x0100000,
+		FilterCollections	 = 0x0200000,
+		FilterNotCollections	 = 0x0400000,
+		FilterApplication	 = 0x0800000,
+		FilterNotApplication	 = 0x1000000,
+		UnknownFilter		 = 0x2000000
 	} Filter;
-	typedef QSet<Filter> Filters;
+	Q_DECLARE_FLAGS(Filters, Filter);
 
 	/**
 	 * Returns the filters supported by the current backend
@@ -174,41 +179,41 @@ public:
 	 * Describes the different groups
 	 */
 	typedef enum {
-		Accessibility,
-		Accessories,
-		AdminTools,
-		Communication,
-		DesktopGnome,
-		DesktopKde,
-		DesktopOther,
-		DesktopXfce,
-		Education,
-		Fonts,
-		Games,
-		Graphics,
-		Internet,
-		Legacy,
-		Localization,
-		Maps,
-		Multimedia,
-		Network,
-		Office,
-		Other,
-		PowerManagement,
-		Programming,
-		Publishing,
-		Repos,
-		Security,
-		Servers,
-		System,
-		Virtualization,
-		Science,
-		Documentation,
-		Electronics,
-		Collections,
-		Vendor,
-		Newest,
-		UnknownGroup = -1
+		GroupAccessibility,
+		GroupAccessories,
+		GroupAdminTools,
+		GroupCommunication,
+		GroupDesktopGnome,
+		GroupDesktopKde,
+		GroupDesktopOther,
+		GroupDesktopXfce,
+		GroupEducation,
+		GroupFonts,
+		GroupGames,
+		GroupGraphics,
+		GroupInternet,
+		GroupLegacy,
+		GroupLocalization,
+		GroupMaps,
+		GroupMultimedia,
+		GroupNetwork,
+		GroupOffice,
+		GroupOther,
+		GroupPowerManagement,
+		GroupProgramming,
+		GroupPublishing,
+		GroupRepos,
+		GroupSecurity,
+		GroupServers,
+		GroupSystem,
+		GroupVirtualization,
+		GroupScience,
+		GroupDocumentation,
+		GroupElectronics,
+		GroupCollections,
+		GroupVendor,
+		GroupNewest,
+		UnknownGroup
 	} Group;
 	typedef QSet<Group> Groups;
 
@@ -217,15 +222,21 @@ public:
 	 */
 	Groups getGroups();
 
+	/**
+	 * Returns a list containing the MIME types supported by the current backend
+	 */
+	QStringList getMimeTypes();
+
 	/**
 	 * Describes the current network state
 	 */
 	typedef enum {
-    	Offline,
-    	Online,
-    	Slow,
-    	Fast,
-    	UnknownNetworkState = -1
+		NetworkOffline,
+		NetworkOnline,
+		NetworkWired,
+		NetworkWifi,
+		NetworkMobile,
+		UnknownNetworkState
 	} NetworkState;
 
 	/**
@@ -270,8 +281,8 @@ public:
 	 * Describes a signature type
 	 */
 	typedef enum {
-		Gpg,
-		UnknownSignatureType = -1
+		SignatureGpg,
+		UnknownSignatureType
 	} SignatureType;
 
 	/**
@@ -302,67 +313,81 @@ public:
 		ProvidesCodec,
 		ProvidesMimetype,
 		ProvidesFont,
-		UnknownProvidesType = -1
+		ProvidesHardwareDriver,
+		UnknownProvidesType
 	} ProvidesType;
 
 	/**
 	 * Lists the different types of error
 	 */
 	typedef enum {
-		Oom,
-		NoNetwork,
-		NotSupported,
-		InternalError,
-		GpgFailure,
-		PackageIdInvalid,
-		PackageNotInstalled,
-		PackageNotFound,
-		PackageAlreadyInstalled,
-		PackageDownloadFailed,
-		GroupNotFound,
-		GroupListInvalid,
-		DepResolutionFailed,
-		FilterInvalid,
-		CreateThreadFailed,
-		TransactionError,
-		TransactionCancelled,
-		NoCache,
-		RepoNotFound,
-		CannotRemoveSystemPackage,
-		ProcessKill,
-		FailedInitialization,
-		FailedFinalise,
-		FailedConfigParsing,
-		CannotCancel,
-		CannotGetLock,
-		NoPackagesToUpdate,
-		CannotWriteRepoConfig,
-		LocalInstallFailed,
-		BadGpgSignature,
-		MissingGpgSignature,
-		CannotInstallSourcePackage,
-		RepoConfigurationError,
-		NoLicenseAgreement,
-		FileConflicts,
-		PackageConflicts,
-		RepoNotAvailable,
-		InvalidPackageFile,
-		PackageInstallBlocked,
-		PackageCorrupt,
-		AllPackagesAlreadyInstalled,
-		FileNotFound,
-		NoMoreMirrorsToTry,
-		UnknownErrorType = -1
+		ErrorOom,
+		ErrorNoNetwork,
+		ErrorNotSupported,
+		ErrorInternalError,
+		ErrorGpgFailure,
+		ErrorPackageIdInvalid,
+		ErrorPackageNotInstalled,
+		ErrorPackageNotFound,
+		ErrorPackageAlreadyInstalled,
+		ErrorPackageDownloadFailed,
+		ErrorGroupNotFound,
+		ErrorGroupListInvalid,
+		ErrorDepResolutionFailed,
+		ErrorFilterInvalid,
+		ErrorCreateThreadFailed,
+		ErrorTransactionError,
+		ErrorTransactionCancelled,
+		ErrorNoCache,
+		ErrorRepoNotFound,
+		ErrorCannotRemoveSystemPackage,
+		ErrorProcessKill,
+		ErrorFailedInitialization,
+		ErrorFailedFinalise,
+		ErrorFailedConfigParsing,
+		ErrorCannotCancel,
+		ErrorCannotGetLock,
+		ErrorNoPackagesToUpdate,
+		ErrorCannotWriteRepoConfig,
+		ErrorLocalInstallFailed,
+		ErrorBadGpgSignature,
+		ErrorMissingGpgSignature,
+		ErrorCannotInstallSourcePackage,
+		ErrorRepoConfigurationError,
+		ErrorNoLicenseAgreement,
+		ErrorFileConflicts,
+		ErrorPackageConflicts,
+		ErrorRepoNotAvailable,
+		ErrorInvalidPackageFile,
+		ErrorPackageInstallBlocked,
+		ErrorPackageCorrupt,
+		ErrorAllPackagesAlreadyInstalled,
+		ErrorFileNotFound,
+		ErrorNoMoreMirrorsToTry,
+		ErrorNoDistroUpgradeData,
+		ErrorIncompatibleArchitecture,
+		ErrorNoSpaceOnDevice,
+		ErrorMediaChangeRequired,
+		UnknownErrorType
 	} ErrorType;
 
 	/**
 	 * Describes a message's type
 	 */
 	typedef enum {
-		Warning,
-		Notice,
-		Daemon,
-		UnknownMessageType = -1
+		MessageBrokenMirror,
+		MessageConnectionRefused,
+		MessageParameterInvalid,
+		MessagePriorityInvalid,
+		MessageBackendError,
+		MessageDaemonError,
+		MessageCacheBeingRebuilt,
+		MessageUntrustedPackage,
+		MessageNewerPackageExists,
+		MessageCouldNotFindPackage,
+		MessageConfigFilesChanged,
+		MessagePackageAlreadyInstalled,
+		UnknownMessageType
 	} MessageType;
 
 	/**
@@ -387,18 +412,39 @@ public:
 		RestartApplication,
 		RestartSession,
 		RestartSystem,
-		UnknownRestartType = -1
+		UnknownRestartType
 	} RestartType;
 
 	/**
 	 * Describes an update's state
 	 */
 	typedef enum {
-		UpgradeStable,
-		UpgradeUnstable,
-		UpgradeTesting,
-		UnknownUpgradeType = -1
-	} UpgradeType;
+		UpdateStable,
+		UpdateUnstable,
+		UpdateTesting,
+		UnknownUpdateState
+	} UpdateState;
+
+	/**
+	 * Describes an distro upgrade state
+	 */
+	typedef enum {
+		DistroUpgradeStable,
+		DistroUpgradeUnstable,
+		UnknownDistroUpgrade
+	} DistroUpgradeType;
+
+	/**
+	 * Describes an error at the daemon level (for example, PackageKit crashes or is unreachable)
+	 */
+	typedef enum {
+		DaemonUnreachable,
+		UnkownDaemonError
+	} DaemonError;
+	/**
+	 * Returns the last daemon error that was caught
+	 */
+	DaemonError getLastError();
 
 	/**
 	 * Describes a software update
@@ -422,7 +468,7 @@ public:
 		RestartType restart;
 		QString updateText;
 		QString changelog;
-		UpgradeType state;
+		UpdateState state;
 		QDateTime issued;
 		QDateTime updated;
 	} UpdateInfo;
@@ -467,10 +513,8 @@ public:
 	 * \sa Transaction::package
 	 *
 	 */
-	Transaction* getDepends(const QList<Package*>& packages, Filters filters = Filters() << NoFilter, bool recursive = true);
-	Transaction* getDepends(Package* package, Filters filters = Filters() << NoFilter, bool recursive = true);
-	Transaction* getDepends(const QList<Package*>& packages, Filter filter, bool recursive = true);
-	Transaction* getDepends(Package* package, Filter filter, bool recursive = true);
+	Transaction* getDepends(const QList<Package*>& packages, Filters filters = NoFilter, bool recursive = true);
+	Transaction* getDepends(Package* package, Filters filters = NoFilter, bool recursive = true);
 
 	/**
 	 * Gets more details about the given \p packages
@@ -500,14 +544,12 @@ public:
 	 *
 	 * \sa Transaction::package
 	 */
-	Transaction* getPackages(Filters filters = Filters() << NoFilter);
-	Transaction* getPackages(Filter filter);
+	Transaction* getPackages(Filters filters = NoFilter);
 
 	/**
 	 * Gets the list of software repositories matching the given \p filters
 	 */
-	Transaction* getRepoList(Filters filter = Filters() << NoFilter);
-	Transaction* getRepoList(Filter filter);
+	Transaction* getRepoList(Filters filter = NoFilter);
 
 	/**
 	 * \brief Searches for the packages requiring the given \p packages
@@ -515,10 +557,8 @@ public:
 	 * The search can be limited using the \p filters parameter. The recursive flag is used to tell
 	 * if the package manager should also search for the package requiring the resulting packages.
 	 */
-	Transaction* getRequires(const QList<Package*>& packages, Filters filters = Filters() << NoFilter, bool recursive = true);
-	Transaction* getRequires(Package* package, Filters filters = Filters() << NoFilter, bool recursive = true);
-	Transaction* getRequires(const QList<Package*>& packages, Filter filter, bool recursive = true);
-	Transaction* getRequires(Package* package, Filter filter, bool recursive = true);
+	Transaction* getRequires(const QList<Package*>& packages, Filters filters = NoFilter, bool recursive = true);
+	Transaction* getRequires(Package* package, Filters filters = NoFilter, bool recursive = true);
 
 	/**
 	 * Retrieves more details about the update for the given \p packages
@@ -531,8 +571,7 @@ public:
 	 *
 	 * The \p filters parameters can be used to restrict the updates returned
 	 */
-	Transaction* getUpdates(Filters filters = Filters() << NoFilter);
-	Transaction* getUpdates(Filter filter);
+	Transaction* getUpdates(Filters filters = NoFilter);
 
 	/**
 	 * Retrieves the available distribution upgrades
@@ -542,7 +581,7 @@ public:
 	/**
 	 * \brief Installs the local packages \p files
 	 *
-	 * \trusted indicate if the packages are signed by a trusted authority
+	 * \p trusted indicate if the packages are signed by a trusted authority
 	 */
 	Transaction* installFiles(const QStringList& files, bool trusted);
 	Transaction* installFile(const QString& file, bool trusted);
@@ -590,39 +629,51 @@ public:
 	 *
 	 * The \p filters can be used to restrict the search
 	 */
-	Transaction* resolve(const QStringList& packageNames, Filters filters = Filters() << NoFilter);
-	Transaction* resolve(Package* package, Filters filters = Filters() << NoFilter);
-	Transaction* resolve(const QStringList& packageNames, Filter filter);
-	Transaction* resolve(const QString& packageName, Filters filters = Filters() << NoFilter);
-	Transaction* resolve(const QString& packageName, Filter filter);
+	Transaction* resolve(const QStringList& packageNames, Filters filters = NoFilter);
+	Transaction* resolve(const QString& packageName, Filters filters = NoFilter);
 
 	/**
 	 * Rolls back the given \p transactions
 	 */
 	Transaction* rollback(Transaction* oldtrans);
 
+	/**
+	 * \brief Search in the packages files
+	 *
+	 * \p filters can be used to restrict the returned packages
+	 */
+	Transaction* searchFile(const QString& search, Filters filters = NoFilter);
+
 	/**
 	 * \brief Search in the packages details
 	 *
 	 * \p filters can be used to restrict the returned packages
 	 */
-	Transaction* searchDetails(const QString& search, Filters filters = Filters() << NoFilter);
+	Transaction* searchDetails(const QString& search, Filters filters = NoFilter);
 
 	/**
 	 * \brief Lists all the packages in the given \p group
 	 *
 	 * \p filters can be used to restrict the returned packages
 	 */
-	Transaction* searchGroup(Client::Group group, Filters filters = Filters() << NoFilter);
-	Transaction* searchGroup(Client::Group group, Filter filter);
+	Transaction* searchGroup(Client::Group group, Filters filters = NoFilter);
 
 	/**
 	 * \brief Search in the packages names
 	 *
 	 * \p filters can be used to restrict the returned packages
 	 */
-	Transaction* searchName(const QString& search, Filters filters = Filters() << NoFilter);
-	Transaction* searchName(const QString& search, Filter filter);
+	Transaction* searchName(const QString& search, Filters filters = NoFilter);
+
+	/**
+	 * \brief Tries to find a package name from a desktop file
+	 *
+	 * This function looks into /var/lib/PackageKit/desktop-files.db and searches for the associated package name.
+	 *
+	 * \p path the path to the desktop file (as shipped by the package)
+	 * \return The associated package, or NULL if there's no result
+	 */
+	Package* searchFromDesktopFile(const QString& path);
 
 	/**
 	 * Update the given \p packages
@@ -638,8 +689,7 @@ public:
 	/**
 	 * Searchs for a package providing a file/a mimetype
 	 */
-	Transaction* whatProvides(ProvidesType type, const QString& search, Filters filters = Filters() << NoFilter);
-	Transaction* whatProvides(ProvidesType type, const QString& search, Filter filter);
+	Transaction* whatProvides(ProvidesType type, const QString& search, Filters filters = NoFilter);
 
 Q_SIGNALS:
 	/**
@@ -649,6 +699,11 @@ Q_SIGNALS:
 	 */
 	void authError(const QString& action);
 
+	/**
+	 * Emitted when the PackageKit daemon is not reachable anymore
+	 */
+	void daemonError(PackageKit::Client::DaemonError e);
+
 	/**
 	 * Emitted when the daemon's locked state changes
 	 */
@@ -687,6 +742,8 @@ private:
 	friend class ClientPrivate;
 	ClientPrivate* d;
 };
+Q_DECLARE_OPERATORS_FOR_FLAGS(Client::Actions)
+Q_DECLARE_OPERATORS_FOR_FLAGS(Client::Filters)
 
 } // End namespace PackageKit
 
