@@ -25,15 +25,27 @@ import sys
 import codecs
 import traceback
 import os.path
-sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
 
 from enums import *
 
 PACKAGE_IDS_DELIM = '&'
 FILENAME_DELIM = '|'
 
+def _to_unicode(txt, encoding='utf-8'):
+    if isinstance(txt, basestring):
+        if not isinstance(txt, unicode):
+            txt = unicode(txt, encoding, errors='replace')
+    return txt
+
 # Classes
 
+class PkError(Exception):
+    def __init__(self, code, details):
+        self.code = code
+        self.details = details
+    def __str__(self):
+        return repr("%s: %s" % (self.code, self.details))
+
 class PackageKitBaseBackend:
 
     def __init__(self, cmds):
@@ -41,6 +53,23 @@ class PackageKitBaseBackend:
         installExceptionHandler(self)
         self.cmds = cmds
         self._locked = False
+        self.lang = "C"
+        self.has_network = False
+        self.percentage_old = 0
+        self.sub_percentage_old = 0
+
+        # try to get LANG
+        try:
+            self.lang = os.environ['LANG']
+        except KeyError, e:
+            print "Error: No LANG envp"
+
+        # try to get NETWORK state
+        try:
+            if os.environ['NETWORK'] == 'TRUE':
+                self.has_network = True
+        except KeyError, e:
+            print "Error: No NETWORK envp"
 
     def doLock(self):
         ''' Generic locking, overide and extend in child class'''
@@ -56,20 +85,23 @@ class PackageKitBaseBackend:
     def percentage(self, percent=None):
         '''
         Write progress percentage
-        @param percent: Progress percentage
+        @param percent: Progress percentage (int preferred)
         '''
-        if percent != None:
-            print "percentage\t%i" % (percent)
-        else:
+        if percent == None:
             print "no-percentage-updates"
+        elif percent == 0 or percent > self.percentage_old:
+            print "percentage\t%i" % (percent)
+            self.percentage_old = percent
         sys.stdout.flush()
 
     def sub_percentage(self, percent=None):
         '''
         send 'subpercentage' signal : subprogress percentage
-        @param percent: subprogress percentage
+        @param percent: subprogress percentage (int preferred)
         '''
-        print "subpercentage\t%i" % (percent)
+        if percent == 0 or percent > self.sub_percentage_old:
+            print "subpercentage\t%i" % (percent)
+            self.sub_percentage_old = percent
         sys.stdout.flush()
 
     def error(self, err, description, exit=True):
@@ -109,6 +141,17 @@ class PackageKitBaseBackend:
         print >> sys.stdout, "package\t%s\t%s\t%s" % (status, package_id, summary)
         sys.stdout.flush()
 
+    def media_change_required(self, mtype, id, text):
+        '''
+        send 'media-change-required' signal
+        @param mtype: the enumerated MEDIA_TYPE_* string
+        @param id: the localised label of the media
+        @param text: the localised text describing the media
+        '''
+        print >> sys.stdout, "media-change-required\t%s\t%s\t%s" % (mtype, id,
+                                                                    text)
+        sys.stdout.flush()
+
     def distro_upgrade(self, dtype, name, summary):
         '''
         send 'distro-upgrade' signal
@@ -358,14 +401,14 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def update_packages(self, package):
+    def update_packages(self, package_ids):
         '''
         Implement the {backend}-update functionality
         Needed to be implemented in a sub class
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def get_details(self, package):
+    def get_details(self, package_ids):
         '''
         Implement the {backend}-get-details functionality
         Needed to be implemented in a sub class
@@ -421,7 +464,7 @@ class PackageKitBaseBackend:
         '''
         self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
-    def download_packages(self, directory, packages):
+    def download_packages(self, directory, package_ids):
         '''
         Implement the {backend}-download-packages functionality
         Needed to be implemented in a sub class
@@ -440,7 +483,7 @@ class PackageKitBaseBackend:
         Implement the {backend}-get-categories functionality
         Needed to be implemented in a sub class
         '''
-        self.error(ERROR_NOT_SUPPORTED,"This function is not implemented in this backend", exit=False)
+        self.error(ERROR_NOT_SUPPORTED, "This function is not implemented in this backend", exit=False)
 
     def customTracebackHandler(self, tb):
         '''
@@ -465,22 +508,22 @@ class PackageKitBaseBackend:
     def dispatch_command(self, cmd, args):
         if cmd == 'download-packages':
             directory = args[0]
-            pkgs = args[1].split(PACKAGE_IDS_DELIM)
-            self.download_packages(directory, pkgs)
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
+            self.download_packages(directory, package_ids)
             self.finished()
         elif cmd == 'get-depends':
             filters = args[0]
-            pkgs = args[1].split(PACKAGE_IDS_DELIM)
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
             recursive = args[2]
-            self.get_depends(filters, pkgs, recursive)
+            self.get_depends(filters, package_ids, recursive)
             self.finished()
         elif cmd == 'get-details':
-            pkgs = args[0].split(PACKAGE_IDS_DELIM)
-            self.get_details(pkgs)
+            package_ids = args[0].split(PACKAGE_IDS_DELIM)
+            self.get_details(package_ids)
             self.finished()
         elif cmd == 'get-files':
-            pkgs = args[0].split(PACKAGE_IDS_DELIM)
-            self.get_files(pkgs)
+            package_ids = args[0].split(PACKAGE_IDS_DELIM)
+            self.get_files(package_ids)
             self.finished()
         elif cmd == 'get-packages':
             filters = args[0]
@@ -492,13 +535,13 @@ class PackageKitBaseBackend:
             self.finished()
         elif cmd == 'get-requires':
             filters = args[0]
-            pkgs = args[1].split(PACKAGE_IDS_DELIM)
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
             recursive = args[2]
-            self.get_requires(filters, pkgs, recursive)
+            self.get_requires(filters, package_ids, recursive)
             self.finished()
         elif cmd == 'get-update-detail':
-            pkgs = args[0].split(PACKAGE_IDS_DELIM)
-            self.get_update_detail(pkgs)
+            package_ids = args[0].split(PACKAGE_IDS_DELIM)
+            self.get_update_detail(package_ids)
             self.finished()
         elif cmd == 'get-distro-upgrades':
             self.get_distro_upgrades()
@@ -513,8 +556,8 @@ class PackageKitBaseBackend:
             self.install_files(trusted, files_to_inst)
             self.finished()
         elif cmd == 'install-packages':
-            pkgs = args[0].split(PACKAGE_IDS_DELIM)
-            self.install_packages(pkgs)
+            package_ids = args[0].split(PACKAGE_IDS_DELIM)
+            self.install_packages(package_ids)
             self.finished()
         elif cmd == 'install-signature':
             sigtype = args[0]
@@ -527,8 +570,8 @@ class PackageKitBaseBackend:
             self.finished()
         elif cmd == 'remove-packages':
             allowdeps = args[0]
-            packages = args[1].split(PACKAGE_IDS_DELIM)
-            self.remove_packages(allowdeps, packages)
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
+            self.remove_packages(allowdeps, package_ids)
             self.finished()
         elif cmd == 'repo-enable':
             repoid = args[0]
@@ -543,12 +586,12 @@ class PackageKitBaseBackend:
             self.finished()
         elif cmd == 'resolve':
             filters = args[0]
-            packages = args[1].split(PACKAGE_IDS_DELIM)
-            self.resolve(filters, packages)
+            package_ids = args[1].split(PACKAGE_IDS_DELIM)
+            self.resolve(filters, package_ids)
             self.finished()
         elif cmd == 'search-details':
             options = args[0]
-            searchterms = args[1]
+            searchterms = _to_unicode(args[1])
             self.search_details(options, searchterms)
             self.finished()
         elif cmd == 'search-file':
@@ -563,7 +606,7 @@ class PackageKitBaseBackend:
             self.finished()
         elif cmd == 'search-name':
             options = args[0]
-            searchterms = args[1]
+            searchterms = _to_unicode(args[1])
             self.search_name(options, searchterms)
             self.finished()
         elif cmd == 'signature-install':
@@ -571,8 +614,8 @@ class PackageKitBaseBackend:
             self.repo_signature_install(package)
             self.finished()
         elif cmd == 'update-packages':
-            packages = args[0].split(PACKAGE_IDS_DELIM)
-            self.update_packages(packages)
+            package_ids = args[0].split(PACKAGE_IDS_DELIM)
+            self.update_packages(package_ids)
             self.finished()
         elif cmd == 'update-system':
             self.update_system()
@@ -580,7 +623,7 @@ class PackageKitBaseBackend:
         elif cmd == 'what-provides':
             filters = args[0]
             provides_type = args[1]
-            search = args[2]
+            search = _to_unicode(args[2])
             self.what_provides(filters, provides_type, search)
             self.finished()
         elif cmd == 'set-locale':
@@ -615,6 +658,32 @@ class PackageKitBaseBackend:
             self.unLock()
         sys.exit(0)
 
+
+def format_string(text, encoding='utf-8'):
+    '''
+    Format a string to be used on stdout for communication with the daemon.
+    '''
+    if not isinstance(text, unicode):
+        txt = unicode(text, encoding, errors='replace')
+    return text.replace("\n", ";")
+
+def text_to_bool(text):
+    '''Convert a string to a boolean value.'''
+    if text.lower() in ["yes", "true"]:
+        return True
+    return False
+
+def get_package_id(name, version, arch, data):
+    """Returns a package id."""
+    return ";".join((name, version, arch, data))
+
+def split_package_id(id):
+    """
+    Returns a tuple with the name, version, arch and data component of a
+    package id.
+    """
+    return id.split(";", 4)
+
 def exceptionHandler(typ, value, tb, base):
     # Restore original exception handler
     sys.excepthook = sys.__excepthook__
