@@ -36,6 +36,7 @@
 #include "egg-debug.h"
 #include "egg-string.h"
 
+#include "pk-conf.h"
 #include "pk-network.h"
 #include "pk-marshal.h"
 #include "pk-backend-internal.h"
@@ -73,6 +74,13 @@
  */
 #define PK_BACKEND_FINISHED_TIMEOUT_GRACE	10 /* ms */
 
+/* a boolean with unset */
+typedef enum {
+	PK_BACKEND_TRISTATE_FALSE = FALSE,
+	PK_BACKEND_TRISTATE_TRUE = TRUE,
+	PK_BACKEND_TRISTATE_UNSET
+} PkBackendTristate;
+
 struct _PkBackendPrivate
 {
 	GModule			*handle;
@@ -87,7 +95,9 @@ struct _PkBackendPrivate
 	gboolean		 set_error;
 	gboolean		 set_signature;
 	gboolean		 set_eula;
+	gboolean		 simultaneous;
 	gboolean		 has_sent_package;
+	gboolean		 use_time;
 	PkNetwork		*network;
 	PkStore			*store;
 	PkPackageObj		*last_package;
@@ -98,7 +108,7 @@ struct _PkBackendPrivate
 	PkBackendFileChanged	 file_changed_func;
 	gpointer		 file_changed_data;
 	gboolean		 during_initialize;
-	gboolean		 allow_cancel;
+	PkBackendTristate	 allow_cancel;
 	gboolean		 finished;
 	guint			 last_percentage;
 	guint			 last_subpercentage;
@@ -129,6 +139,7 @@ enum {
 	PK_BACKEND_ALLOW_CANCEL,
 	PK_BACKEND_REPO_DETAIL,
 	PK_BACKEND_CATEGORY,
+	PK_BACKEND_MEDIA_CHANGE_REQUIRED,
 	PK_BACKEND_LAST_SIGNAL
 };
 
@@ -352,7 +363,7 @@ pk_backend_get_array (PkBackend *backend, const gchar *key)
 /**
  * pk_backend_get_uint:
  **/
-uint
+guint
 pk_backend_get_uint (PkBackend *backend, const gchar *key)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), 0);
@@ -652,10 +663,9 @@ pk_backend_set_percentage (PkBackend *backend, guint percentage)
 		remaining = pk_time_get_remaining (backend->priv->time);
 		egg_debug ("this will now take ~%i seconds", remaining);
 
-#ifdef PK_IS_DEVELOPER
-		/* Until the predicted time is more sane... */
-		backend->priv->last_remaining = remaining;
-#endif
+		/* value cached from config file */
+		if (backend->priv->use_time)
+			backend->priv->last_remaining = remaining;
 	}
 
 	/* emit the progress changed signal */
@@ -774,15 +784,79 @@ pk_backend_get_status (PkBackend *backend)
 	return backend->priv->status;
 }
 
+/**
+ * pk_backend_package_emulate_finished:
+ **/
+static gboolean
+pk_backend_package_emulate_finished (PkBackend *backend)
+{
+	PkInfoEnum info;
+	gchar *package_id;
+	gboolean ret = FALSE;
+
+	/* simultaneous handles this on it's own */
+	if (backend->priv->simultaneous)
+		return FALSE;
+
+	/* first package in transaction */
+	if (backend->priv->last_package == NULL)
+		return FALSE;
+
+	/* already finished */
+	if (backend->priv->last_package->info == PK_INFO_ENUM_FINISHED)
+		return FALSE;
+
+	/* only makes sense for some values */
+	info = backend->priv->last_package->info;
+	if (info == PK_INFO_ENUM_DOWNLOADING ||
+	    info == PK_INFO_ENUM_UPDATING ||
+	    info == PK_INFO_ENUM_INSTALLING ||
+	    info == PK_INFO_ENUM_REMOVING ||
+	    info == PK_INFO_ENUM_CLEANUP ||
+	    info == PK_INFO_ENUM_OBSOLETING) {
+		package_id = pk_package_id_to_string (backend->priv->last_package->id);
+		pk_backend_package (backend, PK_INFO_ENUM_FINISHED, package_id, backend->priv->last_package->summary);
+		g_free (package_id);
+		ret = TRUE;
+	}
+	return ret;
+}
+
+/**
+ * pk_backend_package_emulate_finished_for_package:
+ **/
+static gboolean
+pk_backend_package_emulate_finished_for_package (PkBackend *backend, const PkPackageObj *obj)
+{
+	/* simultaneous handles this on it's own */
+	if (backend->priv->simultaneous)
+		return FALSE;
+
+	/* first package in transaction */
+	if (backend->priv->last_package == NULL)
+		return FALSE;
+
+	/* sending finished already */
+	if (obj->info == PK_INFO_ENUM_FINISHED)
+		return FALSE;
+
+	/* same package, just info change */
+	if (pk_package_id_equal (backend->priv->last_package->id, obj->id))
+		return FALSE;
+
+	/* emit the old package as finished */
+	return pk_backend_package_emulate_finished (backend);
+}
+
 /**
  * pk_backend_package:
  **/
 gboolean
 pk_backend_package (PkBackend *backend, PkInfoEnum info, const gchar *package_id, const gchar *summary)
 {
-	gchar *summary_safe;
-	PkPackageObj *obj;
-	PkPackageId *id;
+	gchar *summary_safe = NULL;
+	PkPackageObj *obj = NULL;
+	PkPackageId *id = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
@@ -793,7 +867,7 @@ pk_backend_package (PkBackend *backend, PkInfoEnum info, const gchar *package_id
 	ret = pk_package_id_check (package_id);
 	if (!ret) {
 		egg_warning ("package_id invalid and cannot be processed: %s", package_id);
-		return FALSE;
+		goto out;
 	}
 
 	/* replace unsafe chars */
@@ -801,12 +875,27 @@ pk_backend_package (PkBackend *backend, PkInfoEnum info, const gchar *package_id
 
 	/* check against the old one */
 	id = pk_package_id_new_from_string (package_id);
+	if (id == NULL) {
+		egg_warning ("Failed to parse package_id: '%s'", package_id);
+		ret = FALSE;
+		goto out;
+	}
 	obj = pk_package_obj_new (info, id, summary_safe);
+	if (obj == NULL) {
+		egg_warning ("Failed to create object summary: '%s'", summary_safe);
+		ret = FALSE;
+		goto out;
+	}
 	ret = pk_package_obj_equal (obj, backend->priv->last_package);
 	if (ret) {
 		egg_debug ("skipping duplicate %s", package_id);
-		return FALSE;
+		ret = FALSE;
+		goto out;
 	}
+
+	/* simulate the finish here when required */
+	pk_backend_package_emulate_finished_for_package (backend, obj);
+
 	/* update the 'last' package */
 	pk_package_obj_free (backend->priv->last_package);
 	backend->priv->last_package = pk_package_obj_copy (obj);
@@ -814,22 +903,25 @@ pk_backend_package (PkBackend *backend, PkInfoEnum info, const gchar *package_id
 	/* have we already set an error? */
 	if (backend->priv->set_error) {
 		egg_warning ("already set error, cannot process: package %s", package_id);
-		return FALSE;
+		ret = FALSE;
+		goto out;
 	}
 
-	/* we automatically set the transaction status for some infos */
-	if (info == PK_INFO_ENUM_DOWNLOADING) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_DOWNLOAD);
-	} else if (info == PK_INFO_ENUM_UPDATING) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_UPDATE);
-	} else if (info == PK_INFO_ENUM_INSTALLING) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_INSTALL);
-	} else if (info == PK_INFO_ENUM_REMOVING) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_REMOVE);
-	} else if (info == PK_INFO_ENUM_CLEANUP) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_CLEANUP);
-	} else if (info == PK_INFO_ENUM_OBSOLETING) {
-		pk_backend_set_status (backend, PK_STATUS_ENUM_OBSOLETE);
+	/* we automatically set the transaction status for some PkInfoEnums if running
+	 * in non-simultaneous transaction mode */
+	if (!backend->priv->simultaneous) {
+		if (info == PK_INFO_ENUM_DOWNLOADING)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_DOWNLOAD);
+		else if (info == PK_INFO_ENUM_UPDATING)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_UPDATE);
+		else if (info == PK_INFO_ENUM_INSTALLING)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_INSTALL);
+		else if (info == PK_INFO_ENUM_REMOVING)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_REMOVE);
+		else if (info == PK_INFO_ENUM_CLEANUP)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_CLEANUP);
+		else if (info == PK_INFO_ENUM_OBSOLETING)
+			pk_backend_set_status (backend, PK_STATUS_ENUM_OBSOLETE);
 	}
 
 	/* we've sent a package for this transaction */
@@ -838,10 +930,13 @@ pk_backend_package (PkBackend *backend, PkInfoEnum info, const gchar *package_id
 	egg_debug ("emit package %s, %s, %s", pk_info_enum_to_text (info), package_id, summary_safe);
 	g_signal_emit (backend, signals [PK_BACKEND_PACKAGE], 0, obj);
 
+	/* success */
+	ret = TRUE;
+out:
 	pk_package_id_free (id);
 	pk_package_obj_free (obj);
 	g_free (summary_safe);
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -856,11 +951,12 @@ pk_backend_update_detail (PkBackend *backend, const gchar *package_id,
 			  PkUpdateStateEnum state, const gchar *issued_text,
 			  const gchar *updated_text)
 {
-	gchar *update_text_safe;
-	PkUpdateDetailObj *detail;
-	PkPackageId *id;
-	GDate *issued;
-	GDate *updated;
+	gchar *update_text_safe = NULL;
+	PkUpdateDetailObj *detail = NULL;
+	PkPackageId *id = NULL;
+	GDate *issued = NULL;
+	GDate *updated = NULL;
+	gboolean ret = FALSE;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (package_id != NULL, FALSE);
@@ -869,7 +965,7 @@ pk_backend_update_detail (PkBackend *backend, const gchar *package_id,
 	/* have we already set an error? */
 	if (backend->priv->set_error) {
 		egg_warning ("already set error, cannot process: update_detail %s", package_id);
-		return FALSE;
+		goto out;
 	}
 
 	/* convert dates */
@@ -879,14 +975,28 @@ pk_backend_update_detail (PkBackend *backend, const gchar *package_id,
 	/* replace unsafe chars */
 	update_text_safe = pk_strsafe (update_text);
 
-	/* form PkUpdateDetailObj struct */
+	/* form PkPackageId struct */
 	id = pk_package_id_new_from_string (package_id);
+	if (id == NULL) {
+		egg_warning ("Failed to parse package_id: '%s'", package_id);
+		goto out;
+	}
+
+	/* form PkUpdateDetailObj struct */
 	detail = pk_update_detail_obj_new_from_data (id, updates, obsoletes, vendor_url,
 						     bugzilla_url, cve_url, restart,
 						     update_text_safe, changelog,
 						     state, issued, updated);
+	if (detail == NULL) {
+		egg_warning ("Failed to parse detail object");
+		goto out;
+	}
 	g_signal_emit (backend, signals [PK_BACKEND_UPDATE_DETAIL], 0, detail);
 
+	/* we parsed okay */
+	ret = TRUE;
+
+out:
 	pk_package_id_free (id);
 	pk_update_detail_obj_free (detail);
 	g_free (update_text_safe);
@@ -894,7 +1004,7 @@ pk_backend_update_detail (PkBackend *backend, const gchar *package_id,
 		g_date_free (issued);
 	if (updated != NULL)
 		g_date_free (updated);
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -923,21 +1033,30 @@ pk_backend_get_progress (PkBackend *backend,
  * pk_backend_require_restart:
  **/
 gboolean
-pk_backend_require_restart (PkBackend *backend, PkRestartEnum restart, const gchar *details)
+pk_backend_require_restart (PkBackend *backend, PkRestartEnum restart, const gchar *package_id)
 {
+	gboolean ret = FALSE;
+
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (backend->priv->locked != FALSE, FALSE);
 
 	/* have we already set an error? */
 	if (backend->priv->set_error) {
 		egg_warning ("already set error, cannot process: require-restart %s", pk_restart_enum_to_text (restart));
-		return FALSE;
+		goto out;
 	}
 
-	egg_debug ("emit require-restart %s, %s", pk_restart_enum_to_text (restart), details);
-	g_signal_emit (backend, signals [PK_BACKEND_REQUIRE_RESTART], 0, restart, details);
+	/* check we are valid */
+	ret = pk_package_id_check (package_id);
+	if (!ret) {
+		egg_warning ("package_id invalid and cannot be processed: %s", package_id);
+		goto out;
+	}
 
-	return TRUE;
+	egg_debug ("emit require-restart %s, %s", pk_restart_enum_to_text (restart), package_id);
+	g_signal_emit (backend, signals [PK_BACKEND_REQUIRE_RESTART], 0, restart, package_id);
+out:
+	return ret;
 }
 
 /**
@@ -989,6 +1108,21 @@ pk_backend_set_transaction_data (PkBackend *backend, const gchar *data)
 	return TRUE;
 }
 
+/**
+ * pk_backend_set_simultaneous_mode:
+ **/
+gboolean
+pk_backend_set_simultaneous_mode (PkBackend *backend, gboolean simultaneous)
+{
+	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+	g_return_val_if_fail (backend->priv->locked != FALSE, FALSE);
+
+	backend->priv->simultaneous = simultaneous;
+	if (simultaneous)
+		egg_warning ("simultaneous mode is not well tested, use with caution");
+	return TRUE;
+}
+
 /**
  * pk_backend_get_locale:
  *
@@ -1026,9 +1160,10 @@ pk_backend_details (PkBackend *backend, const gchar *package_id,
 		    const gchar *license, PkGroupEnum group,
 		    const gchar *description, const gchar *url, gulong size)
 {
-	gchar *description_safe;
-	PkDetailsObj *details;
-	PkPackageId *id;
+	gchar *description_safe = NULL;
+	PkDetailsObj *details = NULL;
+	PkPackageId *id = NULL;
+	gboolean ret = FALSE;
 
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (package_id != NULL, FALSE);
@@ -1037,20 +1172,35 @@ pk_backend_details (PkBackend *backend, const gchar *package_id,
 	/* have we already set an error? */
 	if (backend->priv->set_error) {
 		egg_warning ("already set error, cannot process: details %s", package_id);
-		return FALSE;
+		goto out;
 	}
 
 	/* replace unsafe chars */
 	description_safe = pk_strsafe (description);
 
+	/* form PkPackageId struct */
 	id = pk_package_id_new_from_string (package_id);
+	if (id == NULL) {
+		egg_warning ("Failed to parse package_id: '%s'", package_id);
+		goto out;
+	}
+
+	/* form PkDetailsObj struct */
 	details = pk_details_obj_new_from_data (id, license, group, description_safe, url, size);
+	if (details == NULL) {
+		egg_warning ("Failed to parse details object");
+		goto out;
+	}
 	g_signal_emit (backend, signals [PK_BACKEND_DETAILS], 0, details);
 
+	/* we parsed okay */
+	ret = TRUE;
+
+out:
 	pk_package_id_free (id);
 	pk_details_obj_free (details);
 	g_free (description_safe);
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -1183,6 +1333,35 @@ pk_backend_eula_required (PkBackend *backend, const gchar *eula_id, const gchar
 	return TRUE;
 }
 
+/**
+ * pk_backend_media_change_required:
+ **/
+gboolean
+pk_backend_media_change_required (PkBackend *backend,
+				  PkMediaTypeEnum media_type,
+				  const gchar *media_id,
+				  const gchar *media_text)
+{
+	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
+	g_return_val_if_fail (media_id != NULL, FALSE);
+	g_return_val_if_fail (media_text != NULL, FALSE);
+	g_return_val_if_fail (backend->priv->locked != FALSE, FALSE);
+
+	/* have we already set an error? */
+	if (backend->priv->set_error) {
+		egg_warning ("already set error, cannot process: media change required");
+		return FALSE;
+	}
+
+	egg_debug ("emit media-change-required %s, %s, %s",
+		  pk_media_type_enum_to_text (media_type), media_id, media_text);
+
+	g_signal_emit (backend, signals [PK_BACKEND_MEDIA_CHANGE_REQUIRED], 0,
+		       media_type, media_id, media_text);
+
+	return TRUE;
+}
+
 /**
  * pk_backend_repo_detail:
  **/
@@ -1255,7 +1434,7 @@ pk_backend_error_timeout_delay_cb (gpointer data)
 	if (backend->priv->finished) {
 		egg_warning ("consistency error");
 		egg_debug_backtrace ();
-		return FALSE;
+		goto out;
 	}
 
 	/* warn the backend developer that they've done something worng
@@ -1267,6 +1446,8 @@ pk_backend_error_timeout_delay_cb (gpointer data)
 	g_signal_emit (backend, signals [PK_BACKEND_MESSAGE], 0, message, buffer);
 
 	pk_backend_finished (backend);
+out:
+	backend->priv->signal_error_timeout = 0;
 	return FALSE;
 }
 
@@ -1333,9 +1514,9 @@ pk_backend_set_allow_cancel (PkBackend *backend, gboolean allow_cancel)
 	}
 
 	/* same as last state? */
-	if (backend->priv->allow_cancel == allow_cancel) {
+	if (backend->priv->allow_cancel == (PkBackendTristate) allow_cancel) {
 		egg_debug ("ignoring same allow-cancel state");
-		return TRUE;
+		return FALSE;
 	}
 
 	/* can we do the action? */
@@ -1353,9 +1534,16 @@ pk_backend_set_allow_cancel (PkBackend *backend, gboolean allow_cancel)
 gboolean
 pk_backend_get_allow_cancel (PkBackend *backend)
 {
+	gboolean allow_cancel = FALSE;
+
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	g_return_val_if_fail (backend->priv->locked != FALSE, FALSE);
-	return backend->priv->allow_cancel;
+
+	/* return FALSE if we never set state */
+	if (backend->priv->allow_cancel != PK_BACKEND_TRISTATE_UNSET)
+		allow_cancel = backend->priv->allow_cancel;
+
+	return allow_cancel;
 }
 
 /**
@@ -1400,7 +1588,7 @@ pk_backend_get_role (PkBackend *backend)
  * Should only be used internally, or from PkRunner when setting CANCELLED.
  **/
 gboolean
-pk_backend_set_exit_code (PkBackend *backend, PkExitEnum exit)
+pk_backend_set_exit_code (PkBackend *backend, PkExitEnum exit_enum)
 {
 	g_return_val_if_fail (PK_IS_BACKEND (backend), PK_ROLE_ENUM_UNKNOWN);
 	g_return_val_if_fail (backend->priv->locked != FALSE, FALSE);
@@ -1408,13 +1596,13 @@ pk_backend_set_exit_code (PkBackend *backend, PkExitEnum exit)
 	if (backend->priv->exit != PK_EXIT_ENUM_UNKNOWN) {
 		egg_warning ("already set exit status: old=%s, new=%s",
 			    pk_exit_enum_to_text (backend->priv->exit),
-			    pk_exit_enum_to_text (exit));
+			    pk_exit_enum_to_text (exit_enum));
 		egg_debug_backtrace ();
 		return FALSE;
 	}
 
 	/* new value */
-	backend->priv->exit = exit;
+	backend->priv->exit = exit_enum;
 	return TRUE;
 }
 
@@ -1432,9 +1620,6 @@ pk_backend_finished_delay (gpointer data)
 	if (backend->priv->exit == PK_EXIT_ENUM_UNKNOWN)
 		pk_backend_set_exit_code (backend, PK_EXIT_ENUM_SUCCESS);
 
-	/* clear all state */
-	pk_store_reset (backend->priv->store);
-
 	egg_debug ("emit finished %i", backend->priv->exit);
 	g_signal_emit (backend, signals [PK_BACKEND_FINISHED], 0, backend->priv->exit);
 	backend->priv->signal_finished = 0;
@@ -1501,6 +1686,9 @@ pk_backend_finished (PkBackend *backend)
 		egg_warning ("GUI will remain unchanged!");
 	}
 
+	/* emulate the last finished package if not done already */
+	pk_backend_package_emulate_finished (backend);
+
 	/* make any UI insensitive */
 	pk_backend_set_allow_cancel (backend, FALSE);
 
@@ -1517,6 +1705,17 @@ pk_backend_finished (PkBackend *backend)
 	return TRUE;
 }
 
+/**
+ * pk_backend_bool_to_text:
+ */
+const gchar *
+pk_backend_bool_to_text (gboolean value)
+{
+	if (value)
+		return "yes";
+	return "no";
+}
+
 /**
  * pk_backend_not_implemented_yet:
  **/
@@ -1546,8 +1745,9 @@ pk_backend_is_online (PkBackend *backend)
 	g_return_val_if_fail (PK_IS_BACKEND (backend), FALSE);
 	state = pk_network_get_network_state (backend->priv->network);
 	if (state == PK_NETWORK_ENUM_ONLINE ||
-	    state == PK_NETWORK_ENUM_SLOW ||
-	    state == PK_NETWORK_ENUM_FAST)
+	    state == PK_NETWORK_ENUM_MOBILE ||
+	    state == PK_NETWORK_ENUM_WIFI ||
+	    state == PK_NETWORK_ENUM_WIRED)
 		return TRUE;
 	return FALSE;
 }
@@ -1794,6 +1994,11 @@ pk_backend_class_init (PkBackendClass *klass)
 			      0, NULL, NULL, pk_marshal_VOID__STRING_STRING_STRING_STRING,
 			      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING,
 			      G_TYPE_STRING, G_TYPE_STRING);
+	signals [PK_BACKEND_MEDIA_CHANGE_REQUIRED] =
+		g_signal_new ("media-change-required",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL, pk_marshal_VOID__UINT_STRING_STRING,
+			      G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING);
 	signals [PK_BACKEND_FINISHED] =
 		g_signal_new ("finished",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
@@ -1839,24 +2044,27 @@ pk_backend_reset (PkBackend *backend)
 	}
 
 	/* if we set an error code notifier, clear */
-	if (backend->priv->signal_error_timeout != 0)
+	if (backend->priv->signal_error_timeout != 0) {
 		g_source_remove (backend->priv->signal_error_timeout);
+		backend->priv->signal_error_timeout = 0;
+	}
 
 	pk_package_obj_free (backend->priv->last_package);
 	backend->priv->set_error = FALSE;
 	backend->priv->set_signature = FALSE;
 	backend->priv->set_eula = FALSE;
-	backend->priv->allow_cancel = FALSE;
 	backend->priv->finished = FALSE;
 	backend->priv->has_sent_package = FALSE;
 	backend->priv->thread = NULL;
 	backend->priv->last_package = NULL;
+	backend->priv->allow_cancel = PK_BACKEND_TRISTATE_UNSET;
 	backend->priv->status = PK_STATUS_ENUM_UNKNOWN;
 	backend->priv->exit = PK_EXIT_ENUM_UNKNOWN;
 	backend->priv->role = PK_ROLE_ENUM_UNKNOWN;
 	backend->priv->last_remaining = 0;
 	backend->priv->last_percentage = PK_BACKEND_PERCENTAGE_DEFAULT;
 	backend->priv->last_subpercentage = PK_BACKEND_PERCENTAGE_INVALID;
+	pk_store_reset (backend->priv->store);
 	pk_time_reset (backend->priv->time);
 
 	return TRUE;
@@ -1868,6 +2076,8 @@ pk_backend_reset (PkBackend *backend)
 static void
 pk_backend_init (PkBackend *backend)
 {
+	PkConf *conf;
+
 	backend->priv = PK_BACKEND_GET_PRIVATE (backend);
 	backend->priv->handle = NULL;
 	backend->priv->name = NULL;
@@ -1882,6 +2092,7 @@ pk_backend_init (PkBackend *backend)
 	backend->priv->signal_finished = 0;
 	backend->priv->signal_error_timeout = 0;
 	backend->priv->during_initialize = FALSE;
+	backend->priv->simultaneous = FALSE;
 	backend->priv->store = pk_store_new ();
 	backend->priv->time = pk_time_new ();
 	backend->priv->network = pk_network_new ();
@@ -1892,6 +2103,11 @@ pk_backend_init (PkBackend *backend)
 	g_signal_connect (backend->priv->file_monitor, "file-changed",
 			  G_CALLBACK (pk_backend_file_monitor_changed_cb), backend);
 
+	/* do we use time estimation? */
+	conf = pk_conf_new ();
+	backend->priv->use_time = pk_conf_get_bool (conf, "UseRemainingTimeEstimation");
+	g_object_unref (conf);
+
 	pk_backend_reset (backend);
 }
 
@@ -1919,6 +2135,7 @@ pk_backend_new (void)
 #include <glib/gstdio.h>
 
 static guint number_messages = 0;
+static guint number_packages = 0;
 
 /**
  * pk_backend_test_message_cb:
@@ -1953,6 +2170,9 @@ static gboolean
 pk_backend_test_func_true (PkBackend *backend)
 {
 	g_usleep (1000*1000);
+	/* trigger duplicate test */
+	pk_backend_package (backend, PK_INFO_ENUM_AVAILABLE, "vips-doc;7.12.4-2.fc8;noarch;linva", "The vips documentation package.");
+	pk_backend_package (backend, PK_INFO_ENUM_AVAILABLE, "vips-doc;7.12.4-2.fc8;noarch;linva", "The vips documentation package.");
 	pk_backend_finished (backend);
 	return TRUE;
 }
@@ -1964,13 +2184,25 @@ pk_backend_test_func_immediate_false (PkBackend *backend)
 	return FALSE;
 }
 
+/**
+ * pk_backend_test_package_cb:
+ **/
+static void
+pk_backend_test_package_cb (PkBackend *backend, const PkPackageObj *obj, EggTest *test)
+{
+	egg_debug ("package:%s", obj->id->name);
+	number_packages++;
+}
+
 void
 pk_backend_test (EggTest *test)
 {
 	PkBackend *backend;
+	PkConf *conf;
 	gchar *text;
 	gboolean ret;
 	const gchar *filename;
+	gboolean developer_mode;
 
 	if (!egg_test_start (test, "PkBackend"))
 		return;
@@ -1983,6 +2215,10 @@ pk_backend_test (EggTest *test)
 	else
 		egg_test_failed (test, NULL);
 
+	/* connect */
+	g_signal_connect (backend, "package",
+			  G_CALLBACK (pk_backend_test_package_cb), test);
+
 	/************************************************************/
 	egg_test_title (test, "create a config file");
 	filename = "/tmp/dave";
@@ -2164,6 +2400,13 @@ pk_backend_test (EggTest *test)
 	egg_test_loop_wait (test, 2000);
 	egg_test_loop_check (test);
 
+	/************************************************************/
+	egg_test_title (test, "check duplicate filter");
+	if (number_packages == 1)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "wrong number of pacakges: %s", number_packages);
+
 	/* reset */
 	pk_backend_reset (backend);
 
@@ -2187,13 +2430,47 @@ pk_backend_test (EggTest *test)
 	egg_test_loop_wait (test, PK_BACKEND_FINISHED_ERROR_TIMEOUT + 400);
 	egg_test_loop_check (test);
 
-#ifdef PK_IS_DEVELOPER
-	egg_test_title (test, "check we enforce finished after error_code");
-	if (number_messages == 1)
-		egg_test_success (test, NULL);
-	else
-		egg_test_failed (test, "we messaged %i times!", number_messages);
-#endif
+	/************************************************************
+	 ****************     CANCEL TRISTATE      ******************
+	 ************************************************************/
+	egg_test_title (test, "get allow cancel after reset");
+	pk_backend_reset (backend);
+	ret = pk_backend_get_allow_cancel (backend);
+	egg_test_assert (test, !ret);
+
+	/************************************************************/
+	egg_test_title (test, "set allow cancel TRUE");
+	ret = pk_backend_set_allow_cancel (backend, TRUE);
+	egg_test_assert (test, ret);
+
+	/************************************************************/
+	egg_test_title (test, "set allow cancel TRUE (repeat)");
+	ret = pk_backend_set_allow_cancel (backend, TRUE);
+	egg_test_assert (test, !ret);
+
+	/************************************************************/
+	egg_test_title (test, "set allow cancel FALSE");
+	ret = pk_backend_set_allow_cancel (backend, FALSE);
+	egg_test_assert (test, ret);
+
+	/************************************************************/
+	egg_test_title (test, "set allow cancel FALSE (after reset)");
+	pk_backend_reset (backend);
+	ret = pk_backend_set_allow_cancel (backend, FALSE);
+	egg_test_assert (test, ret);
+
+	/* if running in developer mode, then expect a Message */
+	conf = pk_conf_new ();
+	developer_mode = pk_conf_get_bool (conf, "DeveloperMode");
+	g_object_unref (conf);
+	if (developer_mode) {
+		/************************************************************/
+		egg_test_title (test, "check we enforce finished after error_code");
+		if (number_messages == 1)
+			egg_test_success (test, NULL);
+		else
+			egg_test_failed (test, "we messaged %i times!", number_messages);
+	}
 
 	g_object_unref (backend);
 
