@@ -45,12 +45,10 @@
 #include "pk-network.h"
 #include "pk-cache.h"
 #include "pk-shared.h"
-#include "pk-update-detail-list.h"
 #include "pk-backend.h"
 #include "pk-backend-internal.h"
 #include "pk-engine.h"
 #include "pk-transaction.h"
-#include "pk-transaction-id.h"
 #include "pk-transaction-db.h"
 #include "pk-transaction-list.h"
 #include "pk-inhibit.h"
@@ -60,8 +58,6 @@
 #include "pk-security.h"
 #include "pk-conf.h"
 
-static void     pk_engine_class_init	(PkEngineClass *klass);
-static void     pk_engine_init		(PkEngine      *engine);
 static void     pk_engine_finalize	(GObject       *object);
 
 #define PK_ENGINE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_ENGINE, PkEnginePrivate))
@@ -91,18 +87,19 @@ static void     pk_engine_finalize	(GObject       *object);
 struct PkEnginePrivate
 {
 	GTimer			*timer;
-	gboolean		 restart_schedule;
+	gboolean		 notify_clients_of_upgrade;
+	gboolean		 shutdown_as_soon_as_possible;
 	PkTransactionList	*transaction_list;
 	PkTransactionDb		*transaction_db;
 	PkCache			*cache;
-	PkUpdateDetailList	*update_detail_cache;
 	PkBackend		*backend;
 	PkInhibit		*inhibit;
 	PkNetwork		*network;
 	PkSecurity		*security;
 	PkNotify		*notify;
 	PkConf			*conf;
-	PkFileMonitor		*file_monitor;
+	PkFileMonitor		*file_monitor_conf;
+	PkFileMonitor		*file_monitor_binary;
 	PkBitfield		 actions;
 	PkBitfield		 groups;
 	PkBitfield		 filters;
@@ -118,9 +115,18 @@ enum {
 	PK_ENGINE_NETWORK_STATE_CHANGED,
 	PK_ENGINE_RESTART_SCHEDULE,
 	PK_ENGINE_UPDATES_CHANGED,
+	PK_ENGINE_QUIT,
 	PK_ENGINE_LAST_SIGNAL
 };
 
+enum {
+	PROP_0,
+	PROP_VERSION_MAJOR,
+	PROP_VERSION_MINOR,
+	PROP_VERSION_MICRO,
+	PROP_LAST,
+};
+
 static guint	     signals [PK_ENGINE_LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (PkEngine, pk_engine, G_TYPE_OBJECT)
@@ -215,17 +221,6 @@ pk_engine_notify_repo_list_changed_cb (PkNotify *notify, PkEngine *engine)
 	g_signal_emit (engine, signals [PK_ENGINE_REPO_LIST_CHANGED], 0);
 }
 
-/**
- * pk_engine_notify_restart_schedule_cb:
- **/
-static void
-pk_engine_notify_restart_schedule_cb (PkNotify *notify, PkEngine *engine)
-{
-	g_return_if_fail (PK_IS_ENGINE (engine));
-	egg_debug ("emitting restart-schedule");
-	g_signal_emit (engine, signals [PK_ENGINE_RESTART_SCHEDULE], 0);
-}
-
 /**
  * pk_engine_notify_updates_changed_cb:
  **/
@@ -241,7 +236,7 @@ pk_engine_notify_updates_changed_cb (PkNotify *notify, PkEngine *engine)
  * pk_engine_finished_cb:
  **/
 static void
-pk_engine_finished_cb (PkBackend *backend, PkExitEnum exit, PkEngine *engine)
+pk_engine_finished_cb (PkBackend *backend, PkExitEnum exit_enum, PkEngine *engine)
 {
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
@@ -252,26 +247,29 @@ pk_engine_finished_cb (PkBackend *backend, PkExitEnum exit, PkEngine *engine)
 /**
  * pk_engine_get_tid:
  **/
-gboolean
-pk_engine_get_tid (PkEngine *engine, gchar **tid, GError **error)
+void
+pk_engine_get_tid (PkEngine *engine, DBusGMethodInvocation *context)
 {
 	gchar *new_tid;
 	gboolean ret;
+	gchar *sender = NULL;
 
-	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
+	g_return_if_fail (PK_IS_ENGINE (engine));
 
 	egg_debug ("GetTid method called");
-	new_tid = pk_transaction_id_generate ();
+	sender = dbus_g_method_get_sender (context);
+	new_tid = pk_transaction_db_generate_id (engine->priv->transaction_db);
 
-	ret = pk_transaction_list_create (engine->priv->transaction_list, new_tid);
+	ret = pk_transaction_list_create (engine->priv->transaction_list, new_tid, sender);
 	egg_debug ("sending tid: '%s'", new_tid);
-	*tid =  g_strdup (new_tid);
-	g_free (new_tid);
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
 
-	return TRUE;
+	/* return TID */
+	dbus_g_method_return (context, new_tid);
+	g_free (new_tid);
+	g_free (sender);
 }
 
 /**
@@ -292,6 +290,22 @@ pk_engine_get_network_state (PkEngine *engine, gchar **state, GError **error)
 	return TRUE;
 }
 
+/**
+ * pk_engine_get_daemon_state:
+ **/
+gboolean
+pk_engine_get_daemon_state (PkEngine *engine, gchar **state, GError **error)
+{
+	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
+
+	*state = pk_transaction_list_get_state (engine->priv->transaction_list);
+
+	/* reset the timer */
+	pk_engine_reset_timer (engine);
+
+	return TRUE;
+}
+
 /**
  * pk_engine_get_transaction_list:
  **/
@@ -360,10 +374,9 @@ pk_engine_state_has_changed (PkEngine *engine, const gchar *reason, GError **err
 
 	/* have we already scheduled priority? */
 	if (engine->priv->signal_state_priority_timeout != 0) {
-		g_set_error (error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_INVALID_STATE,
-			     "Already asked to refresh priority state less than %i seconds ago",
+		egg_warning ("Already asked to refresh priority state less than %i seconds ago",
 			     PK_ENGINE_STATE_CHANGED_PRIORITY_TIMEOUT);
-		return FALSE;
+		goto out;
 	}
 
 	/* don't bombard the user 10 seconds after resuming */
@@ -372,10 +385,9 @@ pk_engine_state_has_changed (PkEngine *engine, const gchar *reason, GError **err
 
 	/* are we normal, and already scheduled normal? */
 	if (!is_priority && engine->priv->signal_state_normal_timeout != 0) {
-		g_set_error (error, PK_ENGINE_ERROR, PK_ENGINE_ERROR_INVALID_STATE,
-			     "Already asked to refresh normal state less than %i seconds ago",
+		egg_warning ("Already asked to refresh normal state less than %i seconds ago",
 			     PK_ENGINE_STATE_CHANGED_NORMAL_TIMEOUT);
-		return FALSE;
+		goto out;
 	}
 
 	/* are we priority, and already scheduled normal? */
@@ -394,7 +406,7 @@ pk_engine_state_has_changed (PkEngine *engine, const gchar *reason, GError **err
 
 	/* reset the timer */
 	pk_engine_reset_timer (engine);
-
+out:
 	return TRUE;
 }
 
@@ -524,9 +536,15 @@ pk_engine_get_seconds_idle (PkEngine *engine)
 	}
 
 	/* have we been updated? */
-	if (engine->priv->restart_schedule) {
-		egg_debug ("need to restart daemon *NOW*");
-		pk_notify_restart_schedule (engine->priv->notify);
+	if (engine->priv->notify_clients_of_upgrade) {
+		egg_debug ("emitting restart-schedule because of binary change");
+		g_signal_emit (engine, signals [PK_ENGINE_RESTART_SCHEDULE], 0);
+		return G_MAXUINT;
+	}
+
+	/* do we need to shutdown quickly */
+	if (engine->priv->shutdown_as_soon_as_possible) {
+		egg_debug ("need to restart daemon asap");
 		return G_MAXUINT;
 	}
 
@@ -547,15 +565,15 @@ pk_engine_suggest_daemon_quit (PkEngine *engine, GError **error)
 	/* can we exit straight away */
 	size = pk_transaction_list_get_size (engine->priv->transaction_list);
 	if (size == 0) {
-		egg_warning ("exit!!");
-		exit (0);
+		egg_debug ("emitting quit");
+		g_signal_emit (engine, signals [PK_ENGINE_QUIT], 0);
 		return TRUE;
 	}
 
 	/* This will wait from 0..10 seconds, depending on the status of
 	 * pk_main_timeout_check_cb() - usually it should be a few seconds
 	 * after the last transaction */
-	engine->priv->restart_schedule = TRUE;
+	engine->priv->shutdown_as_soon_as_possible = TRUE;
 	return TRUE;
 }
 
@@ -616,6 +634,41 @@ out:
 	g_free (error_detail);
 }
 
+/**
+ * pk_engine_get_property:
+ **/
+static void
+pk_engine_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	case PROP_VERSION_MAJOR:
+		g_value_set_uint (value, PK_MAJOR_VERSION);
+		break;
+	case PROP_VERSION_MINOR:
+		g_value_set_uint (value, PK_MINOR_VERSION);
+		break;
+	case PROP_VERSION_MICRO:
+		g_value_set_uint (value, PK_MICRO_VERSION);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+/**
+ * pk_engine_set_property:
+ **/
+static void
+pk_engine_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+	switch (prop_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 /**
  * pk_engine_class_init:
  * @klass: The PkEngineClass
@@ -623,11 +676,38 @@ out:
 static void
 pk_engine_class_init (PkEngineClass *klass)
 {
+	GParamSpec *pspec;
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = pk_engine_finalize;
+	object_class->get_property = pk_engine_get_property;
+	object_class->set_property = pk_engine_set_property;
 
-	/* set up signal that emits 'au' */
+	/**
+	 * PkEngine:version-major:
+	 */
+	pspec = g_param_spec_uint ("version-major", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MAJOR, pspec);
+
+	/**
+	 * PkEngine:version-minor:
+	 */
+	pspec = g_param_spec_uint ("version-minor", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MINOR, pspec);
+
+	/**
+	 * PkEngine:version-micro:
+	 */
+	pspec = g_param_spec_uint ("version-micro", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VERSION_MICRO, pspec);
+
+	/* signals */
 	signals [PK_ENGINE_LOCKED] =
 		g_signal_new ("locked",
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
@@ -658,26 +738,44 @@ pk_engine_class_init (PkEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
 			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	signals [PK_ENGINE_QUIT] =
+		g_signal_new ("quit",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 
 	g_type_class_add_private (klass, sizeof (PkEnginePrivate));
 }
 
 /**
- * pk_engine_file_monitor_changed_cb:
+ * pk_engine_conf_file_changed_cb:
+ *
+ * A config file has changed, we need to reload the daemon
  **/
 static void
-pk_engine_file_monitor_changed_cb (PkFileMonitor *file_monitor, PkEngine *engine)
+pk_engine_conf_file_changed_cb (PkFileMonitor *file_monitor, PkEngine *engine)
 {
 	g_return_if_fail (PK_IS_ENGINE (engine));
-	egg_debug ("setting restart_schedule TRUE");
-	engine->priv->restart_schedule = TRUE;
+	egg_debug ("setting shutdown_as_soon_as_possible TRUE");
+	engine->priv->shutdown_as_soon_as_possible = TRUE;
+}
+
+/**
+ * pk_engine_binary_file_changed_cb:
+ **/
+static void
+pk_engine_binary_file_changed_cb (PkFileMonitor *file_monitor, PkEngine *engine)
+{
+	g_return_if_fail (PK_IS_ENGINE (engine));
+	egg_debug ("setting notify_clients_of_upgrade TRUE");
+	engine->priv->notify_clients_of_upgrade = TRUE;
 }
 
 /**
  * pk_engine_network_state_changed_cb:
  **/
 static void
-pk_engine_network_state_changed_cb (PkNetwork *file_monitor, PkNetworkEnum state, PkEngine *engine)
+pk_engine_network_state_changed_cb (PkNetwork *network, PkNetworkEnum state, PkEngine *engine)
 {
 	const gchar *state_text;
 	g_return_if_fail (PK_IS_ENGINE (engine));
@@ -699,7 +797,8 @@ pk_engine_init (PkEngine *engine)
 	gchar *proxy_ftp;
 
 	engine->priv = PK_ENGINE_GET_PRIVATE (engine);
-	engine->priv->restart_schedule = FALSE;
+	engine->priv->notify_clients_of_upgrade = FALSE;
+	engine->priv->shutdown_as_soon_as_possible = FALSE;
 	engine->priv->mime_types = NULL;
 
 	/* use the config file */
@@ -739,7 +838,6 @@ pk_engine_init (PkEngine *engine)
 
 	/* we save a cache of the latest update lists sowe can do cached responses */
 	engine->priv->cache = pk_cache_new ();
-	engine->priv->update_detail_cache = pk_update_detail_list_new ();
 
 	/* we need to be able to clear this */
 	engine->priv->signal_state_priority_timeout = 0;
@@ -752,21 +850,25 @@ pk_engine_init (PkEngine *engine)
 
 	/* add the interface */
 	engine->priv->notify = pk_notify_new ();
-	g_signal_connect (engine->priv->notify, "restart-schedule",
-			  G_CALLBACK (pk_engine_notify_restart_schedule_cb), engine);
 	g_signal_connect (engine->priv->notify, "repo-list-changed",
 			  G_CALLBACK (pk_engine_notify_repo_list_changed_cb), engine);
 	g_signal_connect (engine->priv->notify, "updates-changed",
 			  G_CALLBACK (pk_engine_notify_updates_changed_cb), engine);
 
 	/* monitor the config file for changes */
-	engine->priv->file_monitor = pk_file_monitor_new ();
+	engine->priv->file_monitor_conf = pk_file_monitor_new ();
 	filename = pk_conf_get_filename ();
-	pk_file_monitor_set_file (engine->priv->file_monitor, filename);
-	g_signal_connect (engine->priv->file_monitor, "file-changed",
-			  G_CALLBACK (pk_engine_file_monitor_changed_cb), engine);
+	pk_file_monitor_set_file (engine->priv->file_monitor_conf, filename);
+	g_signal_connect (engine->priv->file_monitor_conf, "file-changed",
+			  G_CALLBACK (pk_engine_conf_file_changed_cb), engine);
 	g_free (filename);
 
+	/* monitor the binary file for changes */
+	engine->priv->file_monitor_binary = pk_file_monitor_new ();
+	pk_file_monitor_set_file (engine->priv->file_monitor_binary, SBINDIR "/packagekitd");
+	g_signal_connect (engine->priv->file_monitor_binary, "file-changed",
+			  G_CALLBACK (pk_engine_binary_file_changed_cb), engine);
+
 	/* set the proxy */
 	proxy_http = pk_conf_get_string (engine->priv->conf, "ProxyHTTP");
 	proxy_ftp = pk_conf_get_string (engine->priv->conf, "ProxyFTP");
@@ -820,7 +922,8 @@ pk_engine_finalize (GObject *object)
 
 	/* compulsory gobjects */
 	g_timer_destroy (engine->priv->timer);
-	g_object_unref (engine->priv->file_monitor);
+	g_object_unref (engine->priv->file_monitor_conf);
+	g_object_unref (engine->priv->file_monitor_binary);
 	g_object_unref (engine->priv->inhibit);
 	g_object_unref (engine->priv->transaction_list);
 	g_object_unref (engine->priv->transaction_db);
@@ -829,7 +932,6 @@ pk_engine_finalize (GObject *object)
 	g_object_unref (engine->priv->notify);
 	g_object_unref (engine->priv->backend);
 	g_object_unref (engine->priv->cache);
-	g_object_unref (engine->priv->update_detail_cache);
 	g_object_unref (engine->priv->conf);
 	g_free (engine->priv->mime_types);
 
@@ -855,30 +957,107 @@ pk_engine_new (void)
 #ifdef EGG_TEST
 #include "egg-test.h"
 
+static PkNotify *notify = NULL;
+static gboolean _quit = FALSE;
+static gboolean _locked = FALSE;
+static gboolean _restart_schedule = FALSE;
+
+/**
+ * pk_test_quit_cb:
+ **/
+static void
+pk_test_quit_cb (PkEngine *engine, EggTest *test)
+{
+	_quit = TRUE;
+}
+
+/**
+ * pk_test_locked_cb:
+ **/
+static void
+pk_test_locked_cb (PkEngine *engine, gboolean is_locked, EggTest *test)
+{
+	_locked = is_locked;
+}
+
+/**
+ * pk_test_updates_changed_cb:
+ **/
+static void
+pk_test_updates_changed_cb (PkEngine *engine, EggTest *test)
+{
+	egg_test_loop_quit (test);
+}
+
+/**
+ * pk_test_repo_list_changed_cb:
+ **/
+static void
+pk_test_repo_list_changed_cb (PkEngine *engine, EggTest *test)
+{
+	egg_test_loop_quit (test);
+}
+
+/**
+ * pk_test_restart_schedule_cb:
+ **/
+static void
+pk_test_restart_schedule_cb (PkEngine *engine, EggTest *test)
+{
+	_restart_schedule = TRUE;
+	egg_test_loop_quit (test);
+}
+
+/**
+ * pk_test_emit_updates_changed_cb:
+ **/
+static gboolean
+pk_test_emit_updates_changed_cb (EggTest *test)
+{
+	PkNotify *notify2;
+	notify2 = pk_notify_new ();
+	pk_notify_updates_changed (notify2);
+	g_object_unref (notify2);
+	return FALSE;
+}
+
+/**
+ * pk_test_emit_repo_list_changed_cb:
+ **/
+static gboolean
+pk_test_emit_repo_list_changed_cb (EggTest *test)
+{
+	PkNotify *notify2;
+	notify2 = pk_notify_new ();
+	pk_notify_repo_list_changed (notify2);
+	g_object_unref (notify2);
+	return FALSE;
+}
+
 void
 pk_engine_test (EggTest *test)
 {
 	gboolean ret;
 	PkEngine *engine;
 	PkBackend *backend;
+	PkInhibit *inhibit;
 	guint idle;
-	gchar *tid;
 	gchar *actions;
+	guint elapsed;
 
 	if (!egg_test_start (test, "PkEngine"))
 		return;
 
-	/* don't do these when doing make distcheck */
-#ifndef PK_IS_DEVELOPER
-	egg_test_end (test);
-	return;
-#endif
-
 	/************************************************************/
 	egg_test_title (test, "get a backend instance");
 	backend = pk_backend_new ();
 	egg_test_assert (test, backend != NULL);
 
+	/************************************************************/
+	egg_test_title (test, "get a notify instance");
+	notify = pk_notify_new ();
+	egg_test_assert (test, notify != NULL);
+
 	/* set the type, as we have no pk-main doing this for us */
 	/************************************************************/
 	egg_test_title (test, "set the backend name");
@@ -890,7 +1069,21 @@ pk_engine_test (EggTest *test)
 	engine = pk_engine_new ();
 	egg_test_assert (test, engine != NULL);
 
-	/************************************************************/
+	/* connect up signals */
+	g_signal_connect (engine, "quit",
+			  G_CALLBACK (pk_test_quit_cb), test);
+	g_signal_connect (engine, "locked",
+			  G_CALLBACK (pk_test_locked_cb), test);
+	g_signal_connect (engine, "updates-changed",
+			  G_CALLBACK (pk_test_updates_changed_cb), test);
+	g_signal_connect (engine, "repo-list-changed",
+			  G_CALLBACK (pk_test_repo_list_changed_cb), test);
+	g_signal_connect (engine, "restart-schedule",
+			  G_CALLBACK (pk_test_restart_schedule_cb), test);
+
+	/************************************************************
+	 **********           GET IDLE TIMES              ***********
+	 ************************************************************/
 	egg_test_title (test, "get idle at startup");
 	idle = pk_engine_get_seconds_idle (engine);
 	if (idle < 1)
@@ -919,14 +1112,93 @@ pk_engine_test (EggTest *test)
 	else
 		egg_test_failed (test, "idle = %i", idle);
 
+	/************************************************************
+	 **********        TEST PROXY NOTIFY              ***********
+	 ************************************************************/
+	egg_test_title (test, "force test notify updates-changed");
+	g_timeout_add (25, (GSourceFunc) pk_test_emit_updates_changed_cb, test);
+	egg_test_success (test, NULL);
+	egg_test_loop_wait (test, 50);
+	egg_test_loop_check (test);
+
 	/************************************************************/
-	egg_test_title (test, "create a tid we never use");
-	ret = pk_engine_get_tid (engine, &tid, NULL);
-	egg_test_assert (test, ret);
-	egg_test_title_assert (test, "tid is non-null", tid != NULL);
-	g_free (tid);
+	egg_test_title (test, "force test notify repo-list-changed");
+	g_timeout_add (25, (GSourceFunc) pk_test_emit_repo_list_changed_cb, test);
+	egg_test_success (test, NULL);
+	egg_test_loop_wait (test, 50);
+	egg_test_loop_check (test);
+
+	/************************************************************/
+	egg_test_title (test, "force test notify wait updates-changed");
+	pk_notify_wait_updates_changed (notify, 500);
+	egg_test_loop_wait (test, 1000);
+	elapsed = egg_test_elapsed (test);
+	if (elapsed > 400 && elapsed < 600)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "failed to quit (%i)", elapsed);
+
+	/************************************************************
+	 **********               LOCKING                 ***********
+	 ************************************************************/
+	egg_test_title (test, "test locked");
+	inhibit = pk_inhibit_new ();
+	pk_inhibit_add (inhibit, GUINT_TO_POINTER (999));
+	if (_locked)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "not locked");
+
+	/************************************************************/
+	egg_test_title (test, "test locked");
+	pk_inhibit_remove (inhibit, GUINT_TO_POINTER (999));
+	if (!_locked)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "not locked");
+	g_object_unref (inhibit);
+
+	/************************************************************/
+	egg_test_title (test, "test not locked");
+	if (!_locked)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "still locked");
+
+	/************************************************************
+	 **********          BINARY UPGRADE TEST          ***********
+	 ************************************************************/
+	egg_test_title_assert (test, "restart_schedule not set", !_restart_schedule);
+	ret = g_file_set_contents (SBINDIR "/packagekitd", "overwrite", -1, NULL);
+
+	/************************************************************/
+	egg_test_title_assert (test, "touched binary file", ret);
+	egg_test_loop_wait (test, 5000);
+
+	/************************************************************/
+	egg_test_title (test, "get idle after we touched the binary");
+	idle = pk_engine_get_seconds_idle (engine);
+	if (idle == G_MAXUINT)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "idle = %i", idle);
+
+	/************************************************************/
+	egg_test_title_assert (test, "restart_schedule set", _restart_schedule);
+
+	/************************************************************
+	 **********             DAEMON QUIT               ***********
+	 ************************************************************/
+	egg_test_title_assert (test, "not already quit", !_quit);
+	egg_test_title (test, "suggest quit with no transactions (should get quit signal)");
+	pk_engine_suggest_daemon_quit (engine, NULL);
+	if (_quit)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "did not quit");
 
 	g_object_unref (backend);
+	g_object_unref (notify);
 	g_object_unref (engine);
 
 	egg_test_end (test);
