@@ -65,8 +65,8 @@ typedef enum
 
 
 GQuark		 pk_engine_error_quark			(void);
-GType		 pk_engine_error_get_type		(void) G_GNUC_CONST;
-GType		 pk_engine_get_type		  	(void) G_GNUC_CONST;
+GType		 pk_engine_error_get_type		(void);
+GType		 pk_engine_get_type		  	(void);
 PkEngine	*pk_engine_new				(void);
 
 /* general */
@@ -89,12 +89,14 @@ gboolean	 pk_engine_get_groups			(PkEngine	*engine,
 gboolean	 pk_engine_get_mime_types		(PkEngine	*engine,
 							 gchar		**types,
 							 GError		**error);
-gboolean	 pk_engine_get_tid			(PkEngine	*engine,
-							 gchar		**tid,
-							 GError		**error);
+void		 pk_engine_get_tid			(PkEngine	*engine,
+							 DBusGMethodInvocation *context);
 gboolean	 pk_engine_get_network_state		(PkEngine	*engine,
 							 gchar		**state,
 							 GError		**error);
+gboolean	 pk_engine_get_daemon_state		(PkEngine	*engine,
+							 gchar		**state,
+							 GError		**error);
 gboolean	 pk_engine_get_time_since_action	(PkEngine	*engine,
 							 const gchar	*role_text,
 							 guint		*seconds,
