@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <packagekit-glib/packagekit.h>
+#include <gio/gdesktopappinfo.h>
 #include <sqlite3.h>
 
 #ifdef USE_SECURITY_POLKIT
@@ -36,10 +37,6 @@
   #include <polkit-dbus/polkit-dbus.h>
 #endif
 
-#ifdef PK_BUILD_GIO
-  #include <gio/gdesktopappinfo.h>
-#endif
-
 #include "egg-debug.h"
 
 #include "pk-post-trans.h"
@@ -74,7 +71,7 @@ G_DEFINE_TYPE (PkPostTrans, pk_post_trans, G_TYPE_OBJECT)
  * pk_post_trans_finished_cb:
  **/
 static void
-pk_post_trans_finished_cb (PkBackend *backend, PkExitEnum exit, PkPostTrans *post)
+pk_post_trans_finished_cb (PkBackend *backend, PkExitEnum exit_enum, PkPostTrans *post)
 {
 	if (g_main_loop_is_running (post->priv->loop))
 		g_main_loop_quit (post->priv->loop);
@@ -225,8 +222,7 @@ pk_post_trans_sqlite_add_filename_details (PkPostTrans *post, const gchar *filen
 	gchar *error_msg = NULL;
 	sqlite3_stmt *sql_statement = NULL;
 	gint rc = -1;
-	gint show = TRUE;
-#ifdef PK_BUILD_GIO
+	gint show;
 	GDesktopAppInfo *info;
 
 	/* find out if we should show desktop file in menus */
@@ -235,9 +231,8 @@ pk_post_trans_sqlite_add_filename_details (PkPostTrans *post, const gchar *filen
 		egg_warning ("could not load desktop file %s", filename);
 		goto out;
 	}
-	show = g_app_info_should_show (G_APP_INFO(info));
+	show = g_app_info_should_show (G_APP_INFO (info));
 	g_object_unref (info);
-#endif
 
 	egg_debug ("add filename %s from %s with md5: %s (show: %i)", filename, package, md5, show);
 
@@ -518,7 +513,6 @@ pk_post_trans_update_files_check_running_cb (PkBackend *backend, const gchar *pa
 	guint len;
 	gboolean ret;
 	gchar **files;
-	gchar *details;
 	PkPackageId *id;
 
 	id = pk_package_id_new_from_string (package_id);
@@ -541,9 +535,8 @@ pk_post_trans_update_files_check_running_cb (PkBackend *backend, const gchar *pa
 		 * suggest an application restart instead */
 
 		/* send signal about session restart */
-		details = g_strdup_printf ("package %s updated, and %s is running", id->name, files[i]);
-		pk_backend_require_restart (post->priv->backend, PK_RESTART_ENUM_SESSION, details);
-		g_free (details);
+		egg_debug ("package %s updated, and %s is running", id->name, files[i]);
+		pk_backend_require_restart (post->priv->backend, PK_RESTART_ENUM_SESSION, package_id);
 	}
 	g_strfreev (files);
 	pk_package_id_free (id);
@@ -571,6 +564,7 @@ pk_post_trans_update_process_list (PkPostTrans *post)
 	name = g_dir_read_name (dir);
 	pk_obj_list_clear (PK_OBJ_LIST(post->priv->running_exec_list));
 	while (name != NULL) {
+		contents = NULL;
 		uid_file = g_build_filename ("/proc", name, "loginuid", NULL);
 
 		/* is a process file */
@@ -603,6 +597,7 @@ pk_post_trans_update_process_list (PkPostTrans *post)
 		pk_obj_list_add (PK_OBJ_LIST(post->priv->running_exec_list), exec);
 out:
 		g_free (uid_file);
+		g_free (contents);
 		name = g_dir_read_name (dir);
 	}
 	g_dir_close (dir);
