@@ -50,8 +50,8 @@
 #include "pk-marshal.h"
 #include "pk-conf.h"
 
-static void     pk_spawn_class_init	(PkSpawnClass *klass);
-static void     pk_spawn_init		(PkSpawn      *spawn);
+#include "pk-sysdep.h"
+
 static void     pk_spawn_finalize	(GObject       *object);
 
 #define PK_SPAWN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_SPAWN, PkSpawnPrivate))
@@ -69,6 +69,7 @@ struct PkSpawnPrivate
 	gboolean		 finished;
 	gboolean		 is_sending_exit;
 	gboolean		 is_changing_dispatcher;
+	gboolean		 allow_sigkill;
 	PkSpawnExitType		 exit;
 	GString			*stdout_buf;
 	GString			*stderr_buf;
@@ -144,6 +145,27 @@ pk_spawn_emit_whole_lines (PkSpawn *spawn, GString *string)
 	return TRUE;
 }
 
+/**
+ * pk_spawn_exit_type_enum_to_text:
+ **/
+static const gchar *
+pk_spawn_exit_type_enum_to_text (PkSpawnExitType type)
+{
+	if (type == PK_SPAWN_EXIT_TYPE_SUCCESS)
+		return "success";
+	if (type == PK_SPAWN_EXIT_TYPE_FAILED)
+		return "failed";
+	if (type == PK_SPAWN_EXIT_TYPE_DISPATCHER_CHANGED)
+		return "dispatcher-changed";
+	if (type == PK_SPAWN_EXIT_TYPE_DISPATCHER_EXIT)
+		return "dispatcher-exit";
+	if (type == PK_SPAWN_EXIT_TYPE_SIGQUIT)
+		return "sigquit";
+	if (type == PK_SPAWN_EXIT_TYPE_SIGKILL)
+		return "sigkill";
+	return "unknown";
+}
+
 /**
  * pk_spawn_check_child:
  **/
@@ -220,7 +242,7 @@ pk_spawn_check_child (PkSpawn *spawn)
 		spawn->priv->exit = PK_SPAWN_EXIT_TYPE_DISPATCHER_EXIT;
 
 	/* don't emit if we just closed an invalid dispatcher */
-	egg_debug ("emitting exit %i", spawn->priv->exit);
+	egg_debug ("emitting exit %s", pk_spawn_exit_type_enum_to_text (spawn->priv->exit));
 	g_signal_emit (spawn, signals [PK_SPAWN_EXIT], 0, spawn->priv->exit);
 
 	return FALSE;
@@ -257,11 +279,22 @@ pk_spawn_sigkill_cb (PkSpawn *spawn)
 	return FALSE;
 }
 
+/**
+ * pk_spawn_is_running:
+ *
+ * Is this instance controlling a script?
+ *
+ **/
+gboolean
+pk_spawn_is_running (PkSpawn *spawn)
+{
+	return (spawn->priv->child_pid != -1);
+}
+
 /**
  * pk_spawn_kill:
  *
- * We send SIGQUIT and after a few ms SIGKILL
- *
+ * We send SIGQUIT and after a few ms SIGKILL (if allowed)
  **/
 gboolean
 pk_spawn_kill (PkSpawn *spawn)
@@ -269,6 +302,7 @@ pk_spawn_kill (PkSpawn *spawn)
 	gint retval;
 
 	g_return_val_if_fail (PK_IS_SPAWN (spawn), FALSE);
+	g_return_val_if_fail (spawn->priv->kill_id == 0, FALSE);
 
 	/* check if process has already gone */
 	if (spawn->priv->finished) {
@@ -290,7 +324,26 @@ pk_spawn_kill (PkSpawn *spawn)
 	}
 
 	/* the program might not be able to handle SIGQUIT, give it a few seconds and then SIGKILL it */
-	spawn->priv->kill_id = g_timeout_add (PK_SPAWN_SIGKILL_DELAY, (GSourceFunc) pk_spawn_sigkill_cb, spawn);
+	if (spawn->priv->allow_sigkill)
+		spawn->priv->kill_id = g_timeout_add (PK_SPAWN_SIGKILL_DELAY, (GSourceFunc) pk_spawn_sigkill_cb, spawn);
+
+	return TRUE;
+}
+
+/**
+ * pk_spawn_set_allow_sigkill:
+ *
+ * Set whether the spawned backends are allowed to be SIGKILLed if they do not
+ * respond to SIGQUIT. This ensures that Cancel() works as expected, but
+ * somtimes can corrupt databases if they are open.
+ **/
+gboolean
+pk_spawn_set_allow_sigkill (PkSpawn *spawn, gboolean allow_sigkill)
+{
+	g_return_val_if_fail (PK_IS_SPAWN (spawn), FALSE);
+
+	egg_debug ("setting SIGKILL: %i", allow_sigkill);
+	spawn->priv->allow_sigkill = allow_sigkill;
 
 	return TRUE;
 }
@@ -328,7 +381,7 @@ pk_spawn_send_stdin (PkSpawn *spawn, const gchar *command)
 	/* write to the waiting process */
 	wrote = write (spawn->priv->stdin_fd, buffer, length);
 	if (wrote != length) {
-		egg_warning ("wrote %i/%i bytes on fd %i", wrote, length, spawn->priv->stdin_fd);
+		egg_warning ("wrote %i/%i bytes on fd %i (%s)", wrote, length, spawn->priv->stdin_fd, strerror (errno));
 		ret = FALSE;
 	}
 out:
@@ -374,7 +427,8 @@ pk_spawn_exit (PkSpawn *spawn)
 	} while (ret && count++ < 50);
 
 	/* the script exited okay */
-	ret = TRUE;
+	if (count < 50)
+		ret = TRUE;
 out:
 	spawn->priv->is_sending_exit = FALSE;
 	return ret;
@@ -390,9 +444,10 @@ gboolean
 pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 {
 	gboolean ret;
+	gboolean idleio;
 	guint i;
 	guint len;
-	gint nice;
+	gint nice_value;
 	gchar *command;
 
 	g_return_val_if_fail (PK_IS_SPAWN (spawn), FALSE);
@@ -440,7 +495,15 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 		/* kill off existing instance */
 		egg_debug ("changing dispatcher (exit old instance)");
 		spawn->priv->is_changing_dispatcher = TRUE;
-		pk_spawn_exit (spawn);
+		ret = pk_spawn_exit (spawn);
+		if (!ret) {
+			egg_warning ("failed to exit previous instance");
+			/* remove poll, as we can't reply on pk_spawn_check_child() */
+			if (spawn->priv->poll_id != 0) {
+				g_source_remove (spawn->priv->poll_id);
+				spawn->priv->poll_id = 0;
+			}
+		}
 		spawn->priv->is_changing_dispatcher = FALSE;
 	}
 
@@ -456,13 +519,20 @@ pk_spawn_argv (PkSpawn *spawn, gchar **argv, gchar **envp)
 				 NULL);
 
 	/* get the nice value and ensure we are in the valid range */
-	nice = pk_conf_get_int (spawn->priv->conf, "BackendSpawnNiceValue");
-	nice = CLAMP(nice, -20, 19);
+	nice_value = pk_conf_get_int (spawn->priv->conf, "BackendSpawnNiceValue");
+	nice_value = CLAMP(nice_value, -20, 19);
 
 	/* don't completely bog the system down */
-	if (nice != 0) {
-		egg_debug ("renice to %i", nice);
-		setpriority (PRIO_PROCESS, spawn->priv->child_pid, nice);
+	if (nice_value != 0) {
+		egg_debug ("renice to %i", nice_value);
+		setpriority (PRIO_PROCESS, spawn->priv->child_pid, nice_value);
+	}
+
+	/* perhaps set idle IO priority */
+	idleio = pk_conf_get_bool (spawn->priv->conf, "BackendSpawnIdleIO");
+	if (idleio) {
+		egg_debug ("setting ioprio class to idle");
+		pk_ioprio_set_idle (spawn->priv->child_pid);
 	}
 
 	/* we failed to invoke the helper */
@@ -541,6 +611,7 @@ pk_spawn_init (PkSpawn *spawn)
 	spawn->priv->finished = FALSE;
 	spawn->priv->is_sending_exit = FALSE;
 	spawn->priv->is_changing_dispatcher = FALSE;
+	spawn->priv->allow_sigkill = TRUE;
 	spawn->priv->last_argv0 = NULL;
 	spawn->priv->last_envp = NULL;
 	spawn->priv->exit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
@@ -567,16 +638,25 @@ pk_spawn_finalize (GObject *object)
 	g_return_if_fail (spawn->priv != NULL);
 
 	/* disconnect the poll in case we were cancelled before completion */
-	if (spawn->priv->poll_id != 0)
+	if (spawn->priv->poll_id != 0) {
 		g_source_remove (spawn->priv->poll_id);
+		spawn->priv->poll_id = 0;
+	}
 
 	/* disconnect the SIGKILL check */
-	if (spawn->priv->kill_id != 0)
+	if (spawn->priv->kill_id != 0) {
 		g_source_remove (spawn->priv->kill_id);
+		spawn->priv->kill_id = 0;
+	}
 
 	/* still running? */
-	if (spawn->priv->stdin_fd != -1)
+	if (spawn->priv->stdin_fd != -1) {
+		egg_warning ("killing as still running");
 		pk_spawn_kill (spawn);
+		/* just hope the script responded to SIGQUIT */
+		if (spawn->priv->kill_id != 0)
+			g_source_remove (spawn->priv->kill_id);
+	}
 
 	/* free the buffers */
 	g_string_free (spawn->priv->stdout_buf, TRUE);
@@ -775,7 +855,7 @@ pk_spawn_test (EggTest *test)
 	/************************************************************
 	 **********           Killing tests               ***********
 	 ************************************************************/
-	egg_test_title (test, "make sure run correct helper, and kill it");
+	egg_test_title (test, "make sure run correct helper, and cancel it using SIGKILL");
 	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
 	path = egg_test_get_data_file ("pk-spawn-test.sh");
 	argv = g_strsplit (path, " ", 0);
@@ -792,6 +872,35 @@ pk_spawn_test (EggTest *test)
 	egg_test_loop_wait (test, 5000);
 	egg_test_loop_check (test);
 
+	/************************************************************/
+	egg_test_title (test, "make sure finished in SIGKILL");
+	if (mexit == PK_SPAWN_EXIT_TYPE_SIGKILL)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "finish %i!", mexit);
+
+	/* get new object */
+	new_spawn_object (test, &spawn);
+
+	/************************************************************/
+	egg_test_title (test, "make sure dumb helper ignores SIGQUIT");
+	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
+	path = egg_test_get_data_file ("pk-spawn-test.sh");
+	argv = g_strsplit (path, " ", 0);
+	pk_spawn_set_allow_sigkill (spawn, FALSE);
+	ret = pk_spawn_argv (spawn, argv, NULL);
+	g_free (path);
+	g_strfreev (argv);
+	if (ret)
+		egg_test_success (test, NULL);
+	else
+		egg_test_failed (test, "did not run helper");
+
+	g_timeout_add_seconds (1, cancel_cb, spawn);
+	/* wait for finished */
+	egg_test_loop_wait (test, 10000);
+	egg_test_loop_check (test);
+
 	/************************************************************/
 	egg_test_title (test, "make sure finished in SIGQUIT");
 	if (mexit == PK_SPAWN_EXIT_TYPE_SIGQUIT)
@@ -803,7 +912,7 @@ pk_spawn_test (EggTest *test)
 	new_spawn_object (test, &spawn);
 
 	/************************************************************/
-	egg_test_title (test, "make sure run correct helper, and quit it");
+	egg_test_title (test, "make sure run correct helper, and SIGQUIT it");
 	mexit = PK_SPAWN_EXIT_TYPE_UNKNOWN;
 	path = egg_test_get_data_file ("pk-spawn-test-sigquit.sh");
 	argv = g_strsplit (path, " ", 0);
@@ -850,7 +959,8 @@ pk_spawn_test (EggTest *test)
 	file = egg_test_get_data_file ("pk-spawn-dispatcher.py");
 	path = g_strdup_printf ("%s\tsearch-name\tnone\tpower manager", file);
 	argv = g_strsplit (path, "\t", 0);
-	ret = pk_spawn_argv (spawn, argv, NULL);
+	envp = g_strsplit ("NETWORK=TRUE LANG=C", " ", 0);
+	ret = pk_spawn_argv (spawn, argv, envp);
 	g_free (file);
 	g_free (path);
 	if (ret)
@@ -884,7 +994,7 @@ pk_spawn_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "run the dispatcher with new input");
-	ret = pk_spawn_argv (spawn, argv, NULL);
+	ret = pk_spawn_argv (spawn, argv, envp);
 	if (ret)
 		egg_test_success (test, NULL);
 	else
@@ -945,6 +1055,7 @@ pk_spawn_test (EggTest *test)
 		egg_test_failed (test, "dispatcher closed twice");
 
 	g_strfreev (argv);
+	g_strfreev (envp);
 	g_object_unref (spawn);
 
 	egg_test_end (test);
