@@ -63,15 +63,18 @@ typedef enum {
 	PK_SPAWN_EXIT_TYPE_UNKNOWN
 } PkSpawnExitType;
 
-GType		 pk_spawn_get_type		  	(void) G_GNUC_CONST;
+GType		 pk_spawn_get_type		  	(void);
 PkSpawn		*pk_spawn_new				(void);
 
 gboolean	 pk_spawn_argv				(PkSpawn	*spawn,
 							 gchar		**argv,
 							 gchar		**envp)
 							 G_GNUC_WARN_UNUSED_RESULT;
+gboolean	 pk_spawn_is_running			(PkSpawn	*spawn);
 gboolean	 pk_spawn_kill				(PkSpawn	*spawn);
 gboolean	 pk_spawn_exit				(PkSpawn	*spawn);
+gboolean	 pk_spawn_set_allow_sigkill		(PkSpawn	*spawn,
+							 gboolean	 allow_sigkill);
 
 G_END_DECLS
 
