@@ -43,8 +43,6 @@
 #include "pk-time.h"
 #include "pk-marshal.h"
 
-static void     pk_time_class_init	(PkTimeClass *klass);
-static void     pk_time_init		(PkTime      *time);
 static void     pk_time_finalize	(GObject     *object);
 
 #define PK_TIME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_TIME, PkTimePrivate))
@@ -81,28 +79,28 @@ G_DEFINE_TYPE (PkTime, pk_time, G_TYPE_OBJECT)
  * Return value: if we set the average limits correctly
  **/
 gboolean
-pk_time_set_average_limits (PkTime *time, guint average_min, guint average_max)
+pk_time_set_average_limits (PkTime *self, guint average_min, guint average_max)
 {
-	g_return_val_if_fail (PK_IS_TIME (time), FALSE);
-	time->priv->average_min = average_min;
-	time->priv->average_max = average_max;
+	g_return_val_if_fail (PK_IS_TIME (self), FALSE);
+	self->priv->average_min = average_min;
+	self->priv->average_max = average_max;
 	return TRUE;
 }
 
 /**
  * pk_time_set_value_limits:
- * @time: This class instance
+ * @self: This class instance
  * @average_min: the smallest value that is acceptable for time (in seconds)
  * @average_max: the largest value that is acceptable for time (in seconds)
  *
  * Return value: if we set the value limits correctly
  **/
 gboolean
-pk_time_set_value_limits (PkTime *time, guint value_min, guint value_max)
+pk_time_set_value_limits (PkTime *self, guint value_min, guint value_max)
 {
-	g_return_val_if_fail (PK_IS_TIME (time), FALSE);
-	time->priv->value_min = value_min;
-	time->priv->value_max = value_max;
+	g_return_val_if_fail (PK_IS_TIME (self), FALSE);
+	self->priv->value_min = value_min;
+	self->priv->value_max = value_max;
 	return TRUE;
 }
 
@@ -112,15 +110,15 @@ pk_time_set_value_limits (PkTime *time, guint value_min, guint value_max)
  * Returns time running in ms
  **/
 guint
-pk_time_get_elapsed (PkTime *time)
+pk_time_get_elapsed (PkTime *self)
 {
 	gdouble elapsed;
 
-	g_return_val_if_fail (PK_IS_TIME (time), 0);
+	g_return_val_if_fail (PK_IS_TIME (self), 0);
 
-	elapsed = g_timer_elapsed (time->priv->timer, NULL);
+	elapsed = g_timer_elapsed (self->priv->timer, NULL);
 	elapsed *= 1000;
-	elapsed += time->priv->time_offset;
+	elapsed += self->priv->time_offset;
 
 	return (guint) elapsed;
 }
@@ -144,7 +142,7 @@ pk_time_get_gradient (PkTimeItem *item1, PkTimeItem *item2)
  * Returns time in seconds
  **/
 guint
-pk_time_get_remaining (PkTime *time)
+pk_time_get_remaining (PkTime *self)
 {
 	guint i;
 	guint averaged = 0;
@@ -157,9 +155,9 @@ pk_time_get_remaining (PkTime *time)
 	PkTimeItem *item;
 	PkTimeItem *item_prev;
 
-	g_return_val_if_fail (PK_IS_TIME (time), 0);
+	g_return_val_if_fail (PK_IS_TIME (self), 0);
 
-	length = time->priv->array->len;
+	length = self->priv->array->len;
 	if (length < 2) {
 		egg_debug ("array too small");
 		return 0;
@@ -167,23 +165,23 @@ pk_time_get_remaining (PkTime *time)
 
 	/* get as many as we can */
 	for (i=length-1; i>0; i--) {
-		item_prev = g_ptr_array_index (time->priv->array, i-1);
-		item = g_ptr_array_index (time->priv->array, i);
+		item_prev = g_ptr_array_index (self->priv->array, i-1);
+		item = g_ptr_array_index (self->priv->array, i);
 		grad = pk_time_get_gradient (item, item_prev);
-		egg_debug ("gradient between %i/%i=%f", i-1, i, grad);
+//		egg_debug ("gradient between %i/%i=%f", i-1, i, grad);
 		if (grad < 0.00001 || grad > 100) {
 			egg_debug ("ignoring gradient: %f", grad);
 		} else {
 			grad_ave += grad;
 			averaged++;
-			if (averaged > time->priv->average_max) {
+			if (averaged > self->priv->average_max) {
 				break;
 			}
 		}
 	}
 
 	egg_debug ("averaged %i points", averaged);
-	if (averaged < time->priv->average_min) {
+	if (averaged < self->priv->average_min) {
 		egg_debug ("not enough samples for accurate time: %i", averaged);
 		return 0;
 	}
@@ -193,11 +191,11 @@ pk_time_get_remaining (PkTime *time)
 	egg_debug ("grad_ave=%f", grad_ave);
 
 	/* just for debugging */
-	elapsed = pk_time_get_elapsed (time);
+	elapsed = pk_time_get_elapsed (self);
 	egg_debug ("elapsed=%i", elapsed);
 
 	/* 100 percent to be complete */
-	item = g_ptr_array_index (time->priv->array, length - 1);
+	item = g_ptr_array_index (self->priv->array, length - 1);
 	percentage_left = 100 - item->percentage;
 	egg_debug ("percentage_left=%i", percentage_left);
 	estimated = (gfloat) percentage_left / grad_ave;
@@ -206,9 +204,9 @@ pk_time_get_remaining (PkTime *time)
 	estimated /= 1000;
 	egg_debug ("estimated=%f seconds", estimated);
 
-	if (estimated < time->priv->value_min) {
+	if (estimated < self->priv->value_min) {
 		estimated = 0;
-	} else if (estimated > time->priv->value_max) {
+	} else if (estimated > self->priv->value_max) {
 		estimated = 0;
 	}
 	return (guint) estimated;
@@ -218,22 +216,22 @@ pk_time_get_remaining (PkTime *time)
  * pk_time_add_data:
  **/
 gboolean
-pk_time_add_data (PkTime *time, guint percentage)
+pk_time_add_data (PkTime *self, guint percentage)
 {
 	PkTimeItem *item;
 	guint elapsed;
 
-	g_return_val_if_fail (PK_IS_TIME (time), FALSE);
+	g_return_val_if_fail (PK_IS_TIME (self), FALSE);
 
 	/* check we are going up */
-	if (percentage < time->priv->last_percentage) {
+	if (percentage < self->priv->last_percentage) {
 		egg_warning ("percentage cannot go down!");
 		return FALSE;
 	}
-	time->priv->last_percentage = percentage;
+	self->priv->last_percentage = percentage;
 
 	/* get runtime in ms */
-	elapsed = pk_time_get_elapsed (time);
+	elapsed = pk_time_get_elapsed (self);
 
 	egg_debug ("adding %i at %i (ms)", percentage, elapsed);
 
@@ -241,7 +239,7 @@ pk_time_add_data (PkTime *time, guint percentage)
 	item = g_new0 (PkTimeItem, 1);
 	item->time = elapsed;
 	item->percentage = percentage;
-	g_ptr_array_add (time->priv->array, item);
+	g_ptr_array_add (self->priv->array, item);
 
 	return TRUE;
 }
@@ -250,18 +248,18 @@ pk_time_add_data (PkTime *time, guint percentage)
  * pk_time_free_data:
  **/
 static gboolean
-pk_time_free_data (PkTime *time)
+pk_time_free_data (PkTime *self)
 {
 	guint i;
 	guint length;
 	gpointer mem;
 
-	g_return_val_if_fail (PK_IS_TIME (time), FALSE);
+	g_return_val_if_fail (PK_IS_TIME (self), FALSE);
 
-	length = time->priv->array->len;
+	length = self->priv->array->len;
 	for (i=0; i<length; i++) {
-		mem = g_ptr_array_index (time->priv->array, 0);
-		g_ptr_array_remove_index (time->priv->array, 0);
+		mem = g_ptr_array_index (self->priv->array, 0);
+		g_ptr_array_remove_index (self->priv->array, 0);
 		g_free (mem);
 	}
 	return TRUE;
@@ -271,18 +269,18 @@ pk_time_free_data (PkTime *time)
  * pk_time_reset:
  **/
 gboolean
-pk_time_reset (PkTime *time)
+pk_time_reset (PkTime *self)
 {
-	g_return_val_if_fail (PK_IS_TIME (time), FALSE);
+	g_return_val_if_fail (PK_IS_TIME (self), FALSE);
 
-	time->priv->time_offset = 0;
-	time->priv->last_percentage = 0;
-	time->priv->average_min = PK_TIME_AVERAGE_DEFAULT_MIN;
-	time->priv->average_max = PK_TIME_AVERAGE_DEFAULT_MAX;
-	time->priv->value_min = PK_TIME_VALUE_DEFAULT_MIN;
-	time->priv->value_max = PK_TIME_VALUE_DEFAULT_MAX;
-	g_timer_reset (time->priv->timer);
-	pk_time_free_data (time);
+	self->priv->time_offset = 0;
+	self->priv->last_percentage = 0;
+	self->priv->average_min = PK_TIME_AVERAGE_DEFAULT_MIN;
+	self->priv->average_max = PK_TIME_AVERAGE_DEFAULT_MAX;
+	self->priv->value_min = PK_TIME_VALUE_DEFAULT_MIN;
+	self->priv->value_max = PK_TIME_VALUE_DEFAULT_MAX;
+	g_timer_reset (self->priv->timer);
+	pk_time_free_data (self);
 
 	return TRUE;
 }
@@ -304,12 +302,12 @@ pk_time_class_init (PkTimeClass *klass)
  * @time: This class instance
  **/
 static void
-pk_time_init (PkTime *time)
+pk_time_init (PkTime *self)
 {
-	time->priv = PK_TIME_GET_PRIVATE (time);
-	time->priv->array = g_ptr_array_new ();
-	time->priv->timer = g_timer_new ();
-	pk_time_reset (time);
+	self->priv = PK_TIME_GET_PRIVATE (self);
+	self->priv->array = g_ptr_array_new ();
+	self->priv->timer = g_timer_new ();
+	pk_time_reset (self);
 }
 
 /**
@@ -319,15 +317,15 @@ pk_time_init (PkTime *time)
 static void
 pk_time_finalize (GObject *object)
 {
-	PkTime *time;
+	PkTime *self;
 
 	g_return_if_fail (PK_IS_TIME (object));
 
-	time = PK_TIME (object);
-	g_return_if_fail (time->priv != NULL);
-	g_ptr_array_foreach (time->priv->array, (GFunc) g_free, NULL);
-	g_ptr_array_free (time->priv->array, TRUE);
-	g_timer_destroy (time->priv->timer);
+	self = PK_TIME (object);
+	g_return_if_fail (self->priv != NULL);
+	g_ptr_array_foreach (self->priv->array, (GFunc) g_free, NULL);
+	g_ptr_array_free (self->priv->array, TRUE);
+	g_timer_destroy (self->priv->timer);
 
 	G_OBJECT_CLASS (pk_time_parent_class)->finalize (object);
 }
@@ -340,9 +338,9 @@ pk_time_finalize (GObject *object)
 PkTime *
 pk_time_new (void)
 {
-	PkTime *time;
-	time = g_object_new (PK_TYPE_TIME, NULL);
-	return PK_TIME (time);
+	PkTime *self;
+	self = g_object_new (PK_TYPE_TIME, NULL);
+	return PK_TIME (self);
 }
 
 /***************************************************************************
@@ -354,7 +352,7 @@ pk_time_new (void)
 void
 pk_time_test (EggTest *test)
 {
-	PkTime *time = NULL;
+	PkTime *self = NULL;
 	gboolean ret;
 	guint value;
 
@@ -363,12 +361,12 @@ pk_time_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "get PkTime object");
-	time = pk_time_new ();
-	egg_test_assert (test, time != NULL);
+	self = pk_time_new ();
+	egg_test_assert (test, self != NULL);
 
 	/************************************************************/
 	egg_test_title (test, "get elapsed correctly at startup");
-	value = pk_time_get_elapsed (time);
+	value = pk_time_get_elapsed (self);
 	if (value < 10)
 		egg_test_success (test, "elapsed at startup %i", value);
 	else
@@ -376,7 +374,7 @@ pk_time_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "ignore remaining correctly");
-	value = pk_time_get_remaining (time);
+	value = pk_time_get_remaining (self);
 	if (value == 0)
 		egg_test_success (test, NULL);
 	else
@@ -387,7 +385,7 @@ pk_time_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "get elapsed correctly");
-	value = pk_time_get_elapsed (time);
+	value = pk_time_get_elapsed (self);
 	if (value > 900 && value < 1100)
 		egg_test_success (test, "elapsed ~1000ms: %i", value);
 	else
@@ -395,7 +393,7 @@ pk_time_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "ignore remaining correctly when not enough entries");
-	value = pk_time_get_remaining (time);
+	value = pk_time_get_remaining (self);
 	if (value == 0)
 		egg_test_success (test, NULL);
 	else
@@ -403,43 +401,43 @@ pk_time_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "make sure we can add data");
-	ret = pk_time_add_data (time, 10);
+	ret = pk_time_add_data (self, 10);
 	egg_test_assert (test, ret);
 
 	/************************************************************/
 	egg_test_title (test, "make sure we can get remaining correctly");
 	value = 20;
 	while (value < 60) {
-		time->priv->time_offset += 2000;
-		pk_time_add_data (time, value);
+		self->priv->time_offset += 2000;
+		pk_time_add_data (self, value);
 		value += 10;
 	}
-	value = pk_time_get_remaining (time);
+	value = pk_time_get_remaining (self);
 	if (value > 9 && value < 11)
 		egg_test_success (test, NULL);
 	else
 		egg_test_failed (test, "got %i", value);
 
 	/* reset */
-	g_object_unref (time);
-	time = pk_time_new ();
+	g_object_unref (self);
+	self = pk_time_new ();
 
 	/************************************************************/
 	egg_test_title (test, "make sure we can do long times");
 	value = 10;
-	pk_time_add_data (time, 0);
+	pk_time_add_data (self, 0);
 	while (value < 60) {
-		time->priv->time_offset += 4*60*1000;
-		pk_time_add_data (time, value);
+		self->priv->time_offset += 4*60*1000;
+		pk_time_add_data (self, value);
 		value += 10;
 	}
-	value = pk_time_get_remaining (time);
-	if (value > 1199 && value < 1201)
+	value = pk_time_get_remaining (self);
+	if (value >= 1199 && value <= 1201)
 		egg_test_success (test, NULL);
 	else
 		egg_test_failed (test, "got %i", value);
 
-	g_object_unref (time);
+	g_object_unref (self);
 
 	egg_test_end (test);
 }
