@@ -47,7 +47,7 @@ typedef struct
 	GObjectClass	parent_class;
 } PkTransactionDbClass;
 
-GType		 pk_transaction_db_get_type		(void) G_GNUC_CONST;
+GType		 pk_transaction_db_get_type		(void);
 PkTransactionDb	*pk_transaction_db_new			(void);
 gboolean	 pk_transaction_db_empty		(PkTransactionDb	*tdb);
 gboolean	 pk_transaction_db_add			(PkTransactionDb	*tdb,
@@ -75,6 +75,8 @@ gboolean	 pk_transaction_db_action_time_reset	(PkTransactionDb	*tdb,
 							 PkRoleEnum		 role);
 guint		 pk_transaction_db_action_time_since	(PkTransactionDb	*tdb,
 							 PkRoleEnum		 role);
+gchar		*pk_transaction_db_generate_id		(PkTransactionDb	*tdb)
+							 G_GNUC_WARN_UNUSED_RESULT;
 
 G_END_DECLS
 
