@@ -64,6 +64,7 @@ typedef enum
 	PK_TRANSACTION_ERROR_REFUSED_BY_POLICY,
 	PK_TRANSACTION_ERROR_PACKAGE_ID_INVALID,
 	PK_TRANSACTION_ERROR_SEARCH_INVALID,
+	PK_TRANSACTION_ERROR_SEARCH_PATH_INVALID,
 	PK_TRANSACTION_ERROR_FILTER_INVALID,
 	PK_TRANSACTION_ERROR_INPUT_INVALID,
 	PK_TRANSACTION_ERROR_INVALID_STATE,
@@ -71,12 +72,13 @@ typedef enum
 	PK_TRANSACTION_ERROR_COMMIT_FAILED,
 	PK_TRANSACTION_ERROR_INVALID_PROVIDE,
 	PK_TRANSACTION_ERROR_PACK_INVALID,
+	PK_TRANSACTION_ERROR_MIME_TYPE_NOT_SUPPORTED,
 	PK_TRANSACTION_ERROR_LAST
 } PkTransactionError;
 
 GQuark		 pk_transaction_error_quark		(void);
-GType		 pk_transaction_error_get_type		(void) G_GNUC_CONST;
-GType		 pk_transaction_get_type		(void) G_GNUC_CONST;
+GType		 pk_transaction_error_get_type		(void);
+GType		 pk_transaction_get_type		(void);
 PkTransaction	*pk_transaction_new			(void);
 
 /* go go go! */
@@ -90,12 +92,16 @@ const gchar	*pk_transaction_get_tid			(PkTransaction	*transaction);
 gboolean	 pk_transaction_set_tid			(PkTransaction	*transaction,
 							 const gchar	*tid);
 
+/* set DBUS sender */
+gboolean	 pk_transaction_set_sender		(PkTransaction	*transaction,
+							 const gchar	*sender);
+
 /* dbus methods */
 void		 pk_transaction_accept_eula		(PkTransaction	*transaction,
 							 const gchar	*eula_id,
 							 DBusGMethodInvocation *context);
-gboolean	 pk_transaction_cancel			(PkTransaction	*transaction,
-							 GError		**error);
+void		 pk_transaction_cancel			(PkTransaction	*transaction,
+							 DBusGMethodInvocation *context);
 void		 pk_transaction_download_packages	(PkTransaction  *transaction,
 							 gchar		**package_ids,
 							 DBusGMethodInvocation *context);
@@ -208,9 +214,9 @@ void		 pk_transaction_search_name		(PkTransaction	*transaction,
 							 const gchar	*filter,
 							 const gchar	*search,
 							 DBusGMethodInvocation *context);
-gboolean	 pk_transaction_set_locale		(PkTransaction	*transaction,
+void		 pk_transaction_set_locale		(PkTransaction	*transaction,
 							 const gchar	*code,
-							 GError		**error);
+							 DBusGMethodInvocation *context);
 void		 pk_transaction_update_packages		(PkTransaction	*transaction,
 							 gchar		**package_ids,
 							 DBusGMethodInvocation *context);
